#========================================================================================
# package      DE_EPAGES::AvaTax::API::OrderExtractor
# state        public
#----------------------------------------------------------------------------------------
# description  Database Wrapper for BasketOrOrder Object
#========================================================================================
package DE_EPAGES::AvaTax::API::OrderExtractor;

use DE_EPAGES::AvaTax::API::DTO::BaseAddress;
use DE_EPAGES::AvaTax::API::Discount;
use DE_EPAGES::AvaTax::API::LineItem;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Shop::API::Constants qw ( TAX_MODEL_NET );
use DE_EPAGES::AvaTax::API::HashCodeBuilder qw ( CalculateAvataxChecksum );
use DE_EPAGES::AvaTax::API::Constants qw ( 
    AVATAX_REQUEST_OriginCode
    AVATAX_REQUEST_DestinationCode
    AVATAX_REQUEST_AllowedCurrencyID
);

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  constuctor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::AvaTax::API::OrderExtractor
#========================================================================================
sub new {
    my $class = shift;
    my $self = bless {
        'ErrorMessage' => undef,
    }, $class;
    return $self;
}


#========================================================================================
# function     extractDataFrom
# state        public
#----------------------------------------------------------------------------------------
# syntax       extractDataFrom( $BasketOrOrder );
#----------------------------------------------------------------------------------------
# description  Extract Data from Basket or Order and return if it was successful or not
#----------------------------------------------------------------------------------------
# input        $BasketOrOrder | BasketOrOrder | Object
# return       $WasSuccessful | Status | boolean
#========================================================================================
sub extractDataFrom {
    my $self = shift;
    my ($BasketOrOrder) = @_;

    my $WasSuccessful = $self->_getDatabaseObjects($BasketOrOrder);
    return $WasSuccessful;
}

#========================================================================================
# function     _getDatabaseObjects
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getDatabaseObjects( $BasketOrOrder );
#----------------------------------------------------------------------------------------
# description  Checks if needed attributes exists and stores it in the object
#----------------------------------------------------------------------------------------
# input        $BasketOrOrder | BasketOrOrder | Object
# return       $WasSuccessful | Status | boolean
#========================================================================================
sub _getDatabaseObjects {
    my $self = shift;
    my ($BasketOrOrder) = @_;

    $self->{'AvataxGuidMap'} = {};

    return 0 unless $self->_isBasketOrOrder($BasketOrOrder);
    $self->{'BasketOrOrder'} = $BasketOrOrder;
    return 0 unless isValid($self->getCustomerCode());

    return 0 unless $BasketOrOrder->can('container');
    return 0 unless $self->_isValidObjectOfClass( $BasketOrOrder->container(), 'DE_EPAGES::Order::API::Object::LineItemContainer' );

    my $hContainerAttributes = $self->_getLineItemContainerAttributes($BasketOrOrder->container());
    return 0 unless isValid($hContainerAttributes);
    $self->{'hContainerAttributes'} = $hContainerAttributes;

    return 0 unless $BasketOrOrder->can('getSite');
    my $Shop = $BasketOrOrder->getSite();
    return 0 unless $self->_isValidObjectOfClass( $Shop, 'DE_EPAGES::Shop::API::Object::Shop' );

    my $hTaxMethodAvaTax = $self->_getAvaTaxAttributes($Shop);
    return 0 unless isValid($hTaxMethodAvaTax);
    $self->{'AvaTax_CompanyCode'} = $hTaxMethodAvaTax->{'AvaTax_CompanyCode'};

    my $AvaTaxShopAddress = $self->_buildShopAddress($BasketOrOrder);
    return 0 unless isValid($AvaTaxShopAddress);
    $self->{'AvaTaxShopAddress'} = $AvaTaxShopAddress;

    my $AvaTaxCustomerAddress = $self->_buildCustomerAddress($BasketOrOrder);
    return 0 unless isValid($AvaTaxCustomerAddress);
    $self->{'AvaTaxCustomerAddress'} = $AvaTaxCustomerAddress;

    my $DiscountValue = $self->_calculateDiscountValue($BasketOrOrder->container());
    $self->{'DiscountValue'} = $DiscountValue;

    my $aLines = $self->_buildLines($BasketOrOrder->container());
    return 0 unless isValid($aLines);
    $self->{'aLine'} = $aLines;

    my $Customer = $BasketOrOrder->get('Customer');
    if(isValid($Customer)) {
        $self->{'AvataxTaxExemption'} = $self->_getTaxExemptionFrom( $Customer );

        my $BillingAddress = $Customer->get('BillingAddress');
        if(isValid($BillingAddress)){
            $self->{'CustomerVATID'} = $BillingAddress->get('VATID');
        }
    }

    if($BasketOrOrder->class()->existsAttribute('PaidOn')) {
        $self->{'PaidOn'} = $BasketOrOrder->get('PaidOn');
    }

    $self->{'Shop'} = $BasketOrOrder->getSite();
    $self->{'OrderAlias'} = $BasketOrOrder->alias();

    return 1;
}


#========================================================================================
# function     getContainerAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       getContainerAttributes();
#----------------------------------------------------------------------------------------
# description  Return hash with some attributes of the container needed for the Request
#----------------------------------------------------------------------------------------
# return       $self->{'hContainerAttributes'} | hash with attributes of the container | hashref
#========================================================================================
sub getContainerAttributes {
    my $self = shift;
    return $self->{'hContainerAttributes'};
}

#========================================================================================
# function     getAvaTaxCompanyCode
# state        public
#----------------------------------------------------------------------------------------
# syntax       getAvaTaxCompanyCode();
#----------------------------------------------------------------------------------------
# description  Return hash with some attributes of the TaxMethodAvatax needed for the Request
#----------------------------------------------------------------------------------------
# return       $self->{'AvaTax_CompanyCode'} | hash with attributes of the TaxMethodAvatax | hashref
#========================================================================================
sub getAvaTaxCompanyCode {
    my $self = shift;
    return $self->{'AvaTax_CompanyCode'};
}

#========================================================================================
# function     getShopAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       getShopAddress();
#----------------------------------------------------------------------------------------
# description  Return ShopAddress Object
#----------------------------------------------------------------------------------------
# return       $self->{'AvaTaxShopAddress'} | Address Object | DE_EPAGES::AvaTax::API::DTO::BaseAddress
#========================================================================================
sub getShopAddress {
    my $self = shift;
    return $self->{'AvaTaxShopAddress'};
}
#========================================================================================
# function     getCustomerAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       getCustomerAddress();
#----------------------------------------------------------------------------------------
# description  Return CustomerAddress Object
#----------------------------------------------------------------------------------------
# return       $self->{'AvaTaxCustomerAddress'} | Address Object | DE_EPAGES::AvaTax::API::DTO::BaseAddress
#========================================================================================
sub getCustomerAddress {
    my $self = shift;
    return $self->{'AvaTaxCustomerAddress'};
}

#========================================================================================
# function     getBasketOrOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       getBasketOrOrder();
#----------------------------------------------------------------------------------------
# description  Return Basket or CustomerOrder Object
#----------------------------------------------------------------------------------------
# return       $self->{'BasketOrOrder'} | Basket or CustomerOrder Object | Object
#========================================================================================
sub getBasketOrOrder {
    my $self = shift;
    return $self->{'BasketOrOrder'};
}

#========================================================================================
# function     getLineItemContainer
# state        public
#----------------------------------------------------------------------------------------
# syntax       getLineItemContainer();
#----------------------------------------------------------------------------------------
# description  Return LineItemContainer associated to the OrderExtractor
#----------------------------------------------------------------------------------------
# return       LineItemContainer | DE_EPAGES::Order::API::Object::LineItemContainer
#========================================================================================
sub getLineItemContainer {
    my $self = shift;
    return $self->{'BasketOrOrder'}->container();
}


#========================================================================================
# function     getCustomerVATID
# state        public
#----------------------------------------------------------------------------------------
# syntax       getCustomerVATID();
#----------------------------------------------------------------------------------------
# description  Return Customer VATID
#----------------------------------------------------------------------------------------
# return       $self->{'CustomerVATID'} | Customer VATID | String
#========================================================================================
sub getCustomerVATID {
    my $self = shift;
    return $self->{'CustomerVATID'};
}

#========================================================================================
# function     getErrorMessage
# state        public
#----------------------------------------------------------------------------------------
# syntax       getErrorMessage();
#----------------------------------------------------------------------------------------
# description  Return ErrorMessage if case of fail
#----------------------------------------------------------------------------------------
# return       $self->{'ErrorMessage'} | ErrorMessage | hashref
#========================================================================================
sub getErrorMessage {
    my $self = shift;
    return $self->{'ErrorMessage'};
}

#========================================================================================
# function     getDiscountValue
# state        public
#----------------------------------------------------------------------------------------
# syntax       getDiscountValue();
#----------------------------------------------------------------------------------------
# description  Return DiscountValue
#----------------------------------------------------------------------------------------
# return       $self->{'DiscountValue'} | DiscountValue | float
#========================================================================================
sub getDiscountValue {
    my $self = shift;
    return $self->{'DiscountValue'};
}

#========================================================================================
# function     getLines
# state        public
#----------------------------------------------------------------------------------------
# syntax       getLines();
#----------------------------------------------------------------------------------------
# description  Return array with Avatax Line
#----------------------------------------------------------------------------------------
# return       $self->{'aLine'} | Array with avatax line | array
#========================================================================================
sub getLines {
    my $self = shift;
    return $self->{'aLine'};
}

#========================================================================================
# function     getCustomerCode
# state        public
#----------------------------------------------------------------------------------------
# syntax       getCustomerCode();
#----------------------------------------------------------------------------------------
# description  Return CustomerCode
#----------------------------------------------------------------------------------------
# return       CustomerCode | string
#========================================================================================
sub getCustomerCode {
    my $self = shift;
    my $CustomerCode;

    my $Customer = $self->{'BasketOrOrder'}->get('Customer');
    if( defined $Customer ){
        $CustomerCode = $Customer->alias();
    }else{
        $CustomerCode = 'TaxSvcTest';
    }

    return $CustomerCode;
}

#========================================================================================
# function     getAvataxTaxExemption
# state        public
#----------------------------------------------------------------------------------------
# syntax       getAvataxTaxExemption();
#----------------------------------------------------------------------------------------
# description  Returns the tax exemption of the customer of the order
#----------------------------------------------------------------------------------------
# return       AvataxTaxExemption | DE_EPAGES::Object::API::Object::PreDefAttribute
#========================================================================================
sub getAvataxTaxExemption {
    my $self = shift;
    return $self->{'AvataxTaxExemption'};
}

#========================================================================================
# function     getShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       getShop();
#----------------------------------------------------------------------------------------
# description  Returns the shop object
#----------------------------------------------------------------------------------------
# return       Shop object | DE_EPAGES::Shop::API::Object::Shop
#========================================================================================
sub getShop {
    my $self = shift;
    return $self->{'Shop'};
}

#========================================================================================
# function     isPaid
# state        public
#----------------------------------------------------------------------------------------
# syntax       isPaid();
#----------------------------------------------------------------------------------------
# description  Returns paid status
#----------------------------------------------------------------------------------------
# return       PaidOn | boolean
#========================================================================================
sub isPaid {
    my $self = shift;
    return isValid($self->{'PaidOn'});
}

#========================================================================================
# function     getOrderAlias
# state        public
#----------------------------------------------------------------------------------------
# syntax       getOrderAlias();
#----------------------------------------------------------------------------------------
# description  Returns the alias of the order
#----------------------------------------------------------------------------------------
# return       OrderAlias | string
#========================================================================================
sub getOrderAlias {
    my $self = shift;
    $self->{'OrderAlias'};
}

#========================================================================================
# function     _isValidObjectOfClass
# state        private
#----------------------------------------------------------------------------------------
# syntax       _isValidObjectOfClass( $Object, $ExpectedClass );
#----------------------------------------------------------------------------------------
# description  Tests if the given object is in the given package.
#----------------------------------------------------------------------------------------
# input        $Object | object of arbitrary class | object
# input        $ExpectedClass | string with package name | String
# return       $IsObjectOfClass | Is Object Of Class | boolean
#========================================================================================
sub _isValidObjectOfClass {
    my $self = shift;
    my ( $Object, $ExpectedClass ) = @_;
    my $IsObjectOfClass = 1;
    eval { TestObject( 'OrderExtractor: isValidObjectOfClass', $Object, $ExpectedClass ); };
    if ( ExistsError() ) {
        my $Error       = GetError();
        my $ErrorString = $Error->code();
        if (   $ErrorString =~ /WRONG_CLASS/
            || $ErrorString =~ /PARAMETER_MISSING/
            || $ErrorString =~ /PARAMETER_NO_OBJECT/ )
        {
            $IsObjectOfClass = 0;
            my $ErrorMessage = {'Code' => $ErrorString, 'Info' => { 'Object' => $Object }};
            $self->_setErrorMessage($ErrorMessage);
        }
        else {
            LogError($Error); #TODO: create unit test for this line. This condition will never happen. The TestObject only throw 3 different erros
        }
    }
    return $IsObjectOfClass;
}

#========================================================================================
# function     _setErrorMessage
# state        private
#----------------------------------------------------------------------------------------
# syntax       _setErrorMessage( $ErrorMessage );
#----------------------------------------------------------------------------------------
# description  Set ErrorMessage
#----------------------------------------------------------------------------------------
# input        ErrorMessage | hashref
#========================================================================================
sub _setErrorMessage {
    my $self = shift;
    my ($ErrorMessage) = @_;
    $self->{'ErrorMessage'} = $ErrorMessage;
    return;
}


#========================================================================================
# function     _isBasketOrOrder
# state        private
#----------------------------------------------------------------------------------------
# syntax       _isBasketOrOrder( $BasketOrOrder );
#----------------------------------------------------------------------------------------
# description  Check if the input is basket or order and return true in case it is
#----------------------------------------------------------------------------------------
# input        $BasketOrOrder | Basket or CustomerOrder Object | Object
# return       flag if the object is basket or order | boolean
#========================================================================================
sub _isBasketOrOrder {
    my $self = shift;
    my ($BasketOrOrder) = @_;

    my $IsBasketOrOrder = 0;
    return $IsBasketOrOrder unless isValid($BasketOrOrder);

    if (   $BasketOrOrder->instanceOf('CustomerOrder')
        || $BasketOrOrder->instanceOf('Basket') ){
        $IsBasketOrOrder = 1;
    }

    return $IsBasketOrOrder;
}

#========================================================================================
# function     _getAvaTaxAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getAvaTaxAttributes( $Shop );
#----------------------------------------------------------------------------------------
# description  Return Avatax attributes need to create the request
#----------------------------------------------------------------------------------------
# input        $Shop | Shop Object | Object
# return       return hashref with the attributes of AvaTax needed for the request | hashref
#========================================================================================
sub _getAvaTaxAttributes {
    my $self = shift;
    my ($Shop) = @_;

    my $hErrorMessage = undef;
    my $TaxMethod = $Shop->get('TaxMethod');
    if ( not isValid($TaxMethod) ) {
        $hErrorMessage = {'Code' => 'NoTaxMethod', 'Info' => { 'ShopAlias' => $Shop->alias(), }};
        $self->_setErrorMessage($hErrorMessage);
        return;
    }
    my $isInstanceOfTaxMethodAvaTax = $TaxMethod->instanceOf('TaxMethodAvaTax');
    if(not $isInstanceOfTaxMethodAvaTax){
        $hErrorMessage = {'Code' => 'IsNotAvaTax', 'Info' => { 'ShopAlias' => $Shop->alias(), }};
        $self->_setErrorMessage($hErrorMessage);
        return;
    }

    #TODO: replace (MissingLicenseKey, MissingCompanyCode, MissingAccountNumber) for MissingCredentials also check the TLE
    my $hInfo = $TaxMethod->get( [qw( AvaTax_AccountNumber AvaTax_LicenseKey AvaTax_CompanyCode )] );
    if(not (isValid($hInfo->{'AvaTax_AccountNumber'})
        && isValid($hInfo->{'AvaTax_LicenseKey'})
        && isValid($hInfo->{'AvaTax_CompanyCode'}))){
        $hErrorMessage = {'Code' => 'MissingCredentials', 'Info' => $hInfo };
        $self->_setErrorMessage($hErrorMessage);
        return;
    }
    return $hInfo;
}

#========================================================================================
# function     _buildAddressHash
# state        private
#----------------------------------------------------------------------------------------
# syntax       _buildAddressHash( $Address );
#----------------------------------------------------------------------------------------
# description  Return Address attributes need to create the request
#----------------------------------------------------------------------------------------
# input        $Address | Address Object | Object
# return       return hashref with the attributes of Address needed for the request | hashref
#========================================================================================
sub _buildAddressHash {
    my $self = shift;
    my ($Address) = @_;
    my $hErrorMessage = undef;

    my $hInfo = $Address->get([qw( Company FullName Street Street2 Zipcode City State Country )]);
    #TODO: replace (MissingShippingStreet, MissingShippingZipCode, MissingShippingCity, 
    #MissingShippingCountry, ShippingCountryMismatch and ShopAddress) for MissingAddressInfo also check the TLE
    if(not (isValid($hInfo->{'Street'})
        && isValid($hInfo->{'Zipcode'})
        && isValid($hInfo->{'City'})
        && isValid($hInfo->{'Country'}))) {
        $hErrorMessage = {'Code' => 'MissingAddressInfo', 'Info' => $hInfo };
        $self->_setErrorMessage($hErrorMessage);
        return;
    }

    return $hInfo;
}

#========================================================================================
# function     _getLineItemContainerAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getLineItemContainerAttributes( $LineItemContainer );
#----------------------------------------------------------------------------------------
# description  Return LineItemContainer attributes need to create the request
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | LineItemContainer Object | Object
# return       return hashref with the attributes of LineItemContainer needed for the request | hashref
#========================================================================================
sub _getLineItemContainerAttributes {
    my $self = shift;
    my ($LineItemContainer) = @_;

    my $hErrorMessage = undef;
    my $hInfo = $LineItemContainer->get([qw( CurrencyID TaxModel LanguageID )]);

    if($hInfo->{'CurrencyID'} ne AVATAX_REQUEST_AllowedCurrencyID ) {
        $hErrorMessage = {'Code' => 'CurrencyIDMismatch', 'Info' => { 'CurrencyID' => $hInfo->{'CurrencyID'}}};
    }
    if($hInfo->{'TaxModel'} != TAX_MODEL_NET){
        $hErrorMessage = {'Code' => 'TaxModelMismatch', 'Info' => { }};
    }
    if(isValid($hErrorMessage)){
        $self->_setErrorMessage($hErrorMessage);
        return;
    }

    return $hInfo;
}

#========================================================================================
# function     _buildShopAddress
# state        private
#----------------------------------------------------------------------------------------
# syntax       _buildShopAddress( $BasketOrOrder );
#----------------------------------------------------------------------------------------
# description  Return BaseAddress object
#----------------------------------------------------------------------------------------
# input        $BasketOrOrder | Basket or CustomerOrder Object | Object
# return       return the AddressObject | DE_EPAGES::AvaTax::API::DTO::BaseAddress
#========================================================================================
sub _buildShopAddress {
    my $self = shift;
    my ($BasketOrOrder) = @_;

    my $ShopAddress = $BasketOrOrder->get('ShopAddress');
    return unless $self->_isValidObjectOfClass( $ShopAddress, 'DE_EPAGES::Address::API::Object::Address' );

    my $AvaTaxShopAddress = $self->_buildAddressObject($ShopAddress, AVATAX_REQUEST_OriginCode);
    return unless isValid($AvaTaxShopAddress);

    return $AvaTaxShopAddress;
}

#========================================================================================
# function     _buildCustomerAddress
# state        private
#----------------------------------------------------------------------------------------
# syntax       _buildCustomerAddress( $BasketOrOrder );
#----------------------------------------------------------------------------------------
# description  Return Address object with the customer information
#----------------------------------------------------------------------------------------
# input        $BasketOrOrder | Basket or CustomerOrder Object | Object
# return       return the AddressObject | DE_EPAGES::AvaTax::API::DTO::BaseAddress
#========================================================================================
sub _buildCustomerAddress {
    my $self = shift;
    my ($BasketOrOrder) = @_;

    my $CustomerAddress = $BasketOrOrder->get('ShippingAddress');
    if(not isValid($CustomerAddress)){
        $CustomerAddress = $BasketOrOrder->get('BillingAddress');
    }
    return unless $self->_isValidObjectOfClass( $CustomerAddress, 'DE_EPAGES::Address::API::Object::Address' );

    my $AvaTaxCustomerAddress = $self->_buildAddressObject($CustomerAddress, AVATAX_REQUEST_DestinationCode);
    return unless isValid($AvaTaxCustomerAddress);

    return $AvaTaxCustomerAddress;
}

#========================================================================================
# function     _buildAddressObject
# state        private
#----------------------------------------------------------------------------------------
# syntax       _buildAddressObject( $Address, $AddressCode );
#----------------------------------------------------------------------------------------
# description  Return Address object
#----------------------------------------------------------------------------------------
# input        $Address | Epages Address Object | DE_EPAGES::Address::API::Object::Address
# input        $AddressCode | AddressCode (AVATAX_REQUEST_OriginCode or AVATAX_REQUEST_DestinationCode) | string
# return       return the Avatax DTO AddressObject | DE_EPAGES::AvaTax::API::DTO::BaseAddress
#========================================================================================
sub _buildAddressObject {
    my $self = shift;
    my ($Address, $AddressCode) = @_;

    my $hAddress = $self->_buildAddressHash($Address);
    return unless isValid($hAddress);

    my $BaseAddress = DE_EPAGES::AvaTax::API::DTO::BaseAddress->new();
    $BaseAddress->AddressCode($AddressCode);
    $BaseAddress->Line1($hAddress->{'Company'}//$hAddress->{'FullName'});
    $BaseAddress->Line2($hAddress->{'Street'});
    $BaseAddress->Line3($hAddress->{'Street2'});
    $BaseAddress->City($hAddress->{'City'});
    $BaseAddress->PostalCode($hAddress->{'Zipcode'});
    $BaseAddress->Region($hAddress->{'State'});
    $BaseAddress->Country($hAddress->{'Country'}->{'Code2'});

    return $BaseAddress;
}

#========================================================================================
# function     _calculateDiscountValue
# state        private
#----------------------------------------------------------------------------------------
# syntax       _calculateDiscountValue( $LineItemContainer );
#----------------------------------------------------------------------------------------
# description  Calculate the discount value for the given LineItemContainer
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | LineItemContainer | DE_EPAGES::Order::API::Object::LineItemContainer
# return       return the discount value | float
#========================================================================================
sub _calculateDiscountValue {
    my $self = shift;
    my ($LineItemContainer) = @_;

    my $DiscountObject = $self->_createDiscountObject();
    my $DiscountValue = $DiscountObject->getDiscountValue($LineItemContainer);

    return $DiscountValue;
}


#========================================================================================
# function     _createDiscountObject
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createDiscountObject();
#----------------------------------------------------------------------------------------
# description  Wrapper for injection
#----------------------------------------------------------------------------------------
# return       return Discount Object | DE_EPAGES::AvaTax::API::Discount
#========================================================================================
sub _createDiscountObject {
    my $self = shift;
    return DE_EPAGES::AvaTax::API::Discount->new();
}


#========================================================================================
# function     _buildLines
# state        private
#----------------------------------------------------------------------------------------
# syntax       _buildLines( $LineItemContainer );
#----------------------------------------------------------------------------------------
# description  Create a array with Avatax Line with information from the LineItemContainer
#----------------------------------------------------------------------------------------
# input        $LineItemContainer | LineItemContainer | DE_EPAGES::Order::API::Object::LineItemContainer
# return       return array Line | array
#========================================================================================
sub _buildLines {
    my $self = shift;
    my ($LineItemContainer) = @_;

    my $LineItemObject = $self->_createLineItemObject();
    my $aLine = $LineItemObject->getLines($LineItemContainer,$self->getDiscountValue());

    return $aLine;
}

#========================================================================================
# function     _createLineItemObject
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createLineItemObject();
#----------------------------------------------------------------------------------------
# description  Wrapper for injection
#----------------------------------------------------------------------------------------
# return       return Discount Object | DE_EPAGES::AvaTax::API::LineItem
#========================================================================================
sub _createLineItemObject {
    my $self = shift;
    my $AvataxGuidMap = $self->{'AvataxGuidMap'};
    return DE_EPAGES::AvaTax::API::LineItem->new($AvataxGuidMap);
}

#========================================================================================
# function     getAvataxGuidMap
# state        public
#----------------------------------------------------------------------------------------
# syntax       getAvataxGuidMap();
#----------------------------------------------------------------------------------------
# description  Getter for AvataxGuidMap
#----------------------------------------------------------------------------------------
# return       return AvataxGuidMap | DE_EPAGES::AvaTax::API::AvataxGuidMap
#========================================================================================
sub getAvataxGuidMap {
    my $self = shift;
    return $self->{'AvataxGuidMap'};
}

#========================================================================================
# function     hasChanged
# state        public
#----------------------------------------------------------------------------------------
# syntax       hasChanged();
#----------------------------------------------------------------------------------------
# description  Calculates a hash over the underlying basket or order and compares
#               it with attribute AvaTaxHash in the object. 
#----------------------------------------------------------------------------------------
# return       return change flag | Boolean
#========================================================================================
sub hasChanged {
    my $self = shift;
    my $AvataxNewCheckSum = $self->_calculateChecksum();
    my $AvataxOldCheckSum = $self->getLineItemContainer()->get('AvaTax_Checksum') // '';
    if ($AvataxNewCheckSum eq $AvataxOldCheckSum) {
        return 0;
    } else {
        return 1;
    }
}

#========================================================================================
# function     updateHash
# state        public
#----------------------------------------------------------------------------------------
# syntax       updateHash();
#----------------------------------------------------------------------------------------
# description  Calculates a hash over the underlying basket or order stores it in the
#               attribute AvaTaxHash.
#========================================================================================
sub updateHash {
    my $self = shift;
    my $AvataxCheckSum = $self->_calculateChecksum();
    my $LineItemContainer = $self->getLineItemContainer();
    $LineItemContainer->set({'AvaTax_Checksum' => $AvataxCheckSum});
    return;
}

#========================================================================================
# function     _calculateChecksum
# state        private
#----------------------------------------------------------------------------------------
# syntax       _calculateChecksum();
#----------------------------------------------------------------------------------------
# description  Calculates a check sum
#----------------------------------------------------------------------------------------
# return       return AvaTax hash | String
#========================================================================================
sub _calculateChecksum {
    my $self = shift;

    my $AvataxHash = $self->_CalculateAvataxChecksum ( $self );
    return $AvataxHash;
}



#========================================================================================
# function     _getTaxExemptionFrom
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getTaxExemptionFrom( $Customer );
#----------------------------------------------------------------------------------------
# description  Check if the AvataxTaxExemption is there and return it
#----------------------------------------------------------------------------------------
# input        $Customer | Customer Object | DE_EPAGES::Customer::API::Object::Customer
#----------------------------------------------------------------------------------------
# return       return AvataxTaxExemption Object | Object
#========================================================================================
sub _getTaxExemptionFrom {
    my $self = shift;
    my ($Customer) = @_;

    if(isValid($Customer) && $Customer->class()->existsAttribute('AvataxTaxExemption') ) {
        return $Customer->get('AvataxTaxExemption');
    }
    return;
}


sub _CalculateAvataxChecksum {
    my $self = shift;
    my ($OrderExtractor) = @_;
    return CalculateAvataxChecksum($OrderExtractor);
}


1;
