#========================================================================================
# package      DE_EPAGES::AvaTax::Hooks::LineItemContainer
# state        public
#----------------------------------------------------------------------------------------
# description  
#----------------------------------------------------------------------------------------
#========================================================================================
package DE_EPAGES::AvaTax::Hooks::LineItemContainer;

use strict;
use feature 'switch';

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError Error );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Core::API::PerlTools qw ( IsA );
use DE_EPAGES::AvaTax::API::AvaTax qw ( UpdateTax UseAvaTax NotifyShopOwner );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::WebInterface::API::ApplicationServer qw (GetAppServer);

sub ErrorDefinition {
    return {
            'TaxMethodNotUsableAtStorefront' => 'TaxMethod #TaxMethod not usable at storefront. Shop:#Shop, Store:#Store',
    };
}

#========================================================================================
# function     OnAfterRecalculateLineItems
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterRecalculateLineItems( $Input1 );
#----------------------------------------------------------------------------------------
# description  Does tax calculation, based on AfterRecalculateLineItems hook.
#----------------------------------------------------------------------------------------
# input        $hInfo  - used keys:
# <ul>          <li>Object -  LineItemContainer</li>
#               <li>Values - calculated lineitems as hash</li>
# </ul>         | LineItemContainer | hash.ref
#========================================================================================
sub OnAfterRecalculateLineItems {
    my ($hInfo) = @_;
    my $LineItemContainer = $hInfo->{'Object'};
    TestObject('LineItemContainer',$LineItemContainer,'DE_EPAGES::Order::API::Object::LineItemContainer');

    my $Shop = $LineItemContainer->getSite;
    my $BasketOrder = $LineItemContainer->parent;

    return unless (
        IsA($BasketOrder, 'DE_EPAGES::Order::API::Object::Basket') or
        IsA($BasketOrder, 'DE_EPAGES::Order::API::Object::CustomerOrder'));

    eval {
        my $ShippingAddress = $BasketOrder->get('ShippingAddress') // $BasketOrder->get('BillingAddress');
        return unless (defined $ShippingAddress);
        TestObject('ShippingAddress',$ShippingAddress,'DE_EPAGES::Address::API::Object::Address');
        my $ShippingCountry = $ShippingAddress->get('Country') or return;
        my $UseAvaTax = UseAvaTax($LineItemContainer->getSite, {'CountryID' => $ShippingCountry->{'CountryID'}});
        GetLog()->debug('UseAvaTax: '.$UseAvaTax);
        return if ( $UseAvaTax <= 0 );
        my $TaxMethod = $Shop->get('TaxMethod');

        Error('TaxMethodNotUsableAtStorefront',{
            'TaxMethod' => $TaxMethod->alias,
            'Shop'      => $Shop->alias,
            'Store'     => GetCurrentDBHandle->cacheKey,
        }) unless ($TaxMethod->usableAtStorefront());

        UpdateTax($LineItemContainer);
    };
    if (ExistsError()) {
        my $Error = GetError();
        given ($Error->code) {
            # service error or not usable
            when ([qw(
                AvaTaxRequestFailure
                TaxMethodNotUsableAtStorefront
            )]) {
                NotifyShopOwner($Shop, $Error->code);
            }
            when ([qw( AvaTaxCalculationError )]) {
                my $AppServer = GetAppServer();
                if ($AppServer) {
                    my $Servlet = $AppServer->currentServlet();
                    my $Form = $Servlet->form;
                    my $UseShippingAddress = defined( $BasketOrder->get('ShippingAddress') );
                    foreach my $Field (qw( Zipcode City State )){
                        $Form->addFormError({
                            'Name'   => ($UseShippingAddress?'Shipping':'Billing').'Address.'.$Field,
                            'Reason' => 'AvaTaxError',
                        });
                    }
                    $Form->executeFormError;
                }
            }
            # configuration errro
            when ([qw( 
                MissingAccountNumber
                MissingLicenseKey
                MissingCompanyCode
                MissingShopAddress
                MissingShopStreet
                MissingShopZipCode
                MissingShopCity
                MissingShopCountry
                ShopCountryMismatch
                CurrencyIDMismatch
                TaxModelMismatch
            )]) {
                NotifyShopOwner($Shop, $Error->code);
                return;
            }
            # foreign country: accepted
            # Others: Reset to internal tax calculation
            # or other calculation method:  accepted.
            # or avatax service is currently not reachable
            when ([qw(
                ShippingCountryMismatch
                NoTaxMethod
                IsNotAvaTax
                SOAPFault
            )]) {
                return;
            }
            # TODO : what to do on incomplete shipping address?
            when ([qw(
                MissingShippingAddress
                MissingShippingStreet
                MissingShippingZipCode
                MissingShippingCity
                MissingShippingCountry
            )]) { }
        }
        Error($Error);
    }
}

1;