#==============================================================================
# package      DE_EPAGES::Catalog::API::WebService::CatalogService
# base         DE_EPAGES::Content::API::WebService::BaseContentService
# state        private
# description  Implements the Catalog WebService.
#==============================================================================
package DE_EPAGES::Catalog::API::WebService::CatalogService;
use base DE_EPAGES::Content::API::WebService::BaseContentService;

use DE_EPAGES::Core::API::Log        qw (LogDebug);
use DE_EPAGES::Core::API::Error      qw (Error ResetError ExistsError GetError);
use DE_EPAGES::Core::API::PerlTools  qw (ElementOf);
use DE_EPAGES::Object::API::Factory  qw (LoadClassByAlias Transaction);
use DE_EPAGES::Trigger::API::BatchProcessor qw ( RunBatchProcess );

use Readonly;
use strict;

Readonly::Scalar my $DEFAULT_LASTPOSITION => 999999;



#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'OBJECT_ALREADY_EXISTS'       => 'An object with alias #Alias does already exist for parent #Parent.',
        'PARENT_DOES_NOT_EXIST'       => 'Parent Catalog #Parent does not exist.',
        'ROOT_CATALOG_MODIFY'         => 'Root catalog must not be updated or deleted.'
    }
}
#ERRORDEFINITION END

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Service = DE_EPAGES::Catalog::API::WebService::CatalogService->new( %Options );
#----------------------------------------------------------------------------------------
# description  Creates a new web service object as a blessed hash reference and adds
#               the values from the %Options hash.
#----------------------------------------------------------------------------------------
# input        %Options | user-defined values | hash
# return       $Service | web service object | object
#========================================================================================
sub new {
    my $self = shift;
    my @Options = @_;
    $self->SUPER::new((
        'ServiceUrn'      => 'urn://epages.de/WebService/CatalogService/2005/01',
        'ServiceTypesUrn' => 'urn://epages.de/WebService/CatalogTypes/2005/01',
        @Options
    ));
}

#========================================================================================
# function     getInfo
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->getInfo( $aCatalogs, $aAttributes, $aLanguages )
#----------------------------------------------------------------------------------------
# description  WebService-Method: getInfo
#               Returns information about a list of catalogs.
#----------------------------------------------------------------------------------------
# input        $aCatalogs   | list of catalog pathes    | ref.array.string
# input        $aAttributes | list of attribute aliases | ref.array.string
# input        $aLanguages  | list of language codes    | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            info of all given catalogs | SOAP::Data
#========================================================================================
sub getInfo {
    my $self = shift;

    # expects a list of catalog pathes
    my ($aCatalogs, $aAttributes, $aLanguageCodes) = @_;
    LogDebug('aCatalogs=', $aCatalogs);
    LogDebug('aAttributes=', $aAttributes);
    LogDebug('aLanguageCodes=', $aLanguageCodes);

    my $aLanguages = $self->getLanguageIDs($aLanguageCodes);

    my @soapCatalogs = ();
    foreach my $path (@$aCatalogs) {
        next unless(defined($path));
        my $soapCatalog;

        eval {
            # get object
            my $catalog = $self->loadShopObject($path, 'Category');
            $soapCatalog = $self->buildSoapCatalog( $catalog, $aAttributes, $aLanguages );
        };
        if (ExistsError()) {
            my $Error = GetError();
            $soapCatalog = $self->soapData('Catalog', {
                'Path'  => $self->soapString($path ),
                'Error' => $self->soapError($Error)
            }, 'TGetInfo_Return');
            ResetError();
        }
        push(@soapCatalogs, $soapCatalog);

    }
    my $result = $self->soapData('Catalogs', $self->soapList(\@soapCatalogs, 'TGetInfo_Return') );
    $self->mapType([qw (TGetInfo_Return)], $self->getServiceTypesUrn);
    $self->mapType([qw (TLocalizedValue)], $self->getEpagesTypesUrn);

    return $result;
}

#========================================================================================
# function     exists
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->exists( $aCatalogs );
#----------------------------------------------------------------------------------------
# description  WebService-Method: exists
#               Returns exists status of a list of catalogs.
#               Even if the function states that the catalog does not exist another object
#               with this path may exist (Article, Link, FreeContent).
#----------------------------------------------------------------------------------------
# input        $aCatalogs | list of catalog pathes | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            exist status of all given catalogs | SOAP::Data
#========================================================================================
sub exists {
    my $self = shift;

    # expects a list of catalog paths
    my ($aCatalogs) = @_;
    LogDebug('aCatalogs=', $aCatalogs);

    my @soapCatalogs = ();
    foreach my $path (@$aCatalogs) {
        next unless(defined($path));

        my $soapCatalog;
        my $hOutCatalog = {
                'Path'   => $self->soapString($path),
                'exists' => $self->soapBool( $self->existsShopObject($path, 'Category') )
        };

        # build return object
        $soapCatalog = $self->soapData('Catalog', $hOutCatalog, 'TExists_Return');
        push(@soapCatalogs, $soapCatalog);
    }
    my $result = $self->soapData('Catalogs', $self->soapList(\@soapCatalogs, 'TExists_Return') );
    $self->mapType([qw (TExists_Return)], $self->getServiceTypesUrn);

    return $result;
}

#========================================================================================
# function     delete
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->delete( $aCatalogs )
#----------------------------------------------------------------------------------------
# description  WebService-Method: delete
#               Deletes a list of catalogs.
#----------------------------------------------------------------------------------------
# input        $aCatalogs | list of catalog pathes | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            delete status of all given catalogs | SOAP::Data
#========================================================================================
sub delete {
    my $self = shift;

    # expects a list of catalog pathes
    my ($aCatalogs) = @_;
    LogDebug('aCatalogs=', $aCatalogs);

    my $Shop = $self->site;

    my @soapCatalogs = ();
    my @parentCatalogs = ();

    RunBatchProcess('DeleteCategories', sub {

        foreach my $path (@$aCatalogs) {
            next unless(defined($path));

            my $soapCatalog;

            eval {
                # get object
                my $catalog = $self->loadShopObject($path, 'Category');


                # root catalog must not be deleted
                Error('ROOT_CATALOG_MODIFY') if ($catalog->id == $Shop->get('Categories')->id);

                # get parent catalog for renumbering
                my $parentCatalog = $catalog->parent;

                # delete
                $catalog->delete;

                unless ($catalog->exists() || ElementOf($parentCatalog, @parentCatalogs)) {
                    push(@parentCatalogs, $parentCatalog);
                }

                my $hOutCatalog = {
                    'Path'    => $self->soapString( $path ),
                    'deleted' => $self->soapBool( !$catalog->exists() )
                };

                # build return object
                $soapCatalog = $self->soapData('Catalog', $hOutCatalog, 'TDelete_Return');
            };
            if (ExistsError()) {
                my $Error = GetError();
                $soapCatalog = $self->soapData('Catalog', {
                    'Path'  => $self->soapString( $path ),
                    'Error' => $self->soapError($Error)
                }, 'TDelete_Return');
                ResetError();
            }
            push(@soapCatalogs, $soapCatalog);

        }

        #--- renumber children foreach catalog who lost a child
        foreach my $catalog(@parentCatalogs) {
            $catalog->renumberChildren() if ($catalog->exists());
        }

    });

    my $result = $self->soapData('Catalogs', $self->soapList(\@soapCatalogs, 'TDelete_Return') );
    $self->mapType([qw (TDelete_Return)], $self->getServiceTypesUrn);

    return $result;
}

#========================================================================================
# function     update
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->update( $aCatalogs )
#----------------------------------------------------------------------------------------
# description  WebService-Method: update
#               Updates a list of catalogs.
#----------------------------------------------------------------------------------------
# input        $aCatalogs | list of catalog info (catalog path and
#                              data to update) | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            update status of all given catalogs | SOAP::Data
#========================================================================================
sub update {
    my $self = shift;

    # expects a list of catalogs
    my ($aCatalogs) = @_;
    LogDebug('aCatalogs=', $aCatalogs);

    my $Shop = $self->site;

    my @soapCatalogs = ();
    my @parentCatalogs = ();
    my $result;

    RunBatchProcess('UpdateCategories', sub {

        foreach my $hInCatalog (@$aCatalogs) {
            my $soapCatalog;

            eval {
                $self->checkMandatoryFields($hInCatalog, ['Path']);
                my $path = $hInCatalog->{'Path'};

                # get object
                my $catalog = $self->loadShopObject($path, 'Category');

                # root catalog must not be deleted
                Error('ROOT_CATALOG_MODIFY') if ($catalog->id == $Shop->get('Categories')->id);

                Transaction( sub {
                    # set catalog parameters
                    $self->setParameters($catalog, $hInCatalog);

                    # set Attributes
                    $self->setAttributeValues($catalog, $hInCatalog->{'Attributes'});
                });

                my $hOutCatalog = {
                    'Path'       => $self->soapString( $hInCatalog->{'Path'} ),
                    'updated'    => $self->soapBool( 1 )
                };

                #--- renumbering only necessary if position has been updated
                if (exists($hInCatalog->{'Position'}) || exists($hInCatalog->{'MoveToParentCatalog'})) {
                    my $Parent = $catalog->parent();
                    unless (ElementOf($Parent, @parentCatalogs)) {
                       push(@parentCatalogs, $Parent);
                   }
                }

                # build return object
                $soapCatalog = $self->soapData('Catalog', $hOutCatalog, 'TUpdate_Return');
            };
            if (ExistsError()) {
                my $Error = GetError();
                $soapCatalog = $self->soapData('Catalog', {
                    'Path'  => $self->soapString( $hInCatalog->{'Path'} ),
                    'Error' => $self->soapError($Error)
                }, 'TUpdate_Return');
                ResetError();
            }
            push(@soapCatalogs, $soapCatalog);

        }

        #--- renumber children
        foreach my $catalog(@parentCatalogs) {
            $catalog->renumberChildren();
        }

    });

    $result = $self->soapData('Catalogs', $self->soapList(\@soapCatalogs, 'TUpdate_Return') );
    $self->mapType([qw (TUpdate_Return)], $self->getServiceTypesUrn);

    return $result;
}

#========================================================================================
# function     create
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->create( $aCatalogs )
#----------------------------------------------------------------------------------------
# description  WebService-Method: create
#               Creates a list of catalogs.
#----------------------------------------------------------------------------------------
# input        $aCatalogs | list of catalog info (data to update) | ref.array.string
# return       $SoapData  | a SOAP::Data structure with the
#                            create status of all given catalogs | SOAP::Data
#========================================================================================
sub create {
    my $self = shift;

    # expects a list of catalogs
    my ($aCatalogs) = @_;
    LogDebug('aCatalogs=', $aCatalogs);

    my @soapCatalogs = ();
    my @parentCatalogs = ();
    my $lastPosition = $DEFAULT_LASTPOSITION;

    RunBatchProcess('CreateCategories', sub {

        foreach my $hInCatalog (@$aCatalogs) {
            my $soapCatalog;
    
            eval {
                $self->checkMandatoryFields($hInCatalog, ['Alias', 'ParentCatalog']);
                my $alias = $hInCatalog->{'Alias'};
                my $parentCatalog = $hInCatalog->{'ParentCatalog'};
    
                # load parent catalog
                my $Parent = $self->loadShopObject($parentCatalog, 'Category');
    
                # check if object with this alias does exist already
                Error('OBJECT_ALREADY_EXISTS', {'Alias' => $alias, 'Parent' => $parentCatalog})
                    if ($Parent->existsChild($alias));
    
                my $catalog;
                Transaction( sub {
                    # create new catalog
                    my $Class = LoadClassByAlias('Category');
                    $catalog = $Class->insertObject( {
                        'Alias'    => $alias,
                        'Parent'   => $Parent,
                        'Position' => defined($hInCatalog->{'Position'})  ?  $hInCatalog->{'Position'} : $lastPosition++
                    } );
    
                    # set catalog parameters
                    $self->setParameters($catalog, $hInCatalog);
    
                    # set Attributes
                    $self->setAttributeValues($catalog, $hInCatalog->{'Attributes'});
                });
    
                push(@parentCatalogs, $Parent) unless(ElementOf($Parent, @parentCatalogs));
    
                my $hOutCatalog = {
                    'Alias'         => $self->soapString( $alias ),
                    'Path'          => $self->soapPath( $catalog ),
                    'ParentCatalog' => $self->soapString( $parentCatalog ),
                    'created'       => $self->soapBool( 1 )
                };
    
                # build return object
                $soapCatalog = $self->soapData('Catalog', $hOutCatalog, 'TCreate_Return');
            };
            if (ExistsError()) {
                my $Error = GetError();
                $soapCatalog = $self->soapData('Catalog', {
                    'Alias'         => $self->soapString( $hInCatalog->{'Alias'} ),
                    'ParentCatalog' => $self->soapString( $hInCatalog->{'ParentCatalog'} ),
                    'Error'         => $self->soapError($Error)
                }, 'TCreate_Return');
                ResetError();
            }
            push(@soapCatalogs, $soapCatalog);
        }
    
        #--- renumber children foreach catalog who got a new child
        foreach my $catalog(@parentCatalogs) {
            $catalog->renumberChildren();
        }

    });

    my $result = $self->soapData('Catalogs', $self->soapList(\@soapCatalogs, 'TCreate_Return') );
    $self->mapType([qw (TCreate_Return)], $self->getServiceTypesUrn);

    return $result;
}

#========================================================================================
# function     getRoot
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SoapData = $self->getRoot();
#----------------------------------------------------------------------------------------
# description  WebService-Method: getRoot
#               Get the alias and path of the root catalog.
#----------------------------------------------------------------------------------------
# return       $SoapData  | a SOAP::Data structure | SOAP::Data
#========================================================================================
sub getRoot {
    my $self = shift;

    my $Shop = $self->site;

    my $soapCatalog;
    eval {
        my $root = $Shop->get('Categories');
        my $hOutCatalog = {
            'Path' => $self->soapPath( $root )
        };
        $soapCatalog = $self->soapData('Root', $hOutCatalog, 'TGetRoot_Return');
    };
    if (ExistsError()) {
        my $Error = GetError();
        $soapCatalog = S$self->soapData('Root', {
            'Error' => $self->soapError($Error)
        }, 'TGetRoot_Return');
        ResetError();
    }

    $self->mapType([qw (TGetRoot_Return)], $self->getServiceUrn);
    return $soapCatalog;
}

#========================================================================================
sub _getParent {
    my $self = shift;
    my ($catalog) = @_;

    my $parent = $catalog->parent;
    return $parent->instanceOf('Category')  ?  $parent : undef;
}

#========================================================================================
# function     setParameters
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->setParameters( $catalog, $hCatalog )
#----------------------------------------------------------------------------------------
# description  No WebService-Method!
#               Sets parameters of a catalog object (for create and update catalogs).
#----------------------------------------------------------------------------------------
# input        $catalog  | catalog to update | Catalog object
# input        $hCatalog | data to assign to the catalog
#                             <ul>
#                               <li>IsVisible   - boolean
#                               <li>Position    - integer
#                               <li>Name        - localized string
#                               <li>Text        - localized string
#                             </ul> | ref.hash
#========================================================================================
sub setParameters {
    my $self = shift;
    my ($catalog, $hInCatalog) = @_;

    # set catalog parameters
    my %CatalogParams = ();
    for my $param (qw (IsVisible Position) ) {
        $CatalogParams{$param} = $hInCatalog->{$param} if defined $hInCatalog->{$param};
    }
    $catalog->set(\%CatalogParams);

    # localized parameters
    for my $param (qw(Name Text)) {
        $self->setLocalizedValues($catalog, $param, $hInCatalog->{$param}) if defined $hInCatalog->{$param};
    }
}

#========================================================================================
# function     buildSoapCatalog
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->buildSoapCatalog( $catalog, $aAttributes, $aLanguages )
#----------------------------------------------------------------------------------------
# description  No WebService-Method!
#               Build SOAP::Data object for Catalog.
#----------------------------------------------------------------------------------------
# input        $catalog | catalog | Catalog object
# input        $aAttributes | list af additional attributes | attribute array
# input        $aLanguages | list of passed language IDs available for this shop | languageID array
# return       $SoapData  | a SOAP::Data structure representing the catalog | SOAP::Data
#========================================================================================
sub buildSoapCatalog {
    my $self = shift;
    my ( $catalog, $aAttributes, $aLanguages ) = @_;

    my @soapSubCategories = map { $self->soapPath($_) }
                            @{$catalog->get('SubCategories')};
    my $hOutCatalog = {
        'Alias'             => $self->soapString( $catalog->alias ),
        'Position'          => $self->soapInt( $catalog->get('Position') ),
        'Path'              => $self->soapPath( $catalog ),
        'Class'             => $self->soapString( $catalog->class->get('Path') ),
        'ParentCatalog'     => $self->soapPath( $self->_getParent($catalog) ),
        'IsVisible'         => $self->soapBool( $catalog->get('IsVisible') ),
        'WebUrl'            => $self->soapString( $catalog->get('WebUrl') ),
        'Name'              => $self->soapList( $self->soapLocalizedValues($catalog, 'Name', $aLanguages), 'TLocalizedValue'),
        'Text'              => $self->soapList( $self->soapLocalizedValues($catalog, 'Text', $aLanguages), 'TLocalizedValue'),
        'SubCatalogs'       => $self->soapList( \@soapSubCategories, 'string'),
        'Attributes'        => $self->soapList( $self->soapAttributeValues($catalog, $aAttributes, $aLanguages), 'TAttribute')
    };
    return SOAP::Data->type( 'TGetInfo_Return')->name('Catalog' => $hOutCatalog);
}

1;
