use strict;

use Getopt::Long;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject LoadObject );
use DE_EPAGES::WebInterface::API::MessageCenter qw ( SynchronizeCache );
use Time::HiRes  qw ( time );

sub Main {
    local $| = 1;

    my ($StoreName, $Password, $ShopAlias, $AllShops, $DoReapair, $NoOutput) = GetParameters();

    my $time1 = time;

    RunOnStore(
        Store       => $StoreName,
        DBPassword  => $Password,
        Sub         => sub  {
            my $System = LoadRootObject();
            my $aShopIDs = GetShopIDs($System, $ShopAlias, $AllShops);
            CheckShops($aShopIDs, $DoReapair, $NoOutput);
        },
    );

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();

    my $time2 = time;
    printf "\nneed %.3f seconds\n", $time2-$time1 unless $NoOutput;

    return;
}

sub GetParameters {
    my ($StoreName, $Password, $ShopAlias, $AllShops, $DoReapair, $NoOutput, $Help);

    GetOptions(
        'storename=s' => \$StoreName,
        'passwd=s'    => \$Password,
        'shop=s'      => \$ShopAlias,
        'allShops'    => \$AllShops,
        'repair'      => \$DoReapair,
        'nooutput'    => \$NoOutput,
        'help'        => \$Help,
    );

    usage() if $Help;

    unless ( $StoreName ) {
        print STDERR "Missing parameter -storename\n\n";
        usage();
    }
    unless( $ShopAlias || $AllShops ) {
        print STDERR "Missing parameter -shop\n\n";
        usage();
    }

    return ($StoreName, $Password, $ShopAlias, $AllShops, $DoReapair, $NoOutput);
}

sub GetShopIDs {
    my ($System, $ShopAlias, $AllShops) = @_;

    my $ShopFolder = $System->child('Shops');
    my @ShopIDs = ();

    if ( isValid($ShopAlias) && $ShopFolder->existsChild($ShopAlias) ) {
        push(@ShopIDs, $ShopFolder->findChild($ShopAlias));
    }
    elsif ( $AllShops ) {
        push(@ShopIDs, @{$ShopFolder->findChildren()});
    }

    print STDERR "\n\nno shops are checked, check your parameters\n\n" unless scalar @ShopIDs;

    return \@ShopIDs;
}

sub CheckShops {
    my ($aShopIDs, $DoReapair, $NoOutput) = @_;

    my ($Shop, $aUserIDs);

    foreach my $ShopID ( @{$aShopIDs} ) {
        $Shop = LoadObject($ShopID);

        print "\nchecking shop " . $Shop->alias . "\n\n" unless $NoOutput;

        $aUserIDs = $Shop->child('Users')->findChildren();
        CheckUsers($aUserIDs, $DoReapair, $NoOutput);
    }

    return;
}

sub CheckUsers {
    my ($aUserIDs, $DoReapair, $NoOutput) = @_;

    my ($User, $BillingAddress);

    foreach my $UserID ( @{$aUserIDs} ) {
        $User = LoadObject($UserID);
        next if $User->get('IsMerchant');

        $BillingAddress = $User->get('BillingAddress');

        if ( not defined $BillingAddress ) {
            print 'invalid user found: ' . $User->alias . "\n" unless $NoOutput;
            RepairUser($User, $NoOutput) if $DoReapair;
        }
    }

    return;
}

sub RepairUser {
    my ($User, $NoOutput) = @_;

    eval {
        my $hUser = $User->get( [ 'Alias', 'Customers' ] );
        my $aCustomers = $hUser->{'Customers'};

        unless (scalar @{$aCustomers}) {
            print STDERR 'user '.$hUser->{'Alias'}." has no assigned customer\n\n";
            return;
        }

        my $Customer = $aCustomers->[0];
        my $hCustomer = $Customer->get( [ 'Alias', 'BillingAddress' ] );
        my $BillingAddress = $hCustomer->{'BillingAddress'};

        unless (defined $BillingAddress) {
            print STDERR 'corresponding customer '.$hCustomer->{'Alias'}." has no billing address\n\n";
            return;
        }

        my $Addresses = $User->folder('Addresses');
        my $ClonedBillingAddress = $BillingAddress->clone({
            'Alias'  => $Addresses->nextAlias('BillingAddress %s'),
            'Parent' => $Addresses
        });
        $User->set({ 'BillingAddress' => $ClonedBillingAddress });

        print 'repaired user '.$hUser->{'Alias'}."\n" unless $NoOutput;

        return;
    };
    Warning( GetError() ) if ExistsError();

    return;
}

sub usage {
    print <<END_USAGE;
Description:
    check if user objects are valid and optionally repair them

Usage:
    perl $0 [options] flags objects

options:
    -storename  name of store                                 required
    -passwd     database user password                        ()
    -shop       shop alias

flags:
    -repair     repair invalid user objects
    -allShops   check users on all shops
    -nooutput   do not print any output
    -help       show the command line options

Example:
    perl $0 -storename Store -shop DemoShop
    perl $0 -storename Store -shop DemoShop -repair
    perl $0 -storename Store -allShops
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main,
);

1;
