#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::SOAP::Builder
# state        public
# description  Description
## no critic (Tics::ProhibitLongLines)
#========================================================================================
package DE_EPAGES::DHLIntraship::API::SOAP::Builder;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::CustomerAddress;
use DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::Products;
use DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel;
use DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShopAddress;
use DE_EPAGES::DHLIntraship::API::Tools;
use DE_EPAGES::DHLIntraship::API::Constants qw (
        $CURRENCY
        $DEFAULT_HIGHER_INSURANCE_AMOUNT
        $DEFAULT_BULKFREIGHT_TYPE
);

use feature 'switch';
use strict;



#========================================================================================
sub new {
    my $class = shift;

    my $self  = bless {
        'createShipmentDD'  => \&_createShipmentDD,
    }, $class;
    return $self;
}

#========================================================================================
# function     assemble
# state        public_private
#----------------------------------------------------------------------------------------
# syntax       assemble( $operation, $aShippingLabels );
#----------------------------------------------------------------------------------------
# description  Create the hash with all necessary values for the request
#----------------------------------------------------------------------------------------
# input        $operation | Operation name | String
# input        $aShippingLabels | Ref array with Shipping Labels objects | refarray
# return       $ReturnVariable | ReturnDescription | refhash
#========================================================================================
sub assemble {
    my $self = shift;
    my ($operation, $aShippingLabels) = @_;
    return unless (defined ($self->{$operation}));
    my $ReturnVariable = $self->{$operation}($self, $aShippingLabels);

    return $ReturnVariable;
}

#========================================================================================
sub _createShipmentDD {
    my $self = shift;
    my ($ShippingLabel) = @_;

    my $SLInfo = $self->_getShippingLabelInfo($ShippingLabel);

    my $ShopAddressInfo = $self->_getShopAddressInfo($ShippingLabel);

    my $CustomerAddressInfo = $self->_getCustomerAddressInfo($ShippingLabel);

    my $ProductInfo = $self->_getProductsInfo($ShippingLabel);

    my $CustomerNumber = $self->_getCustomerNumber($ShippingLabel, $SLInfo);

    my $hSFMInfo = $SLInfo->{'ShippingFulfillmentMethod'}->get([qw (    WebserviceUsername
                                                                        WebservicePassword)]);
    $hSFMInfo->{'CustomerNumber'} = $CustomerNumber;
    #For a CustomerNumber is 50000000087201
    #EKP = 5000000008 (first 10 numbers)
    #procedureID = 72 (11 and 12 number)
    #partnerID = 01 (13 and 14 number)
    my $EKP = substr $hSFMInfo->{'CustomerNumber'}, 0, 10; ## no critic (ProhibitMagicNumbers)
    my $partnerId = substr $hSFMInfo->{'CustomerNumber'}, 12, 2; ## no critic (ProhibitMagicNumbers)

    my $returnHash = {
        %{ $self->_assembleHeader($hSFMInfo->{'WebserviceUsername'}, $hSFMInfo->{'WebservicePassword'}) },
        %{ $self->_assembleVersion() },
        'ShipmentOrder' => {
            'SequenceNumber' => $ShippingLabel->id,
            'Shipment' => {
                'ShipmentDetails' => {
                    'ProductCode' => $SLInfo->{'ProductCode'},
                    'ShipmentDate' => $SLInfo->{'EstimatedDeliveryDate'},
                    'EKP' => $EKP,
                    'Attendance' => {'partnerID' => $partnerId},
                    'DeclaredValueOfGoods'         => $ProductInfo->{'LineItemsSum'},
                    'DeclaredValueOfGoodsCurrency' =>  $CURRENCY,
                    'CustomerReference' => $SLInfo->{'Alias'},
                    'ShipmentItem' => {
                        'PackageType' => 'PK'
                    },
                    'Notification' => [{
                        'RecipientName' => $CustomerAddressInfo->{'LastName'},
                        'RecipientEmailAddress' => $CustomerAddressInfo->{'EMail'},
                    }]
                },
                'Shipper' => {%{ $self->_assembleAddress($ShopAddressInfo) }},
                'Receiver' => {%{ $self->_assembleAddress($CustomerAddressInfo) }},
            }
        }
    };
    $returnHash = $self->_fix_CompanyName_FullName_IssueForShipperAndReceiver ( $returnHash, $ShopAddressInfo, $CustomerAddressInfo );
    my $hShipmentItem = $returnHash->{'ShipmentOrder'}{'Shipment'}{'ShipmentDetails'}{'ShipmentItem'};

    $hShipmentItem->{'WeightInKG'} = $ProductInfo->{'TotalWeight'} if (defined($ProductInfo->{'TotalWeight'}));
    $hShipmentItem->{'ParcelDepth'} = $ProductInfo->{'ParcelDepth'} if (defined($ProductInfo->{'ParcelDepth'}));
    $hShipmentItem->{'ParcelLength'} = $ProductInfo->{'ParcelLength'} if (defined($ProductInfo->{'ParcelLength'}));
    $hShipmentItem->{'ParcelWidth'} = $ProductInfo->{'ParcelWidth'} if (defined($ProductInfo->{'ParcelWidth'}));

    my $Service = [];
    my $PackInternacionalOptions    = { %{ $self->_assemblePackInternacional($SLInfo) } };
    my $groupOtherOptions           = { %{ $self->_assembleGroupOther($SLInfo, $ProductInfo) } };
    my $GroupIdentlOptions          = { %{ $self->_assembleGroupIdent($SLInfo) } };
    my $DHLPaketOptions             = { %{ $self->_assemblerDHLPaket($SLInfo) } };

    push(@$Service, $PackInternacionalOptions)  if (keys%{$PackInternacionalOptions});
    push(@$Service, $groupOtherOptions)         if (keys%{$groupOtherOptions});
    push(@$Service, $GroupIdentlOptions)        if (keys%{$GroupIdentlOptions});
    push(@$Service, $DHLPaketOptions)           if (keys%{$DHLPaketOptions});

    $returnHash->{'ShipmentOrder'}{'Shipment'}{'ShipmentDetails'}{'Service'} = $Service if (scalar @$Service);

    return $returnHash;
}

#========================================================================================
# Common information assembly
#========================================================================================
sub _assembleHeader {
    my $self = shift;
    my ($user, $signature) = @_;
    my $returnValue = {
        'header' => {
            'user'     => $user,
            'signature'=> $signature,
            'type'     => '0',

        }
    };
    return $returnValue;
}

#========================================================================================
sub _assembleVersion {
    my $self = shift;

    return {
        'Version' => {
            'majorRelease' => 1,
            'minorRelease' => 0,
            'build'        => 14,
        }
    }
}

#========================================================================================
sub _assembleAddress {
    my $self = shift;
    my ($AddressObject) = @_;

    my $ReceiverData;
    if ($AddressObject->{'IsPickupAddress'}) {
        $ReceiverData->{'Packstation'} = $self->_buildPackstation($AddressObject);
    } else {
        $ReceiverData->{'Address'} = $self->_buildAddress($AddressObject);
    }

    $ReceiverData->{'Company'} = {%{$self->_assembleCompanyTag( $AddressObject )}};
    $ReceiverData->{'Communication'} = {%{$self->_assembleCommunicationTag( $AddressObject )}};

    return $ReceiverData;
}

#----------------------------------------------------------------------------------------
sub _buildPackstation {
    my $self = shift;
    my ($AddressObject) = @_;

    my $PackstationAddress = {
        'PackstationNumber' => $AddressObject->{'DHLPackstationId'},
        'PostNumber'        => $AddressObject->{'DHLPostNumber'},
        'Zip'               => $AddressObject->{'Zipcode'},
        'City'              => $AddressObject->{'City'},
    };

    return $PackstationAddress;
}

#----------------------------------------------------------------------------------------
sub _buildAddress {
    my $self = shift;
    my ($hAddressObject) = @_;

    my $hReceiverAddress;

    $hReceiverAddress = {
        'streetName'    => $hAddressObject->{'StreetOnly'},
        'streetNumber'  => $hAddressObject->{'HouseNumber'},
        'Zip'           => {},
        'city'          => $hAddressObject->{'City'},
        'Origin'        => {
            'countryISOCode' => $hAddressObject->{'Code2'}
        }
    };

    $hReceiverAddress = $self->_setCareOfName($hReceiverAddress, $hAddressObject);

    given( $hAddressObject->{'Code2'} ){
        when ('DE') {$hReceiverAddress->{'Zip'}->{'germany'}  = $hAddressObject->{'Zipcode'}}
        when ('UK') {$hReceiverAddress->{'Zip'}->{'england'}  = $hAddressObject->{'Zipcode'}}
        default     {$hReceiverAddress->{'Zip'}->{'other'}    = $hAddressObject->{'Zipcode'}}
    };

    return $hReceiverAddress;
}

#========================================================================================
# function     _assembleCompanyTag
# state        private
#----------------------------------------------------------------------------------------
# syntax       _assembleCompanyTag( $AddressObject );
#----------------------------------------------------------------------------------------
# description  assemble the company tag.
#----------------------------------------------------------------------------------------
# input        $AddressObject | Receiver address object | object
# return       $hAddressFields | hash structure with address data | refhash
#========================================================================================
sub _assembleCompanyTag {
    my $self = shift;
    my ( $AddressObject ) = @_;

    my $Name1Value = $AddressObject->{'FullName'};
    my $hCompanyField = {};
    $hCompanyField->{'Company'}{'name1'} = $Name1Value;

    if ($AddressObject->{'IsPickupAddress'}) {
        $hCompanyField->{'Company'}{'name2'} = $AddressObject->{'DHLPostNumber'};
        $hCompanyField->{'Company'}{'name1'} = $AddressObject->{'DisplayName'};
    }

    return $hCompanyField;
}

#========================================================================================
# function     _assembleCommunicationTag
# state        private
#----------------------------------------------------------------------------------------
# syntax       _assembleCommunicationTag( $AddressObject );
#----------------------------------------------------------------------------------------
# description  assemble the Communication tag.
#----------------------------------------------------------------------------------------
# input        $AddressObject | Input1Description | object
# return       $hAddressFields | hash structure with address data | refhash
#========================================================================================
sub _assembleCommunicationTag{
    my $self = shift;
    my ( $AddressObject ) = @_;

    my $hTagContend = {};
    $hTagContend->{'phone'} = $AddressObject->{'Phone'}  if(isValid($AddressObject->{'Phone'}));
    $hTagContend->{'email'} = $AddressObject->{'EMail'}  if(isValid($AddressObject->{'EMail'}));
    $hTagContend->{'mobile'} = $AddressObject->{'PhoneCell'}  if(isValid($AddressObject->{'PhoneCell'}));
    return $hTagContend;
}

#========================================================================================
# function     _fix_CompanyName_FullName_IssueForShipperAndReceiver
# state        private
#----------------------------------------------------------------------------------------
# syntax       _fix_CompanyName_FullName_IssueForShipperAndReceiver( $hAssembledAddress, $ShopAddressInfo, $CustomerAddressInfo );
#----------------------------------------------------------------------------------------
# description  DHL hacked there own API for company names
#                         | with company          | without company (default)
#               ---------------------------------------------------------
#               Shipper   | name1=CompanyName     | name1=FullName
#                         | name2=notPrinted      | name2=empthy
#                         | contactPerson=FullName| contactPerson=empthy
#               ---------------------------------------------------------
#               Receiver  | name1=CompanyName     | name1=FullName
#                         | name2=FullName        | name2=empthy
#                         | contactPerson=empthy  | contactPerson=empthy
#----------------------------------------------------------------------------------------
# input        $hAssembledAddress | Assembled data for SOAP requst| ref_hash
# input        $ShopAddressInfo | contains shop specific data | object
# input        $CustomerAddressInfo | contains customer specific data | object
# return       $hAssembledAddress | Assembled data for SOAP requst| ref_hash
#========================================================================================
sub _fix_CompanyName_FullName_IssueForShipperAndReceiver {
    my $self = shift;
    my ( $hAssembledAddress, $ShopAddressInfo, $CustomerAddressInfo ) = @_;
    # shipper
    my $CompanyNameShop = $self->_truncateAttribute($ShopAddressInfo->{'Company'}, 'Company');
    if (isValid($CompanyNameShop)){
        $hAssembledAddress->{'ShipmentOrder'}{'Shipment'}{'Shipper'}{'Company'}{'Company'}{'name1'} = $CompanyNameShop;
        delete $hAssembledAddress->{'ShipmentOrder'}{'Shipment'}{'Shipper'}{'Company'}{'Company'}{'name2'};
        $hAssembledAddress->{'ShipmentOrder'}{'Shipment'}{'Shipper'}{'Communication'}{'contactPerson'} = $ShopAddressInfo->{'FullName'};
    }
    # Receiver
    my $CompanyNameCustomer = $self->_truncateAttribute($CustomerAddressInfo->{'Company'}, 'Company');
    if (isValid($CompanyNameCustomer)){
        $hAssembledAddress->{'ShipmentOrder'}{'Shipment'}{'Receiver'}{'Company'}{'Company'}{'name1'} = $CompanyNameCustomer;
        $hAssembledAddress->{'ShipmentOrder'}{'Shipment'}{'Receiver'}{'Company'}{'Company'}{'name2'} = $CustomerAddressInfo->{'FullName'};
    }

    $hAssembledAddress->{'ShipmentOrder'}{'Shipment'}{'Receiver'}{'Communication'}{'contactPerson'} =
        ($CustomerAddressInfo->{'IsPickupAddress'}) ? $CustomerAddressInfo->{'DisplayName'} : $CustomerAddressInfo->{'FullName'};

    return $hAssembledAddress;
}

#========================================================================================
#  Assemble
#========================================================================================
sub _assemblePackInternacional {
    my $self = shift;
    my ($ShippingLabel) = @_;

    my $returnValue = {'ServiceGroupBusinessPackInternational' => {}};

    $returnValue->{'ServiceGroupBusinessPackInternational'}{'Economy'} = 1 if(defined($self->_getService($ShippingLabel->{'DHLServices'},'Economy')));
    $returnValue->{'ServiceGroupBusinessPackInternational'}{'Premium'} = 1 if(defined($self->_getService($ShippingLabel->{'DHLServices'},'Premium')));
    $returnValue->{'ServiceGroupBusinessPackInternational'}{'Seapacket'} = 1 if(defined($self->_getService($ShippingLabel->{'DHLServices'},'Seapacket')));
    $returnValue->{'ServiceGroupBusinessPackInternational'}{'CoilWithoutHelp'} = 1 if(defined($self->_getService($ShippingLabel->{'DHLServices'},'CoilWithoutHelp')));
    $returnValue->{'ServiceGroupBusinessPackInternational'}{'AmountInternational'} = 1 if(defined($self->_getService($ShippingLabel->{'DHLServices'},'AmountInternational')));

    return keys( %{$returnValue->{'ServiceGroupBusinessPackInternational'}} ) ? $returnValue : {};
}

sub _assembleGroupOther {
    my $self = shift;
    my ($ShippingLabel, $ProductInfo) = @_;
    my $returnValue = {'ServiceGroupOther' => {}};

    $returnValue->{'ServiceGroupOther'}{'DirectInjection'} = 1 if(defined($self->_getService($ShippingLabel->{'DHLServices'},'DirectInjection')));
    $returnValue->{'ServiceGroupOther'}{'Bypass'} = 1 if(defined($self->_getService($ShippingLabel->{'DHLServices'},'Bypass')));
    if( defined($self->_getService($ShippingLabel->{'DHLServices'},'Insurance')) )
    {
        $returnValue->{'ServiceGroupOther'}{'HigherInsurance'} = {'InsuranceAmount' => $DEFAULT_HIGHER_INSURANCE_AMOUNT, 'InsuranceCurrency' => $CURRENCY};
    }

    if( defined($self->_getService($ShippingLabel->{'DHLServices'},'Bulkfreight')) )
    {
        $returnValue->{'ServiceGroupOther'}{'Bulkfreight'} = { 'BulkfreightType' => $DEFAULT_BULKFREIGHT_TYPE };
    }

    return keys( %{$returnValue->{'ServiceGroupOther'}} ) ? $returnValue : {};
}

sub _assembleGroupIdent {
    my $self = shift;
    my ($ShippingLabel) = @_;
    my $returnValue = {'ShipmentServiceGroupIdent' => {}};

    $returnValue->{'ShipmentServiceGroupIdent'}{'Personally'} = 1 if(defined($self->_getService($ShippingLabel->{'DHLServices'},'Personally')));
    $returnValue->{'ShipmentServiceGroupIdent'}{'ReturnReceipt'} = 1 if(defined($self->_getService($ShippingLabel->{'DHLServices'},'ReturnReceipt')));
    $returnValue->{'ShipmentServiceGroupIdent'}{'ProofOfDelivery'} = 1 if(defined($self->_getService($ShippingLabel->{'DHLServices'},'ProofOfDelivery')));

    return keys( %{$returnValue->{'ShipmentServiceGroupIdent'}} ) ? $returnValue : {};
}

sub _assemblerDHLPaket {
    my $self = shift;
    my ($ShippingLabel) = @_;
    my $returnValue = {'ServiceGroupDHLPaket' => {}};

    $returnValue->{'ServiceGroupDHLPaket'}{'Multipack'} = 1 if(defined($self->_getService($ShippingLabel->{'DHLServices'},'Multipack')));
    $returnValue->{'ServiceGroupDHLPaket'}{'RegioPacket'} = 1 if(defined($self->_getService($ShippingLabel->{'DHLServices'},'RegioPacket')));
    $returnValue->{'ServiceGroupDHLPaket'}{'ParticularDelivery'} = 1 if(defined($self->_getService($ShippingLabel->{'DHLServices'},'ParticularDelivery')));

    return keys( %{$returnValue->{'ServiceGroupDHLPaket'}} ) ? $returnValue : {};
}

sub _getService {
    my $self = shift;
    my ($aServices, $ServiceName) = @_;

    my ($returnValue) = grep {$_->{'SelectedOption'} eq $ServiceName}  @$aServices;

    return $returnValue;
}

sub _setCareOfName {
    my $self = shift;
    my ($hReceiverAddress, $hAddressObject) = @_;

    my $AddressExtension = $hAddressObject->{'AddressExtension'};
    my $Street2 = $hAddressObject->{'Street2'};

    my $CareOfName = '';

    if (isValid($AddressExtension)) {
        $CareOfName = $AddressExtension;
    }

    if (isValid($Street2)) {
        $CareOfName .= (isValid($CareOfName)) ? ' ' . $Street2 : $Street2;
    }

    if (isValid($CareOfName)) {
        $hReceiverAddress->{'careOfName'} = $self->_truncateAttribute($CareOfName, 'careOfName');
    }

    return $hReceiverAddress;
}

#----------------------------------------------------------------------------------------
# Wrappers
#----------------------------------------------------------------------------------------
sub _getCustomerAddressInfo {
    my $self = shift;
    my ($ShippingLabel) = @_;

    my $CustomerAddressInfo = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::CustomerAddress->new( $ShippingLabel );
    $CustomerAddressInfo->loadAttributes();

    return $CustomerAddressInfo;
}

#----------------------------------------------------------------------------------------
sub _getCustomerNumber {
    my $self = shift;
    my ($ShippingLabel, $SLInfo) = @_;

    my $Order = $ShippingLabel->get('Order');
    my $Address = ($Order->get('ShippingAddress')) ? $Order->get('ShippingAddress') : $Order->get('BillingAddress');

    my $Tools = DE_EPAGES::DHLIntraship::API::Tools->new();

    return $Tools->GetCustomerNumber($SLInfo->{'ShippingFulfillmentMethod'}, $Address);
}

#----------------------------------------------------------------------------------------
sub _getProductsInfo {
    my $self = shift;
    my ($ShippingLabel) = @_;

    my $ProductsInfo = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::Products->new( $ShippingLabel );
    $ProductsInfo->loadAttributes();

    return $ProductsInfo;
}

#----------------------------------------------------------------------------------------
sub _getShippingLabelInfo {
    my $self = shift;
    my ($ShippingLabel) = @_;

    my $ShippingLabelInfo = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShippingLabel->new( $ShippingLabel );
    $ShippingLabelInfo->loadAttributes();

    return $ShippingLabelInfo;
}

#----------------------------------------------------------------------------------------
sub _getShopAddressInfo {
    my $self = shift;
    my ($ShippingLabel) = @_;

    my $ShopAddressInfo = DE_EPAGES::DHLIntraship::API::ShippingLabelAttributes::ShopAddress->new( $ShippingLabel );
    $ShopAddressInfo->loadAttributes();

    return $ShopAddressInfo;
}

#----------------------------------------------------------------------------------------
sub _truncateAttribute {
    my $self = shift;
    my ($Attribute, $FieldName) = @_;

    my $Tools = DE_EPAGES::DHLIntraship::API::Tools->new();

    return $Tools->truncateAttribute($Attribute, $FieldName);
}


## use critic (Tics::ProhibitLongLines)
1;