#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::PackstationResponse
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for SOAP Capture response.
#========================================================================================
package DE_EPAGES::DHLIntraship::t::unit::PackstationResponse;

use base DE_EPAGES::Core::API::Test;
use strict;

use Data::Dumper;
use DE_EPAGES::Test::Mock::EpagesObject;
use Readonly;
use Test::Exception;
use Test::MockObject::Extends;
use Test::More;
use DE_EPAGES::Test::Mock::Mockify qw ( WasCalled );
use DE_EPAGES::DHLIntraship::API::SOAP::PackstationResponse;
use Math::BigFloat;

sub test {
    my $self = shift;
    $self->_test_packstation_response_positive();
    $self->_test_packstations_response_positive();
    $self->_test_packstation_response_negative();
    done_testing();
}

sub _test_packstation_response_positive {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedOperationName         = $self->_getMockedOperationName();
    my $NumberOfPackstations        = 1;
    my $MockedDHLSoapResponse       = $self->_getMockedDHLSoapResponse($NumberOfPackstations);
    my $MockedPackstationResponse   = $self->_getMockedPackstationResponse();
    my $hResponse                   = $MockedPackstationResponse->parseResponse($MockedOperationName,$MockedDHLSoapResponse);
    my $NumberOfPackstationsResults = scalar(@$hResponse);
    my $ExpectedResponse            = $self->_getAddress();

    lives_ok { $hResponse } "$SubTestName - Successful response parsing.";
    is($NumberOfPackstationsResults,$NumberOfPackstations, "$SubTestName - Number of Packstations is the expected");
    is_deeply(@$hResponse[0],$ExpectedResponse, "$SubTestName - Packstation is the expected");

    return;
}

sub _test_packstations_response_positive {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedOperationName         = $self->_getMockedOperationName();
    my $NumberOfPackstations        = 5;
    my $MockedDHLSoapResponse       = $self->_getMockedDHLSoapResponse($NumberOfPackstations);
    my $MockedPackstationResponse   = $self->_getMockedPackstationResponse();
    my $hResponse                   = $MockedPackstationResponse->parseResponse($MockedOperationName,$MockedDHLSoapResponse);
    my $NumberOfPackstationsResults = scalar(@$hResponse);
    my $ExpectedResponse            = $self->_getAddress();

    lives_ok { $hResponse } "$SubTestName - Successful response parsing.";
    is($NumberOfPackstationsResults,$NumberOfPackstations, "$SubTestName - Number of Packstations is the expected");
    for (my $i=0; $i < $NumberOfPackstations; $i++) {
        is_deeply(@$hResponse[$i],$ExpectedResponse, "$SubTestName - Packstation $i is the expected");
    }

    return;
}

sub _test_packstation_response_negative {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedOperationName         = $self->_getMockedOperationName();
    my $MockedDHLSoapResponse       = $self->_getMockedDHLSoapFaultResponse();
    my $MockedPackstationResponse   = $self->_getMockedPackstationResponse();

    $MockedPackstationResponse->parseResponse($MockedOperationName,$MockedDHLSoapResponse);
    ok( WasCalled($MockedPackstationResponse , '_Warning'), "$SubTestName - Warning function was called because faulty response.");

    return;
}

#----------------------------------------------------------------------------------------
sub _getMockedOperationName {
    my $self = shift;
    return 'operationName';
}

#----------------------------------------------------------------------------------------
sub _getMockedDHLSoapResponse {
    my $self = shift;
    my ($NumberOfPackstations) = @_;

    my @SoapResponseArray ;
    my $hAddress;
    for (my $i=0; $i < $NumberOfPackstations; $i++) {
        $hAddress = $self->_createPackstationAddress();
        push(@SoapResponseArray, $hAddress);
    }

    my $SoapResponse = {
        'parameters' => {
                    'packstation' => [@SoapResponseArray]
        }
    };

    return $SoapResponse;
}

#----------------------------------------------------------------------------------------
sub _getMockedDHLSoapFaultResponse {
    my $self = shift;
    my $SoapFaultResponse = {
                            'Fault' => {
                                'faultcode' => '1234',
                                'reason' => 'Fault'
                            }
                        };
    return $SoapFaultResponse;
}

#----------------------------------------------------------------------------------------
sub _getMockedPackstationResponse {
    my $self = shift;

    my $aParameterList = [];
    my $MockedPackstationResponse =  DE_EPAGES::Test::Mock::Mockify->new(
            'DE_EPAGES::DHLIntraship::API::SOAP::PackstationResponse',
            $aParameterList
    );
    
    $MockedPackstationResponse->addMockWithReturnValueAndParameterCheck('_Warning', undef, ['string', 'hashref']);

    return $MockedPackstationResponse->getMockObject();
}

#----------------------------------------------------------------------------------------
sub _createPackstationAddress {
    my $self = shift;

    my $hAddress = {
        'address' => {
            'remark' => 'Remark',
            'street' => 'Street',
            'streetNo'=>'23',
            'zip'=> '1234',
            'city'=> 'City',
            'district' => 'District'
            },
        'location'  => {
            'distance'=> Math::BigFloat->new('1256'),
            'latitude'=> Math::BigFloat->new('43.23'),
            'longitude'=> Math::BigFloat->new('12.45'),
            },
        'packstationId' => Math::BigFloat->new('123')
    };

    return $hAddress;
}

#----------------------------------------------------------------------------------------
sub _getAddress {
    my $self = shift;

    my $hAddress = {
        'address' => 'Street 23',
        'city'=> 'City',
        'distance'=> '1.3 km',
        'district' => 'District',
        'latitude'=> Math::BigFloat->new('43.23'),
        'longitude'=> Math::BigFloat->new('12.45'),
        'name' => 'Remark',
        'packstationId' => Math::BigFloat->new('123'),
        'zipcode'=> '1234'
    };

    return $hAddress;
}


__PACKAGE__->runScript();
