﻿#!/usr/bin/perl

#============================================================================
# Author: Friedrich Gehring
# Description:
#     Add translations for the Unity MBO to Translation.ActionsUnity.xml
#============================================================================

use utf8;
use strict;
use Getopt::Long;
use DE_EPAGES::Core::API::File qw ( GetFileNames
                                    ExistsFile
                                    ExistsDirectory
                                    GetCachedFile
                                    GetFileContent
                                    WriteFile );
use DE_EPAGES::Core::API::Script qw( RunScript );
use Tie::File;
use XML::Simple;
use XML::SAX::ParserFactory;

use constant DATA_PATH => '/Database/XML/';
use constant PERMISSIONS_UNITY_FILE_NAME => 'PermissionsUnity.xml';
use constant TRANSLATION_FILE_PATTERN => 'Translation.Actions';
use constant TRANSLATION_UNITY_FILE_PATTERN => 'Translation.ActionsUnity.';
use constant UNITY_LANGUAGE_CODES => ['de', 'en', 'en-US'];

# package variable for warning messages
our @Warnings = ();

sub Main {
    my $LanguagePrepared = "de";
    my $InputLanguageCodes;
    my $CartridgeName;
    my $Help;
    my $Verbose = 1;
    my $MoveAll = 0;

    GetOptions(
        'cart=s'  => \$CartridgeName,
        'pre=s'   => \$LanguagePrepared,
        'lang=s'  => \$InputLanguageCodes,
        'output!' => \$Verbose,
        'help'    => \$Help,
        'all'     => \$MoveAll,
    );

    Usage() if $Help;
    Usage( 'Missing required parameter -cart' ) unless defined $CartridgeName;
    my $CartridgePath;

    # construct cartridge path out of name
    if ($ENV{'EPAGES_CARTRIDGES'}) {
        $CartridgePath = $ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/' . $CartridgeName;
    } else {
        die "Environment variable \$EPAGES_CARTRIDGES is not set. Please set it using the command: \n"
        . ". /etc/default/epages6";
    }

    die "Cartridge \"$CartridgeName\" does not exits in $ENV{'EPAGES_CARTRIDGES'}/DE_EPAGES."
        unless ExistsDirectory($CartridgePath);

    # construct Database/XML path
    my $DataPath = $CartridgePath . DATA_PATH;

    die "Path \"$DataPath\" does not exits in cartridge \"$CartridgeName\"."
        unless ExistsDirectory($DataPath);

    # assign language codes
    my $LanguageCodes;
    if (defined $InputLanguageCodes && !(lc $InputLanguageCodes eq 'all')) {
        if (lc $InputLanguageCodes eq 'unity') {
            $LanguageCodes = UNITY_LANGUAGE_CODES;
        } else {
            $LanguageCodes = [split(/,/, $InputLanguageCodes)];
        }
    } else {
        # get language codes for moving
        $LanguageCodes = _getLanguageCodes($DataPath, TRANSLATION_FILE_PATTERN, $LanguagePrepared, $MoveAll);
    }

    die "Error: No " . TRANSLATION_FILE_PATTERN . " files found in path: $DataPath."
        unless scalar @$LanguageCodes;

    # duplicate the actions to translate
    my $aActionsToCompare;
    # get actions to duplicate from permissions file if no file prepared
    # and from translation file otherwise
    $aActionsToCompare = $MoveAll ? _retrieveActionsFromPermissions($DataPath)
        : _retrieveActionsFromPreparedTranslations($LanguagePrepared, $DataPath);
    my $aActionsToTranslate;
    foreach my $LanguageToMove (@$LanguageCodes) {
        my $FilePath = $DataPath . TRANSLATION_UNITY_FILE_PATTERN . $LanguageToMove . '.xml';
        $aActionsToTranslate = _findActionsToTranslate($LanguageToMove, $FilePath, $aActionsToCompare);
        if ($Verbose) {
            print "===================================================================\n";
            print "Language: $LanguageToMove\n";
            print "-------------------------------------------------------------------\n";
        }
        if (scalar @{$aActionsToTranslate}) {
            # create a new Unity file if none exists so far
            _createTranslationFile($FilePath, $LanguageToMove, $Verbose) unless ExistsFile($FilePath);
            _translateActions($aActionsToTranslate, $LanguageToMove, $DataPath, $Verbose);
        } else {
            print "No translations duplicated.\n"
        }
    }

    # print warnings if any
    print "===================================================================\n";
    if (scalar @Warnings) {
        print "\nFinished duplicating translations with the following warnings:\n\n";
        print $_ . "\n" foreach @Warnings;
    } else {
        print "Duplicating translations finished sucessfully.\n"
    }
}

#========================================================================================
# §function     _getLanguageCodes
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       @LanguageCodes
#                   = _getLanguageCodes($Path, $FilePattern, $LanguagePrepared, $MoveAll);
#----------------------------------------------------------------------------------------
# §description  Scans the file list of the directory and
#               returns all language codes for which a file with the file pattern exists
#               except the prepared language (unless $MoveAll = 1)
#----------------------------------------------------------------------------------------
# §input        $LanguagePrepared | language code (en,de ...) | string
# §input        $Path | (absolute) path of the directory | string
# §input        $FilePattern | start pattern ot the file name to match | string
# §input        $MoveAll | true if all languages should be moved | boolean
# §return       $aLanguageCodes | list language codes | array.ref
#========================================================================================
sub _getLanguageCodes {
    my ($Path, $FilePattern, $LanguagePrepared, $MoveAll) = @_;

    my @LanguageCodes;
    foreach my $File (@{GetFileNames($Path)}) {
        # get language codes for all files matching the given template path
        if ($File =~ m/$FilePattern\.(.+)\.xml$/) {
            push @LanguageCodes, $1 if (($1 ne $LanguagePrepared) or $MoveAll);
        }
    }

    return \@LanguageCodes;
}

#========================================================================================
# §function     _retrieveActionsFromPermissions
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $aActions = _retrieveActionsFromPreparedTranslations($LanguagePrepared,
#                   $DataPath);
#----------------------------------------------------------------------------------------
# §description  parses the translation file and returns an array of all included
#               actions
#----------------------------------------------------------------------------------------
# §input        $LanguagePrepared | language code (en,de ...) | string
# §input        $DataPath | (absolute) path of Permission file | string
# §return       $aActions | list of actions | array.ref
#========================================================================================
sub _retrieveActionsFromPreparedTranslations {
    my ($LanguagePrepared, $DataPath) = @_;

    # load prepared file
    my $PreparedPath = $DataPath . TRANSLATION_UNITY_FILE_PATTERN . $LanguagePrepared . '.xml';
    die "File $PreparedPath does not exist." unless ExistsFile($PreparedPath);

    my $aActions = _parseTranslationFile($PreparedPath, $LanguagePrepared);
    push (@Warnings, "For action \'$_\' the prefix "
        . "\'Unity\' seems to be missing in file $PreparedPath.")
        foreach grep { m:\/(MBO-View|MBO-Search): } @$aActions;
    return $aActions;
}

#========================================================================================
# §function     _retrieveActionsFromPermission
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $aActions = _retrieveActionsFromPermissions($DataPath);
#----------------------------------------------------------------------------------------
# §description  parses the Permission.xml file and returns an array of all included
#               actions for the merchant
#----------------------------------------------------------------------------------------
# §input        $DataPath | (absolute) path of Permission file | string
# §return       $aActions | list of actions | array.ref
#========================================================================================
sub _retrieveActionsFromPermissions {
    my ($DataPath) = @_;

    # load permission file
    my $PermissionsUnityPath = $DataPath . PERMISSIONS_UNITY_FILE_NAME;
    die "File $PermissionsUnityPath does not exist." unless ExistsFile($PermissionsUnityPath);

    my $PermissionsRawXML = GetCachedFile( $PermissionsUnityPath ,
        sub() { return GetFileContent($PermissionsUnityPath, { binary => 1 } ) } );
    my $PermissionsXML = XMLin($PermissionsRawXML, ForceArray => 1 );
    return _parsePermissionXML($PermissionsXML);
}

#========================================================================================
# §function     _findActionsToTranslate
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $aActionsToTranslate = _findActionsToTranslate($Language, $FilePath,
#                   $aActionsToCompare);
#----------------------------------------------------------------------------------------
# §description  Compares the actions of a translation file to some predefined actions
#               and returns an array of all actions that are present in both lists
#----------------------------------------------------------------------------------------
# §input        $Language | language code (en,de ...) | string
# §input        $FilePath | (absolute) file path of translation file | string
# §input        $aActionsToCompare | predefined list of actions | array.ref
# §return       $aActionsToTranslate | list of actions to translate | array.ref
#========================================================================================
sub _findActionsToTranslate {
    my ($Language, $FilePath, $aActionsToCompare) = @_;

    if (!ExistsFile($FilePath)) {
        # translate all actions if no translations already given
        return $aActionsToCompare;
    } else {
        # compare to existing translations
        my @Actions = @{ _parseTranslationFile($FilePath, $Language) };
        # return all actions that have not been translated yet
        my @ActionsToTranslate = map { $_ } grep { not $_ ~~ @Actions } @$aActionsToCompare;
        return \@ActionsToTranslate;
    }
}

#========================================================================================
# §function     _createUnityFile
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       _createTranslationFile($Path, $Language, $Verbose);
#----------------------------------------------------------------------------------------
# §description  creates a new translation file to the given path
#----------------------------------------------------------------------------------------
# §input        $Path | Path where the file should be located | string
# §input        $Language | language code (en,de ...) | string
# §input        $Verbose | if true output is printed, if false not | boolean
#========================================================================================
sub _createTranslationFile {
    my ($FilePath, $Language, $Verbose) = @_;

    my ($LanguageCode) = split /-/, $Language;

    # content for an empty tanslation file
    my $Content = "<?xml version='1.0' encoding='utf-8'?>\n"
        . "<epages>\n"
        . "  <Language Language=\"$LanguageCode\">\n"
        . "  </Language>\n"
        . "</epages>\n";

    if ($Verbose) {
        print "Create File: $FilePath\n";
        print "-------------------------------------------------------------------\n";
    }

    WriteFile($FilePath, \$Content);

    return;
}

#========================================================================================
# §function     _translateActions
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       _translateActions($aActionsToTranslate, $Language, $DataPath, $Verbose);
#----------------------------------------------------------------------------------------
# §description  Duplicates the translations of the actions to move from the tanslation
#               file to the related unity file
#----------------------------------------------------------------------------------------
# §input        $aActionsToTranslate | list of actions to translate | array.ref
# §input        $LanguageCode | language code (en,de ...) | string
# §input        $DataPath | (absolute) path of the translation file | string
# §input        $Verbose | if true output is printed, if false not | boolean
#========================================================================================
sub _translateActions {
    my ($aActionsToTranslate, $LanguageCode, $DataPath, $Verbose) = @_;

    my $Ep6Path = $DataPath . TRANSLATION_FILE_PATTERN . '.' . $LanguageCode . '.xml';
    my $UnityPath = $DataPath . TRANSLATION_UNITY_FILE_PATTERN . $LanguageCode . '.xml';
    my @Ep6FileLines;
    my @UnityFileLines;

    # tie the lines of the translation file to an array
    tie @Ep6FileLines, 'Tie::File', $Ep6Path
        or die "Error: Unable to open file $Ep6Path.";
    my $hKeywordLines = _assignLineNumberToKeyword(\@Ep6FileLines);
    # sort actions to translate in order of the line numbers
    @$aActionsToTranslate = sort { $hKeywordLines->{$a}->{'Start'}
        <=> $hKeywordLines->{$b}->{'Start'} } @$aActionsToTranslate;

    # tie the lines of the dictionary to an array
    tie @UnityFileLines, 'Tie::File', $UnityPath
        or die "Error: Unable to open file $UnityPath.";

    my $LinesToFileEnding = 1;
    my $LinesInFile = scalar @UnityFileLines;
    # scan the end of the dictionary
    # until it is possible to add translation lines above the </Language> tag
    while ($UnityFileLines[-$LinesToFileEnding] !~ m:</Language>:) {
        $LinesToFileEnding++;
        die "Error: No closing </Language> tag in file $UnityPath"
            if $LinesToFileEnding > $LinesInFile;
    };

    if ($Verbose) {
        print "$Ep6Path: Move translations of actions to $UnityPath\n";
        print "-------------------------------------------------------------------\n";
    }
    my $Counter=0;
    my $LineToMove;

    # duplicating all actions from the beginning of the file to the end
    foreach my $ActionToTranslate ( @$aActionsToTranslate ) {
        # only duplicate actions that are assigned to a line number
        if (exists $hKeywordLines->{$ActionToTranslate}->{'Start'}) {
            print "Action: $ActionToTranslate\n" if $Verbose;
            my $LineToStart = $hKeywordLines->{$ActionToTranslate}->{'Start'};
            my $LineToEnd = $hKeywordLines->{$ActionToTranslate}->{'End'};
            for (my $KeywordLine = $LineToStart; $KeywordLine <= $LineToEnd; $KeywordLine++) {
                # find line to move
                $LineToMove = $Ep6FileLines[$KeywordLine];
                # insert "Unity" prefix
                $LineToMove =~ s/\/MBO-/\/UnityMBO-/ if $LineToMove =~ m/Path=/;
                # add the line to the end of the translation file,
                # offset $LinesToFileEnding the closing </Language> </epages> tags
                splice @UnityFileLines, -$LinesToFileEnding, 0, $LineToMove;
                # count translations that have been duplicated
                $Counter++;
            }
        } else {
            $ActionToTranslate =~ s/\/UnityMBO-/\/MBO-/;
            push @Warnings, "Action '" . $ActionToTranslate . "' not found in file" . $Ep6Path . "."
                unless $ActionToTranslate =~ m:\/(MBO-View|MBO-Search):;
        }
    }
    print "No translations duplicated.\n" if ($Counter == 0 and $Verbose);

    untie @Ep6FileLines;
    untie @UnityFileLines;
}

#========================================================================================
# §function     _assignLineNumberToKeyword
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $hKeywordLines = _assignLineNumberToKeyword(\@FileLines);
#----------------------------------------------------------------------------------------
# §description  For an array that contains the lines of a translation file a hash
#               is created whose keys are the keywords that are mapped
#               to the corresponding line number in the file (starting with 0)
#----------------------------------------------------------------------------------------
# §input        $aLineArray | Array of lines from a translation file | array.ref
# §return       $hKeywords | keyword to linenumber map | hash.ref
#========================================================================================
sub _assignLineNumberToKeyword {
    my ($aLineArray) = @_;

    my $XML;
    my %Keywords;

    my $NumberOfLines = scalar @$aLineArray;
    for (my $LineNumber = 0; $LineNumber < $NumberOfLines;  $LineNumber++) {
        my $element = $$aLineArray[$LineNumber];
        # take only lines with a Path attribute into account
        if ($element =~ m/Path=/) {
            my $LineNumberStart = $LineNumber;
            # find all lines related to an object
            while ($element !~ m:</Object>: and $LineNumber < $NumberOfLines) {
                $LineNumber++;
                $element .= $$aLineArray[$LineNumber];
            }
            if ($element =~ m:</Object>:) {
                # parse XML to filter out the keyword
                $XML = XMLin($element);
                my $Path = $XML->{'Path'};
                $Path =~ s/\/MBO-/\/UnityMBO-/;
                # store line area of the translation for the keyword
                $Keywords{$Path}{'Start'} = $LineNumberStart;
                $Keywords{$Path}{'End'} = $LineNumber;
            } else {
                $element =~ s/\R//g;
                push (@Warnings,
                    "Translation \'".$element."\' in Line ".($LineNumber+1).
                    " has no closing </Object> tag. Error in XML file.");
            }
        }
    }

    return \%Keywords;
}

#========================================================================================
# §function     _parseTranslationXML
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Translations = _parseTranslationXML($XML, $LanguageCode);
#----------------------------------------------------------------------------------------
# §description  parses the given XML structure into a hash
#----------------------------------------------------------------------------------------
# §input        $XML | raw hash of the files XML structure | hash.ref
# §input        $LanguageCode | language code (en,de ...) | string
# §return       $hTranslationsPrepared | hash that assigns the translations
#                   to the actions | hash.ref
#========================================================================================
sub _parseTranslationXML {
    my ($XML, $LanguageCode) = @_;
    my $hParams = {
        'FirstLevel' => 'Language',
        'SecondLevel' => 'Object',
        'Attribute' => 'Language',
        'AttributeValue' => $LanguageCode
    };

    my $aObjects = _parseXML($XML, $hParams);
    my %TranslationsPrepared;

    # map the content of the translation to its path
    foreach my $Object (@$aObjects) {
        foreach my $Attribute (@{$Object->{'Attribute'}}) {
            $TranslationsPrepared{$Object->{'Path'}}{$Attribute->{'Name'}} = $Attribute->{'content'};
        }
    }

    return \%TranslationsPrepared;
}

#========================================================================================
# §function     _parsePermissionFile
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Translations = _parseTranslationFile($XML);
#----------------------------------------------------------------------------------------
# §description  parses the given XML structure into a hash
#----------------------------------------------------------------------------------------
# §input        $XML | raw hash of the files XML structure | hash.ref
# §return       $hTranslationsPrepared | hash that assigns the translations
#                   to the actions | hash.ref
#========================================================================================
sub _parsePermissionXML {
    my ($XML) = @_;
    my $hParams = {
        'FirstLevel' => 'Role',
        'SecondLevel' => 'RoleAction',
        'Attribute' => 'Path',
        'AttributeValue' => '/Classes/Shop/Roles/Merchant'
    };

    my $aRoleActions = _parseXML($XML, $hParams);

    push (@Warnings, "For action \'$_->{'Action'}\' of class \'$_->{'Class'}\' the prefix "
        . "\'Unity\' seems to be missing in file PermissionsUnity.xml.")
        foreach grep { $_->{'Action'} =~ m/^(MBO-View|MBO-Search)/ } @$aRoleActions;

    # construct the path for all found actions
    my @Actions = map { '/Classes/' . $_->{'Class'} . '/Actions/' . $_->{'Action'} }
        grep { $_->{'Action'} =~ m/^(UnityMBO-View|UnityMBO-Search)/ } @$aRoleActions;
    return \@Actions;
}

#========================================================================================
# §function     _parseXML
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Translations = _parseTranslationFile($XML, $hParams);
#----------------------------------------------------------------------------------------
# §description  parses the given XML into a list for a given structure
#----------------------------------------------------------------------------------------
# §input        $XML | raw hash of the files XML structure | hash.ref
# §input        $hParams | parameters for the structure: <ul>
#                   <li>FirstLevel - xml structure to parse on fist level- string
#                   <li>Attribute - attribute to gep for - string
#                   <li>AttributeValue - related value- string
#                   <li>SecondLevel - second xml level to store - string
#               </ul>| hash.ref
# §return       $aList | list of the remaining XML structure | array.ref
#========================================================================================
sub _parseXML {
    my ($XML, $hParams) = @_;

    # create a list of the related XML studture
    # we expect it to have only one entry
    my @List = map { $_->{$hParams->{'SecondLevel'}} }
        grep { $_->{$hParams->{'Attribute'}} eq $hParams->{'AttributeValue'} }
        @{$XML->{$hParams->{'FirstLevel'}}};

    die "$hParams->{'Attribute'} $hParams->{'AttributeValue'} is not defined."
        unless scalar @List;
    die "$hParams->{'Attribute'} $hParams->{'AttributeValue'} is defined multiple times."
        if scalar @List > 1;

    return \@{$List[0]};
}

#========================================================================================
# §function     _parseTranslationFile
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $aActions = _parseTranslationFile($Path, $Language);
#----------------------------------------------------------------------------------------
# §description  parses the translation file at the given path and returns a list of its actions
#----------------------------------------------------------------------------------------
# §input        $Path | (absolute) path of the translation file | string
# §input        $Language | language code (en,de ...) | string
# §return       $aActions | list of actions | array.ref
#========================================================================================
sub _parseTranslationFile {
    my ($Path, $Language) = @_;

    my $RawXML = GetCachedFile( $Path ,
        sub() { return GetFileContent($Path, { binary => 1 } ) } );
    my $XML = XMLin($RawXML, ForceArray => 1 );
    my ($LanguageCode) = split /-/, $Language;
    my $Translations = _parseTranslationXML($XML, $LanguageCode);
    my @Actions = keys %$Translations;
    return \@Actions;
}

#========================================================================================
# §function     Usage
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       Usage ()
#----------------------------------------------------------------------------------------
# §description  Prints the required parameters of the script
#========================================================================================
sub Usage() {
    my ($Message) = @_;

    print "\n$Message\n\n" if $Message;
    print <<END_USAGE;
Description:
    Given a cartridge with already defined translations files for the actions as well as
    the Unity actions this script moves the translations for already defined actions to
    the related Unity actions. As prerequisite this process has to be done once for a
    specified language.

Usage:
    \$PERL $0 [options] [flags]

options:
    -cart       name of the cartridge for which the Unity actions should be translated
    -pre        language for which the translations are already perpared (default = de)
    -lang       languages that should be translated (comma separated, without spaces),
                unity = de, en, en-US (default = all)

flags:
    -help       show the command line options
    -nooutput   do not print any output
    -all        translate all existing Unity actions (retrieved from the PermissionsUnity.xml)
                (no preparation needed; -lang option will be ignored)

Examples:
    \$PERL $0 -cart Ebay
    \$PERL $0 -cart Ebay -pre en -lang unity -nooutput
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main
);

1;