#========================================================================================
# package      DE_EPAGES::DocdataPay::API::SOAP::Tools
# state        public
#----------------------------------------------------------------------------------------
# description  tools for Soap request to Docdata
#========================================================================================
package DE_EPAGES::DocdataPay::API::SOAP::Tools;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::DocdataPay::API::Constants qw ( $H_MAXIMAL_SOAP_ATTRIBUTE_LENGHT RATE_TO_INTEGER );

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  Constructor
#----------------------------------------------------------------------------------------
# return       $self | self | object
#========================================================================================
sub new {
    my $class = shift;
    my $self = bless {}, $class;
    return $self;
}

#========================================================================================
# function     getAttributesAndTruncateIfNeeded
# state        public
#----------------------------------------------------------------------------------------
# syntax       getAttributesAndTruncateIfNeeded( $DataSorce, $aFieldNames );
#----------------------------------------------------------------------------------------
# description  truncates, if needed, a list of attributes
#----------------------------------------------------------------------------------------
# input        $DataBaseObject | an ePages Database object | object
# input        $aFieldNames | list of attributes names | arrayRef
# return       $hAttributes | truncated attributes | refhash
#========================================================================================
sub getAttributesAndTruncateIfNeeded {
    my $self = shift;
    my ( $DataBaseObject, $aFieldNames ) = @_;
    my $hAttributes = $DataBaseObject->get($aFieldNames);

    foreach my $FieldName ( @{$aFieldNames} ) {
        my $Attribute = $hAttributes->{$FieldName};
        $hAttributes->{$FieldName} = $self->truncateAttribute( $Attribute, $FieldName );
    }

    return $hAttributes;
}

#========================================================================================
# function     truncateAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       truncateAttribute( $Attribute, $FieldName );
#----------------------------------------------------------------------------------------
# description  truncates an Attributes to the maximum allowed length
#----------------------------------------------------------------------------------------
# input        $Attribute | an attribute | String
# input        $aFieldNames | list of attributes names | arrayRef
# return       $TruncatedAttribute | a truncated attribute | String
#========================================================================================
sub truncateAttribute {
    my $self = shift;
    my ( $Attribute, $FieldName ) = @_;
    my $TruncatedAttribute = $Attribute;
    my $MaxLength          = $H_MAXIMAL_SOAP_ATTRIBUTE_LENGHT->{$FieldName};
    my $OriginalLength     = length($Attribute);
    if ( isValid($MaxLength) && $MaxLength <= $OriginalLength ) {
        $TruncatedAttribute = substr( $Attribute, 0, $MaxLength );
    }
    return $TruncatedAttribute;
}

#========================================================================================
# function     convertNumberToDocdataFormat
# state        public
#----------------------------------------------------------------------------------------
# syntax       convertNumberToDocdataFormat($Number);
#----------------------------------------------------------------------------------------
# description  Converts a number to Docdata's format (no separators, two fixed decimals)
#----------------------------------------------------------------------------------------
# input       $Number | Number to convert | Integer
# return       $DocdataNumber | The converted number | Integer
#========================================================================================
sub convertNumberToDocdataFormat {
    my $self = shift;
    my ($Number) = @_;

    if ( ref($Number) ) {
        Error( 'ParameterIsNotANumber',
            { 'Message' => 'The parameter sent to convertNumberToDocdataFormat is not a number' } );
    }

    return sprintf("%.2f", $Number) * RATE_TO_INTEGER;
}

#========================================================================================
# function     removeDashesFromGUID
# state        public
#----------------------------------------------------------------------------------------
# syntax       removeDashesFromGUID($GUID);
#----------------------------------------------------------------------------------------
# description  Removes dashes from a GUID.
#----------------------------------------------------------------------------------------
# input       $GUID         | GUID                | String
# return       $ModifiedGUID | GUID without dashes | String
#========================================================================================
sub removeDashesFromGUID {
    my $self = shift;
    my ($GUID) = @_;

    $GUID =~ tr/-//d;

    return $GUID;
}

#========================================================================================
# function     addDashesToGUID
# state        public
#----------------------------------------------------------------------------------------
# syntax       addDashesToGUID($ModifiedGUID);
#----------------------------------------------------------------------------------------
# description  Adds dashes to a GUID which doesn't have them.
#----------------------------------------------------------------------------------------
# input       $ModifiedGUID | GUID without dashes | String
# return       $GUID         | GUID                | String
#========================================================================================
sub addDashesToGUID {
    my $self = shift;
    my ($ModifiedGUID) = @_;

    unless ( $ModifiedGUID =~ /[A-Fa-f0-9]{32}/ ) {
        Error(
            'WrongDocdataPaymentID',
            {
                'Message' => 'DocdataPaymentID is not an hexadecimal number or its length is different from 32 characters'
            }
        );
    }

    $ModifiedGUID =~ s/(\w{8})(\w{4})(\w{4})(\w{4})(\w{12})/$1-$2-$3-$4-$5/;

    return $ModifiedGUID;
}

1;
