#========================================================================================
# package      DE_EPAGES::Flex::API::Cartridge
# base         DE_EPAGES::Design::API::DesignInstaller
# state        public
#----------------------------------------------------------------------------------------
# description  This is the main cartridge class for install/patch/uninstall.
#========================================================================================
package DE_EPAGES::Flex::API::Cartridge;
use base DE_EPAGES::Design::API::DesignInstaller;

use DE_EPAGES::Object::API::XML::ObjectDeleteHandler qw( DeleteObjects );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::Core::API::File qw( CreateDirectory CopyDirectory RenameDirectory );
use DE_EPAGES::Object::API::Factory qw (LoadRootObject);
use DE_EPAGES::Core::API::PerlTools qw (UsePackage);
use DE_EPAGES::Database::API::Config qw (StoreType);

use strict;

#========================================================================================
# function     new
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer = DE_EPAGES::Flex::API::Cartridge->new(%options);
#----------------------------------------------------------------------------------------
# description  Creates a new installer object of DE_EPAGES::Flex.
#----------------------------------------------------------------------------------------
# input        %options | options for un/install
#               <ul>
#                 <li>IsRecursive - recursive un/installation of required or dependent
#                     cartridges
#               </ul>
#               | hash
# return       $Installer | cartridge installer | object
#========================================================================================
sub new {
    my ($class, %options) = @_;

    my $self = __PACKAGE__->SUPER::new(
        %options,
        'CartridgeDirectory' => 'DE_EPAGES/Flex',
        'Version'            => '7.6.0',
        'Patches'            => ['6.15.1','6.15.2','6.15.3','6.15.4','6.16.0','6.16.1','6.16.2','6.16.3','6.17.0','6.17.1','6.17.2','6.17.3','6.17.4','6.17.5','6.17.6','6.17.7','6.17.8','6.17.9','6.17.9.1','6.17.9.2','6.17.9.3','6.17.10','6.17.11','6.17.12','6.17.12.1','6.17.12.2','6.17.13','6.17.13.1','6.17.13.2','6.17.14','6.17.14.1','6.17.15','6.17.15.1','6.17.16','6.17.17','6.17.17.1','6.17.18','6.17.19','6.17.20','6.17.20.1','6.17.21','6.17.21.1','6.17.22','6.17.23','6.17.24','6.17.24.1','6.17.25','6.17.26','6.17.26.1','6.17.27','6.17.27.1','6.17.28','6.17.28.1','6.17.29','6.17.29.1','6.17.30','6.17.31','6.17.32','6.17.33','6.17.33.1','6.17.34','6.17.34.1','6.17.35','6.17.36','6.17.36.1','6.17.37','6.17.38','6.17.38.1','6.17.39','6.17.39.1','6.17.40','6.17.40.1','6.17.41','6.17.41.1','6.17.42','6.17.43','6.17.43.1','6.17.44','6.17.45','6.17.45.1','6.17.46','6.17.47','6.17.47.1','6.17.48','6.17.48.1','6.17.49','6.17.49.1','6.17.50','6.17.51','6.17.51.1','6.17.52','6.17.53','7.0.0','7.1.0','7.2.0','7.3.0','7.4.0','7.5.0'],
    );
    return bless $self, $class;
}

#========================================================================================
# function     install
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->install;
#----------------------------------------------------------------------------------------
# description  Installs the cartridge.
#========================================================================================
sub install {
    my $self = shift;

    # write the Flex config files to $EPAGES_CONFIG/Flex
    my $SourceConfigDir = $ENV{'EPAGES_CARTRIDGES'}.'/'.$self->cartridgeDirectory.'/Data/Config';
    my $ConfigDir = $ENV{'EPAGES_CONFIG'}.'/Flex';
    CreateDirectory($ConfigDir);
    CopyDirectory( $SourceConfigDir, $ConfigDir, undef , {} );

    $self->startInstall;
    $self->SUPER::install;

    # create /opt/eproot/Shared/Stores/Site/{ShopImport,ShopPublicFiles}
    my $System    = LoadRootObject();
    my $StoreName = $System->get('StoreName');
    my $StoreType = StoreType($StoreName);
    my $Version   = $System->get('EpagesVersion');
    my $StoresSite = "$ENV{EPAGES_STORES}/Site";
    my $StoresStore = "$ENV{EPAGES_STORES}/$StoreName";
    my $StoresStoreIsNew = not -d "$StoresStore/ShopImport";

    my $CartridgePackage = 'DE_EPAGES::ShopConfiguration::API::Cartridge';
    UsePackage($CartridgePackage);
    my $Cartridge = $CartridgePackage->new('StoreName' => $StoreName,
                                           'StoreType' => $StoreType,
                                           'StoreTypeVersion' => $Version);
    $Cartridge->copyPrivateFiles;

    if ($StoresStoreIsNew) {
        RenameDirectory("$StoresStore/$_", "$StoresSite/$_", {create_directory => 1})
            foreach qw(ShopImport ShopPublicFiles);
    } else {
        CopyDirectory("$StoresStore/$_", "$StoresSite/$_", undef, {recursive => 1})
            foreach qw(ShopImport ShopPublicFiles);
    }

    # import store settings
    ImportObjects( $self->xmlDirectory .'/StoreSettings.xml' );

    $self->finishInstall;
    return;
}

#========================================================================================
# function     uninstall
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Installer->uninstall;
#----------------------------------------------------------------------------------------
# description  Uninstalls the cartridge.
#========================================================================================
sub uninstall {
    my $self = shift;

    $self->startUninstall;

    DeleteObjects( $self->xmlDirectory .'/StoreSettings.xml' );

    $self->SUPER::uninstall;
    $self->finishUninstall;
    return;
}

#========================================================================================
# function patch
# state private
#----------------------------------------------------------------------------------------
# syntax $Installer->patch;
#----------------------------------------------------------------------------------------
# description Patches the cartridge.
#========================================================================================
sub patch {
    my $self = shift;

    $self->SUPER::patch;

    # copy private files from shopconfiguration into webroot
    $self->_copyPrivateSiteFiles;

}

#========================================================================================
# function _copyPrivateSiteFiles
# state private
#----------------------------------------------------------------------------------------
# syntax $Installer->_copyPrivateSiteFiles;
#----------------------------------------------------------------------------------------
# description copy private site files of all site cartridges with ShopImport-Data
# to EPAGES_STORES/Site/{ShopImport,ShopPublicFiles} on Store-Shards.
#========================================================================================
sub _copyPrivateSiteFiles {
my $self = shift;

# create $EPAGES_STORES/Site/{ShopImport,ShopPublicFiles}
    my $System    = LoadRootObject();
    my $StoreName = $System->get('StoreName');
    my $StoreType = StoreType($StoreName);
    my $Version   = $System->get('EpagesVersion');
    my $StoresSite = "$ENV{EPAGES_STORES}/Site";

    my $CartridgePackage = 'DE_EPAGES::ShopConfiguration::API::Cartridge';

    # use package
    UsePackage($CartridgePackage);

    # get cartridge installer object
    my $Cartridge = $CartridgePackage->new('StoreName'        => $StoreName,
                                           'StoreType'        => $StoreType,
                                           'StoreTypeVersion' => $Version);

    # copy private site files into webroot
    my $SourceDir = join ('/', $ENV{EPAGES_CARTRIDGES}, $Cartridge->cartridgeDirectory, 'Data/Private');
    $Cartridge->copyDirectory($SourceDir, $StoresSite, 'Shared/Stores', $self->storeName, 1);

    return;
}

1;
