#========================================================================================
# package      DE_EPAGES::Ipayment::API::CGI::AuthOrPreauth
# state        public
# description  Provides the needed request parameters for a auth or preauth CGI request
#========================================================================================
package DE_EPAGES::Ipayment::API::CGI::AuthOrPreauth;

use DE_EPAGES::Ipayment::API::Database::BasketOrOrder;
use DE_EPAGES::Ipayment::API::Tools qw ( PreparePaymentURL );
use DE_EPAGES::Ipayment::API::CGI::BillsafeArticleList;

use strict;



#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new ( );
#----------------------------------------------------------------------------------------
# description  Constructor.
#----------------------------------------------------------------------------------------
# return       self | DE_EPAGES::Ipayment::API::CGI::AuthOrPreauth
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless({}, $class);
    return $self;
}



#========================================================================================
# function     _getDatabaseBasketOrOrder
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getDatabaseBasketOrOrder( );
#----------------------------------------------------------------------------------------
# description  Returns a DatabaseBasketOrOrder
#----------------------------------------------------------------------------------------
# return       A DatabaseBasketOrOrder | DE_EPAGES::Ipayment::API::Database::BasketOrOrder
#========================================================================================
sub _getDatabaseBasketOrOrder {
    my $self = shift;
    my $DatabaseBasketOrOrder = DE_EPAGES::Ipayment::API::Database::BasketOrOrder->new();
    return $DatabaseBasketOrOrder;
}



#========================================================================================
# function     _getBillsafeArticleList
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getBillsafeArticleList( );
#----------------------------------------------------------------------------------------
# description  Returns a BillsafeArticleList
#----------------------------------------------------------------------------------------
# return       A BillsafeArticleList | DE_EPAGES::Ipayment::API::CGI::BillsafeArticleList
#========================================================================================
sub _getBillsafeArticleList {
    my $self = shift;
    my $BillsafeArticleList = DE_EPAGES::Ipayment::API::CGI::BillsafeArticleList->new();
    return $BillsafeArticleList;
}



#========================================================================================
# function     getRequestParameters
# state        public
#----------------------------------------------------------------------------------------
# syntax       getRequestParameters( $BasketOrOrder );
#----------------------------------------------------------------------------------------
# description  Creates Hash with the parameters to be send to ipayment
#----------------------------------------------------------------------------------------
# input        $BasketOrOrder |  Basket or CustomerOrder | object
# return       Parameters used for the auth or preauth CGI request | refhash
#========================================================================================
sub getRequestParameters ($){
    my $self = shift;
    my ($BasketOrOrder) = @_;

    my $DatabaseBasketOrOrder = $self->_getDatabaseBasketOrOrder();
    $DatabaseBasketOrOrder->extractFromBasketOrOrder($BasketOrOrder);

    my $AccountID = $DatabaseBasketOrOrder->getAccountID();
    my $PaymentURL = PreparePaymentURL($AccountID, $self->{'IsInTestMode'});

    my $hCountry = $DatabaseBasketOrOrder->getBillingAddress()->get('Country');
    my $hHiddenInputs = {
        'redirect_action' => 'post',
        'advanced_strict_id_check' => 'true',
        'shopper_id' => $DatabaseBasketOrOrder->getGUID(),
        'addr_name' => $DatabaseBasketOrOrder->getBillingAddress()->get('FullName'),
        'addr_street' => $DatabaseBasketOrOrder->getBillingAddress()->get('Street'),
        'addr_zip' => $DatabaseBasketOrOrder->getBillingAddress()->get('Zipcode'),
        'addr_city' => $DatabaseBasketOrOrder->getBillingAddress()->get('City'),
        'addr_country' => $hCountry->{'Code2'},
        'addr_email' => $DatabaseBasketOrOrder->getBillingAddress()->get('EMail'),
        'trx_paymenttyp' => $DatabaseBasketOrOrder->getPaymentType(),
        'trx_typ' => $DatabaseBasketOrOrder->getTransactionType(),
        'client_name' => 'ePages',
        'client_version' => $DatabaseBasketOrOrder->getEpagesVersion(),
        'commission_partner_code' => $DatabaseBasketOrOrder->getPartnerId(),
        'backlink' => $DatabaseBasketOrOrder->getCancelRedirectUrl(),
    };

    if($DatabaseBasketOrOrder->getPaymentType() eq 'cc') {
        $hHiddenInputs->{'cc_typ'} = $DatabaseBasketOrOrder->getPaymentTypeName();
    }

    if($DatabaseBasketOrOrder->getPaymentType() eq 'paypal') {
        $hHiddenInputs->{'silent'} = 'true';
        $hHiddenInputs->{'paypal_flow'} = 'ECM';
        $hHiddenInputs->{'return_paymentdata_details'} = 'true';
    }

    if($DatabaseBasketOrOrder->getPaymentType() eq 'invoice') {
        my $BillsafeArticleList = $self->_getBillsafeArticleList();

        $hHiddenInputs->{'silent'} = 'true'; #TODO: Why Silent if you will be redirected to BillSAFE?!
        $hHiddenInputs->{'account_id'} = $DatabaseBasketOrOrder->getAccountID();
        $hHiddenInputs->{'trxuser_id'} = $DatabaseBasketOrOrder->getUserID();
        $hHiddenInputs->{'trxpassword'} = $DatabaseBasketOrOrder->getUserPassword();
        $hHiddenInputs->{'trx_amount'} = $DatabaseBasketOrOrder->getGrandTotalInMinorUnits();
        $hHiddenInputs->{'trx_currency'} = $DatabaseBasketOrOrder->getCurrencyID();
        $hHiddenInputs->{'redirect_url'} = $DatabaseBasketOrOrder->getRedirectUrl();
        $hHiddenInputs->{'hidden_trigger_url'} = $DatabaseBasketOrOrder->getHiddenTriggerUrl();
        $hHiddenInputs->{'invoice_text'} = $BillsafeArticleList->extractArticleList($BasketOrOrder);
    }

    if ($DatabaseBasketOrOrder->getPaymentType() eq 'pp') {
        $hHiddenInputs->{'redirect_url'} = $DatabaseBasketOrOrder->getRedirectUrl();
    }

    # It seems that for BillSAFE pregenerated sessions do not work!!!
    # So for BillSAFE we have to send the Credentials and the Amount via CGI!
    # (Then we have to use HMAC for security reasons too...)
    if($DatabaseBasketOrOrder->getPaymentType() ne 'invoice') {
        $hHiddenInputs->{'ipayment_session_id'} = $DatabaseBasketOrOrder->getSessionID();
    }

    my $hFormData = {
        'PaymentUrl' => $PaymentURL,
        'HiddenInputs' => $hHiddenInputs,
    };

    return $hFormData;
}



1;