#========================================================================================
# package      package DE_EPAGES::Ipayment::API::Database::BasketOrOrder
# state        public
#----------------------------------------------------------------------------------------
# description  Database Wrapper for BasketOrOrder Object
#========================================================================================
package DE_EPAGES::Ipayment::API::Database::BasketOrOrder;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::Ipayment::API::Constants qw (
    HOURS_PER_DAY
    MINUTES_PER_HOUR
    SECONDS_PER_MINUTE
);
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Object::API::Table::Currency qw ( GetInfoCurrency );
use DE_EPAGES::Shop::API::Url qw ( BuildShopUrl );

use strict;



#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       new( );
#----------------------------------------------------------------------------------------
# description  Constructor
#----------------------------------------------------------------------------------------
# return       $self | self | DE_EPAGES::Ipayment::API::Database::BasketOrOrder
#========================================================================================
sub new {
    my $class = shift;
    my $self  = bless {}, $class;
    return $self;
}



#========================================================================================
# function     extractFromBasketOrOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       extractFromBasketOrOrder( $BasketOrOrder );
#----------------------------------------------------------------------------------------
# description  Extracts the attribute values from the BasketOrOrder object
#----------------------------------------------------------------------------------------
# input        $BasketOrOrder | BasketOrOrder | Object
# return       $WasSuccessful | Status | boolean
#========================================================================================
sub extractFromBasketOrOrder {
    my $self = shift;
    my ($BasketOrOrder) = @_;

    my $WasSuccessful = $self->_getDatabaseObjects($BasketOrOrder);
    return $WasSuccessful;
}



#========================================================================================
# function     _getDatabaseObjects
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getDatabaseObjects( $BasketOrOrder );
#----------------------------------------------------------------------------------------
# description  Checks if needed attributes exists and stores it in the object
#----------------------------------------------------------------------------------------
# input        $BasketOrOrder | BasketOrOrder | Object
# return       $WasSuccessful | Status | boolean
#========================================================================================
sub _getDatabaseObjects {
    my $self = shift;
    my ($BasketOrOrder) = @_;

    return 0 unless isValid($BasketOrOrder);
    $self->{'Order'} = $BasketOrOrder;

    return 0 unless $BasketOrOrder->can('getSite');
    $self->{'Shop'} = $BasketOrOrder->getSite();
    return 0 unless $self->_isValidObjectOfClass( $self->{'Shop'}, 'DE_EPAGES::Shop::API::Object::Shop' );

    return 0 unless $BasketOrOrder->can('container');
    $self->{'LineItemContainer'} = $BasketOrOrder->container();
    return 0 unless $self->_isValidObjectOfClass( $self->{'LineItemContainer'}, 'DE_EPAGES::Order::API::Object::LineItemContainer' );

    $self->{'Payment'} = $self->{'LineItemContainer'}->get('Payment');
    return 0 unless $self->_isValidObjectOfClass($self->{'Payment'}, 'DE_EPAGES::Ipayment::API::Object::LineItemPaymentIpayment');

    $self->{'PaymentMethod'} = $self->{'Payment'}->get('PaymentMethod');
    return 0 unless $self->_isValidObjectOfClass($self->{'PaymentMethod'}, 'DE_EPAGES::Ipayment::API::Object::PaymentMethodIpayment');

    $self->{'PaymentTypeMethod'} = $self->{'PaymentMethod'}->get('PaymentTypeMethod');
    return 0 unless $self->_isValidObjectOfClass($self->{'PaymentTypeMethod'}, 'DE_EPAGES::ExternalPayment::API::Object::PaymentTypeMethod');

    $self->{'BillingAddress'} = $BasketOrOrder->get('BillingAddress');
    return 0 unless $self->_isValidObjectOfClass($self->{'BillingAddress'}, 'DE_EPAGES::Address::API::Object::Address');

    $self->{'CancelRedirectUrl'} = $self->_BuildShopUrl(
        $BasketOrOrder,
        {'ViewAction' => 'ViewPaymentIpaymentCancel'},
        {'Type' => 'sf/mobile', 'UseSSL' => 1},
    );

    $self->{'RedirectUrl'} = $self->_BuildShopUrl(
        $self->{'Shop'},
        {'ChangeAction' => 'Redirect'},
        {'Type' => 'sf', 'UseSSL' => 1,}
     );

     $self->{'SilentErrorUrl'} = $self->_BuildShopUrl(
         $BasketOrOrder,
         {'ViewAction' => 'ViewPaymentIpaymentCancel'},
         {'Type' => 'sf/mobile', 'UseSSL' => 1},
     );

    $self->{'HiddenTriggerUrl'} = $self->_BuildShopUrl(
        $self->{'Shop'},
        {'ChangeAction' => 'HiddenTrigger'},
        {'Type' => 'sf', 'UseSSL' => 1,}
     );

    return 1;
}



#========================================================================================
# function     _isValidObjectOfClass
# state        private
#----------------------------------------------------------------------------------------
# syntax       _isValidObjectOfClass( $Object, $ExpectedClass );
#----------------------------------------------------------------------------------------
# description  Tests if the given object is in the given package.
#----------------------------------------------------------------------------------------
# input        $Object | object of arbitrary class | object
# input        $ExpectedClass | string with package name | String
# return       $IsObjectOfClass | Is Object Of Class | boolean
#========================================================================================
sub _isValidObjectOfClass {
    my $self = shift;
    my ($Object, $ExpectedClass) = @_;
    my $IsObjectOfClass = 1;
    eval {
        TestObject('DatabaseBasketOrOrder: isValidObjectOfClass', $Object, $ExpectedClass);
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        my $ErrorString = $Error->code();
        if ($ErrorString =~ /WRONG_CLASS/ ||
            $ErrorString =~ /PARAMETER_NO_OBJECT/ ||
            $ErrorString =~ /PARAMETER_MISSING/ ||
            $ErrorString =~ /PARAMETER_NO_OBJECT/
            ) {
            $IsObjectOfClass = 0;
        } else {
            Error($Error);
        }
    }
    return $IsObjectOfClass;
}



#========================================================================================
# function     getAccountID
# state        public
#----------------------------------------------------------------------------------------
# syntax       getAccountID( );
#----------------------------------------------------------------------------------------
# description  Returns AccountID
#----------------------------------------------------------------------------------------
# return       $AccountID | AccountID | String
#========================================================================================
sub getAccountID {
    my $self = shift;
    return $self->{'PaymentMethod'}->get('AccountID');
}



#========================================================================================
# function     getUserID
# state        public
#----------------------------------------------------------------------------------------
# syntax       getUserID( );
#----------------------------------------------------------------------------------------
# description  Returns UserID
#----------------------------------------------------------------------------------------
# return       $UserID | UserID | String
#========================================================================================
sub getUserID {
    my $self = shift;
    return $self->{'PaymentMethod'}->get('UserID');
}



#========================================================================================
# function     getUserPassword
# state        public
#----------------------------------------------------------------------------------------
# syntax       getUserPassword( );
#----------------------------------------------------------------------------------------
# description  Returns UserPassword
#----------------------------------------------------------------------------------------
# return       $UserPassword | UserPassword | String
#========================================================================================
sub getUserPassword {
    my $self = shift;
    return $self->{'PaymentMethod'}->get('UserPassword');
}



#========================================================================================
# function     getAdminActionPassword
# state        public
#----------------------------------------------------------------------------------------
# syntax       getAdminActionPassword( );
#----------------------------------------------------------------------------------------
# description  Returns AdminActionPassword
#----------------------------------------------------------------------------------------
# return       $AdminActionPassword | AdminActionPassword | String
#========================================================================================
sub getAdminActionPassword {
    my $self = shift;
    return $self->{'PaymentMethod'}->get('AdminActionPassword');
}

#========================================================================================
# function     getPreAuthTimePeriodInSeconds
# state        public
#----------------------------------------------------------------------------------------
# syntax       getPreAuthTimePeriodInSeconds( );
#----------------------------------------------------------------------------------------
# description  Returns PreAuthTimePeriod in seconds
#----------------------------------------------------------------------------------------
# return       $PreAuthTimePeriod | pre auth time period in seconds | Interger
#========================================================================================
sub getPreAuthTimePeriodInSeconds {
    my $self = shift;
    my $Factor_DayToSeconds = (HOURS_PER_DAY * MINUTES_PER_HOUR * SECONDS_PER_MINUTE);
    my $TimePeriodInDays = $self->{'PaymentMethod'}->get('PreAuthTimePeriodInDays');
    my $TimePeriodInSeconds = $TimePeriodInDays * $Factor_DayToSeconds;
    return $TimePeriodInSeconds;
}

#========================================================================================
# function     getCurrencyID
# state        public
#----------------------------------------------------------------------------------------
# syntax       getCurrencyID( );
#----------------------------------------------------------------------------------------
# description  Returns CurrencyID
#----------------------------------------------------------------------------------------
# return       $CurrencyID | CurrencyID | String
#========================================================================================
sub getCurrencyID {
    my $self = shift;
    return $self->{'LineItemContainer'}->get('CurrencyID');
}



#========================================================================================
# function     getMinorUnitDigitCount
# state        publi
#----------------------------------------------------------------------------------------
# syntax       getMinorUnitDigitCount( );
#----------------------------------------------------------------------------------------
# description  Returns the count of minor units used in the currency of the order
#----------------------------------------------------------------------------------------
# return       Count of minor units for used currency | integer
#========================================================================================
sub getMinorUnitDigitCount {
    my $self = shift;
    my $MinorUnitDigitCount = $self->_getMinorUnitDigitCount($self->getCurrencyID());
    return $MinorUnitDigitCount;
}



#========================================================================================
# function     getGrandTotal
# state        public
#----------------------------------------------------------------------------------------
# syntax       getGrandTotal( );
#----------------------------------------------------------------------------------------
# description  Returns GrandTotal
#----------------------------------------------------------------------------------------
# return       $GrandTotal | GrandTotal | float
#========================================================================================
sub getGrandTotal {
    my $self = shift;
    return $self->{'LineItemContainer'}->get('GrandTotal');
}

#========================================================================================
# function     getTransactionType
# state        public
#----------------------------------------------------------------------------------------
# syntax       getTransactionType( );
#----------------------------------------------------------------------------------------
# description  Returns TransactionType
#----------------------------------------------------------------------------------------
# return       $TransactionType | TransactionType | String
#========================================================================================
sub getTransactionType {
    my $self = shift;
    return $self->{'PaymentMethod'}->get('SelectedTransactionType');
}



#========================================================================================
# function     getPaymentType
# state        public
#----------------------------------------------------------------------------------------
# syntax       getPaymentType( );
#----------------------------------------------------------------------------------------
# description  Returns PaymentType
#----------------------------------------------------------------------------------------
# return       $PaymentType | PaymentType | String
#========================================================================================
sub getPaymentType {
    my $self = shift;
    return $self->{'PaymentTypeMethod'}->get('ProductType');
}



#========================================================================================
# function     getPaymentTypeName
# state        public
#----------------------------------------------------------------------------------------
# syntax       getPaymentTypeName( );
#----------------------------------------------------------------------------------------
# description  Returns PaymentType Name (Name of Credit Card)
#----------------------------------------------------------------------------------------
# return       PaymentTypeName | String
#========================================================================================
sub getPaymentTypeName {
    my $self = shift;
    return $self->{'PaymentTypeMethod'}->alias();
}



#========================================================================================
# function     getIsTestTransaction
# state        public
#----------------------------------------------------------------------------------------
# syntax       getIsTestTransaction( );
#----------------------------------------------------------------------------------------
# description  Returns IsTestTransaction
#----------------------------------------------------------------------------------------
# return       $IsTestTransaction | IsTestTransaction | boolean
#========================================================================================
sub getIsTestTransaction {
    my $self = shift;
    return $self->{'Payment'}->get('IsTestTransaction');
}



#========================================================================================
# function     getLineItemContainer
# state        public
#----------------------------------------------------------------------------------------
# syntax       getLineItemContainer( );
#----------------------------------------------------------------------------------------
# description  Returns the LineItemContainer of the order
#----------------------------------------------------------------------------------------
# return       The LineItemContainer of the order | LineItemContainer
#========================================================================================
sub getLineItemContainer {
    my $self = shift;
    return $self->{'LineItemContainer'};
}



#========================================================================================
# function     getLanguageID
# state        public
#----------------------------------------------------------------------------------------
# syntax       getLanguageID( );
#----------------------------------------------------------------------------------------
# description  Returns the LanguageID of the language used in the order
#----------------------------------------------------------------------------------------
# return       LanguageID of the order | integer
#========================================================================================
sub getLanguageID {
    my $self = shift;
    return $self->{'Order'}->get('User')->get('LanguageID');
}



#========================================================================================
# function     getLineItemPaymentIpayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       getLineItemPaymentIpayment( );
#----------------------------------------------------------------------------------------
# description  Returns LineItemPaymentIpayment
#----------------------------------------------------------------------------------------
# return       $LineItemPaymentIpayment | LineItemPaymentIpayment | DE_EPAGES::Ipayment::API::Object::LineItemPaymentIpayment
#========================================================================================
sub getLineItemPaymentIpayment {
    my $self = shift;
    return $self->{'Payment'};
}



#========================================================================================
# function     getRedirectUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       getRedirectUrl( );
#----------------------------------------------------------------------------------------
# description  get the redirect URL
#----------------------------------------------------------------------------------------
# return       $RedirectUrl | redirect URL | String
#========================================================================================
sub getRedirectUrl {
    my $self = shift;
    return $self->{'RedirectUrl'};
}

#========================================================================================
# function     getCancelRedirectUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       getCancelRedirectUrl( );
#----------------------------------------------------------------------------------------
# description  Gets 'Cancel' redirect URL (backlink).
#----------------------------------------------------------------------------------------
# return       $CancelRedirectUrl | Cancel redirect URL | String
#========================================================================================
sub getCancelRedirectUrl {
    my $self = shift;
    return $self->{'CancelRedirectUrl'};
}



#========================================================================================
# function     getSilentErrorUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       getSilentErrorUrl( );
#----------------------------------------------------------------------------------------
# description  get the silent error Url
#----------------------------------------------------------------------------------------
# return       $RedirectUrl | silent error Url | String
#========================================================================================
sub getSilentErrorUrl {
    my $self = shift;
    return $self->{'SilentErrorUrl'};
}



#========================================================================================
# function     getHiddenTriggerUrl
# state        public
#----------------------------------------------------------------------------------------
# syntax       getHiddenTriggerUrl( );
#----------------------------------------------------------------------------------------
# description  get the hidden trigger Url
#----------------------------------------------------------------------------------------
# return       $RedirectUrl | get hidden trigger Url | String
#========================================================================================
sub getHiddenTriggerUrl {
    my $self = shift;
    return $self->{'HiddenTriggerUrl'};
}



#========================================================================================
# function     getGUID
# state        public
#----------------------------------------------------------------------------------------
# syntax       getGUID( );
#----------------------------------------------------------------------------------------
# description  get the GUID of LineItemPaymentIPayment
#----------------------------------------------------------------------------------------
# return       $GUID | GUID of LineItemPaymentIPayment | String
#========================================================================================
sub getGUID {
    my $self = shift;
    return $self->{'Payment'}->guid();
}



#========================================================================================
# function     getBillingAddress
# state        public
#----------------------------------------------------------------------------------------
# syntax       getBillingAddress( );
#----------------------------------------------------------------------------------------
# description  get the Billing address of the BasketOrOrder
#----------------------------------------------------------------------------------------
# return       $BillingAdress | BillingAdress | DE_EPAGES::Address::API::Object::Address
#========================================================================================
sub getBillingAddress {
    my $self = shift;
    return $self->{'BillingAddress'};
}



#========================================================================================
# function     getGrandTotalInMinorUnits
# state        public
#----------------------------------------------------------------------------------------
# syntax       getGrandTotalInMinorUnits( );
#----------------------------------------------------------------------------------------
# description  Returns the total amount of the order in the minor unit
#               e.g. in Cents instead of EUR
#----------------------------------------------------------------------------------------
# return       Total Order amount in the minor unit | integer
#========================================================================================
sub getGrandTotalInMinorUnits {
    my $self = shift;
    my $MinorUnitDigitCount = $self->_getMinorUnitDigitCount($self->getCurrencyID());
    my $TotalAmountInMinorUnits = $self->getGrandTotal() * (10 ** $MinorUnitDigitCount); ## no critic (ProhibitMagicNumbers)
    return $TotalAmountInMinorUnits;
}



#========================================================================================
# function     _BuildShopUrl
# state        private
#----------------------------------------------------------------------------------------
# syntax       _BuildShopUrl( );
#----------------------------------------------------------------------------------------
# description  wrap to hide core bindings
#----------------------------------------------------------------------------------------
# input        $Object | any object that belongs to a shop | object
# input        $hParams | url get parameters; values may be strings or arrays of strings | ref.hash
# input        $hOptions | h options | ref.hash
# input        $Servlet | the current servlet (optional) | object
# return       BuildShopUrl | shop URL | String
#========================================================================================
sub _BuildShopUrl {
    my $self = shift;
    my ($Object, $hParams, $hOptions, $Servlet) = @_;
    return BuildShopUrl($Object, $hParams, $hOptions, $Servlet);
}



#========================================================================================
# function     _getMinorUnitDigitCount
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getMinorUnitDigitCount( $CurrencyID );
#----------------------------------------------------------------------------------------
# description  Returns the number of digits of the minor unit of a currency
#----------------------------------------------------------------------------------------
# input        $CurrencyID | CurrencyID | String
# return       $MinorUnitDigitCount | minor Unit digit count | integer
#========================================================================================
sub _getMinorUnitDigitCount {
    my $self = shift;
    my ($CurrencyID) = @_;
    return GetInfoCurrency($CurrencyID)->{'MinorUnit'};
}



#========================================================================================
# function     getSessionID
# state        public
#----------------------------------------------------------------------------------------
# syntax       getSessionID( );
#----------------------------------------------------------------------------------------
# description  Returns the SessionID
#----------------------------------------------------------------------------------------
# return       SessionID | String
#========================================================================================
sub getSessionID {
    my $self = shift;
    my $LineItemPaymentIpayment = $self->{'Payment'};
    my $SessionId = $LineItemPaymentIpayment->get('SessionID');
    return $SessionId;
}

#========================================================================================
# function     getSelectedTransactionType
# state        public
#----------------------------------------------------------------------------------------
# syntax       getSelectedTransactionType( );
#----------------------------------------------------------------------------------------
# description  Returns the Selected Transaction Type
#----------------------------------------------------------------------------------------
# return       $SelectedTransactionType | String
#========================================================================================
sub getSelectedTransactionType {
    my $self = shift;
    my $LineItemPaymentIpayment = $self->{'Payment'};
    my $SelectedTransactionType = $LineItemPaymentIpayment->get('SelectedTransactionType');
    return $SelectedTransactionType;
}

#========================================================================================
# function     getTransID
# state        public
#----------------------------------------------------------------------------------------
# syntax       getTransID( );
#----------------------------------------------------------------------------------------
# description  get the TransID of LineItemPaymentIPayment
#----------------------------------------------------------------------------------------
# return       $TransID | TransID of LineItemPaymentIPayment | String
#========================================================================================
sub getTransID {
    my $self = shift;
    return $self->{'Payment'}->get('TransID');
}



#========================================================================================
# function     getTransStatus
# state        public
#----------------------------------------------------------------------------------------
# syntax       getTransStatus( );
#----------------------------------------------------------------------------------------
# description  get the TransStatus of LineItemPaymentIPayment
#----------------------------------------------------------------------------------------
# return       $TransStatus | TransStatus of LineItemPaymentIPayment | String
#========================================================================================
sub getTransStatus {
    my $self = shift;
    return $self->{'Payment'}->get('TransStatus');
}

#========================================================================================
# function     getTransTime
# state        public
#----------------------------------------------------------------------------------------
# syntax       getTransTime( );
#----------------------------------------------------------------------------------------
# description  get the transaction time from line item payment
#----------------------------------------------------------------------------------------
# return       $TransTime | transaction time from line item payment | String
#========================================================================================
sub getTransTime {
    my $self = shift;
    return $self->{'Payment'}->get('TransTime');
}

#========================================================================================
# function     isInTestMode
# state        public
#----------------------------------------------------------------------------------------
# syntax       isInTestMode( );
#----------------------------------------------------------------------------------------
# description  Checks if the PaymentMethod is in test mode
#----------------------------------------------------------------------------------------
# return       $IsInTestMode | Boolean for test mode | Boolean
#========================================================================================
sub isInTestMode {
    my $self = shift;
    return $self->{'PaymentMethod'}->get('IsInTestMode');
}



#========================================================================================
# function     getShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       getShop( );
#----------------------------------------------------------------------------------------
# description  Returns the Shop object of the order
#----------------------------------------------------------------------------------------
# return       $Shop | Shop object | Shop
#========================================================================================
sub getShop {
    my $self = shift;
    return $self->{'Shop'};
}



#========================================================================================
# function     getPartnerId
# state        public
#----------------------------------------------------------------------------------------
# syntax       getPartnerId( );
#----------------------------------------------------------------------------------------
# description  Returns the ipayment partner ID of ePages
#----------------------------------------------------------------------------------------
# return       partner ID | String
#========================================================================================
sub getPartnerId {
    my $self = shift;
    my $System = $self->_LoadRootObject();
    my $PartnerId = $System->get('Ipayment_PartnerId');
    return $PartnerId;
}



#========================================================================================
# function     getEpagesVersion
# state        public
#----------------------------------------------------------------------------------------
# syntax       getEpagesVersion( );
#----------------------------------------------------------------------------------------
# description  Returns the installed ePages version
#----------------------------------------------------------------------------------------
# return       ePages Version | String
#========================================================================================
sub getEpagesVersion {
    my $self = shift;
    my $System = $self->_LoadRootObject();
    my $EpagesVersion = $System->get('EpagesVersion');
    return $EpagesVersion;
}



#========================================================================================
# function     LoadRootObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object = LoadRootObject();
#----------------------------------------------------------------------------------------
# description  Wrapper for core bindings
#----------------------------------------------------------------------------------------
# return       root object | object
#========================================================================================
sub _LoadRootObject {
    my $self = shift;
    return LoadRootObject();
}

1;