#==============================================================================
# package      DE_EPAGES::Ipayment::UI::BasketOrder
# base         DE_EPAGES::Presentation::UI::Object;
# state        private
#==============================================================================
package DE_EPAGES::Ipayment::UI::BasketOrder;
use base DE_EPAGES::Presentation::UI::Object;

use DE_EPAGES::Core::API::Log qw ( GetLog LogContext );
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime isValid );
use DE_EPAGES::Ipayment::API::CGI::AuthOrPreauth;
use DE_EPAGES::Ipayment::API::Database::BasketOrOrder;
use DE_EPAGES::Ipayment::API::Logger;
use DE_EPAGES::Ipayment::API::Payment qw ( SendPaymentRequest UpdateTransactionStatus );
use DE_EPAGES::Object::API::Factory qw ( ExistsObjectByGUID LoadObjectByGUID );
use DE_EPAGES::Order::API::BasketOrderValidator qw ( IsAValidBasketOrOrder );
use DE_EPAGES::Order::API::Object::BasketOrderProcess;

use strict;

#========================================================================================
# function     ViewPaymentIpaymentCancel
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->ViewPaymentIpaymentCancel($Servlet);
#----------------------------------------------------------------------------------------
# description  Handles Ipayment cancel backlinks.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | hashref
#========================================================================================
sub ViewPaymentIpaymentCancel {
    my $self = shift;
    my ($Servlet) = @_;

    $self->BlockMonitor($Servlet);

    my $BasketOrOrder = $Servlet->object();
    my $EntryPoint = 'ViewPaymentIpaymentCancel';
    $self->_LogContext($BasketOrOrder, $EntryPoint, sub {
        my $hFormParams = $Servlet->params();
        $self->_LogPayment('hFormParams', $hFormParams);
        if(!IsAValidBasketOrOrder($BasketOrOrder)) {
            GetLog->warn('This backlink expects a basket or an order. Maybe someone tried to manually call it.');
            return;
        }
        my $LineItemPaymentGUID = $hFormParams->{'shopper_id'};
        if($BasketOrOrder->instanceOf('CustomerOrder')) {
            $self->_setStatusToCancelled($BasketOrOrder);
        }
        elsif(isValid($LineItemPaymentGUID)) {
            $BasketOrOrder = $self->_setRelatedOrderToCancelled($LineItemPaymentGUID);
            if($BasketOrOrder->instanceOf('CustomerOrder')) {
                $Servlet->vars('RELATED_ORDER_ID', $BasketOrOrder->id());
            }
        }
        my $Shop = $BasketOrOrder->getSite();
        if($Shop->isUnity()) {
            $self->_setUnityPaymentErrorCodeToCancelled($Servlet, $BasketOrOrder);
        }
        return;
    });

    return;
}

#========================================================================================
# function     ViewPaymentIpayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewPaymentIpayment( $Servlet );
#----------------------------------------------------------------------------------------
# description  view action for classical checkout
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# return       $self | self | DE_EPAGES::Ipayment::UI::BasketOrder
#========================================================================================
sub ViewPaymentIpayment {
    my $self = shift;
    my ($Servlet) = @_;

    my $BasketOrCustomerorder  = $Servlet->object;
    my $LineItemPayment = $BasketOrCustomerorder->get('LineItemContainer')->get('Payment');
    my $PaymentMethod   = $LineItemPayment->get('PaymentMethod');
    DE_EPAGES::Ipayment::API::Logger->instance()->setSiteAliasWithObject($PaymentMethod);


    $self->{'GUID'} = $LineItemPayment->get('GUID');
    $self->{'ShopObject'} = $BasketOrCustomerorder->getSite();
    $self->{'IsInTestMode'} = $PaymentMethod->get('IsInTestMode');
    $self->{'IsIn3dSecureMode'} = $PaymentMethod->get('IsIn3dSecureMode');

    # start payment
    SendPaymentRequest();

    $self->_SetLineItemPayment($LineItemPayment);
    $self->_PrepareServlet($Servlet);
    return $self;
}



#========================================================================================
# function     _SetLineItemPayment
# state        private
#----------------------------------------------------------------------------------------
# syntax       _SetLineItemPayment( $LineItemPayment );
#----------------------------------------------------------------------------------------
# description  it sets the current status into the LineItemPayment
#----------------------------------------------------------------------------------------
# input        $LineItemPayment | Payment LineItem | object
#========================================================================================
sub _SetLineItemPayment {
    my $self = shift;
    my ($LineItemPayment) = @_;

    $LineItemPayment->set( {
            'IsTestTransaction' => $self->{'IsInTestMode'},
            'TransID' => $self->{'GUID'},
            'TransTime' => GetCurrentDateTime(),
         } );
    return;
}



#========================================================================================
# function     _PrepareServlet
# state        private
#----------------------------------------------------------------------------------------
# syntax       _PrepareServlet( $Servlet );
#----------------------------------------------------------------------------------------
# description  it extends the servlet with all needed parameters
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub _PrepareServlet {
    my $self = shift;
    my ($Servlet) = @_;

    $Servlet->vars('IsTestTransaction', $self->{'IsInTestMode'});

    my $BasketOrOrder = $Servlet->object();

    my $AuthOrPreauth = DE_EPAGES::Ipayment::API::CGI::AuthOrPreauth->new();
    my $ReturnParams = $AuthOrPreauth->getRequestParameters($BasketOrOrder);

    $Servlet->vars('HiddenInputs', $ReturnParams->{'HiddenInputs'});
    $Servlet->vars('PaymentUrl', $ReturnParams->{'PaymentUrl'});
    $Servlet->vars('IsIn3dSecureMode', $self->{'IsIn3dSecureMode'});

    return;
}



#========================================================================================
# function     ViewMultiPaymentIpayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewMultiPaymentIpayment( $Servlet );
#----------------------------------------------------------------------------------------
# description  view action for multi step checkout
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# return       $self | self | DE_EPAGES::Ipayment::UI::BasketOrder
#========================================================================================
sub ViewMultiPaymentIpayment {
    my $self = shift;
    my ($Servlet) = @_;

    my $BasketOrderProcess = DE_EPAGES::Order::API::Object::BasketOrderProcess->new($Servlet);
    $Servlet->vars('BasketOrderProcess', $BasketOrderProcess->tleHash);

    $self->ViewPaymentIpayment($Servlet);
    return $self;
}

#========================================================================================
# function     _LogContext
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_LogContext($BasketOrOrder, $EntryPoint, $FunctionPointer);
#----------------------------------------------------------------------------------------
# description  Calls LogContext with a predefined context.
#----------------------------------------------------------------------------------------
# input        $BasketOrOrder   | Basket or order                       | object
# input        $EntryPoint      | Method which started the related flow | string
# input        $FunctionPointer | Function pointer
#========================================================================================
sub _LogContext {
    shift;
    my ($BasketOrOrder, $EntryPoint, $FunctionPointer) = @_;

    my $Shop = $BasketOrOrder->getSite();

    LogContext({
        'BasketOrOrder ID' => $BasketOrOrder->id(),
        'BasketOrOrder Alias' => $BasketOrOrder->alias(),
        'Shop ID' => $Shop->id(),
        'Shop Alias' => $Shop->alias(),
        'EntryPoint' => $EntryPoint
        }, $FunctionPointer);

    return;
}

#========================================================================================
# function     _LogPayment
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_LogPayment($Name, $VariableToPrint);
#----------------------------------------------------------------------------------------
# description  Logs something into communication log.
#----------------------------------------------------------------------------------------
# input       $Name            | Name of what will be logged | string
# input       $VariableToPrint | Structure that will be printed
#========================================================================================
sub _LogPayment {
    shift;
    my ($Name, $VariableToPrint) = @_;

    my $Logger = DE_EPAGES::Ipayment::API::Logger->instance();

    $Logger->logPayment('Ipayment', $Name, $VariableToPrint);

    return;
}

#========================================================================================
# function     _setUnityPaymentErrorCodeToCancelled
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_setUnityPaymentErrorCodeToCancelled($Servlet, $BasketOrOrder);
#----------------------------------------------------------------------------------------
# description  Sets field PaymentErrorCode to 'cancelled' and then tells the Servlet
#               where to redirect.
#----------------------------------------------------------------------------------------
# input        $Servlet       | Servlet         | hashref
# input        $BasketOrOrder | Basket or order | object
#========================================================================================
sub _setUnityPaymentErrorCodeToCancelled {
    shift;
    my ($Servlet, $BasketOrOrder) = @_;

    my $LineItemPayment = $BasketOrOrder->container()->get('Payment');
    $LineItemPayment->set({'PaymentErrorCode' => 'Canceled'});

    $Servlet->redirect($BasketOrOrder->get('WebUrlUnitySSL'));

    return;
}

#========================================================================================
# function     _setStatusToCancelled
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_setStatusToCancelled($BasketOrOrder);
#----------------------------------------------------------------------------------------
# description  Sets an order and its related transaction status to cancelled.
#----------------------------------------------------------------------------------------
# input        $BasketOrOrder | Basket or order | object
#========================================================================================
sub _setStatusToCancelled {
    shift;
    my ($BasketOrOrder) = @_;

    my $LineItemPayment = $BasketOrOrder->container()->get('Payment');
    my $hTransactionParams = UpdateTransactionStatus($LineItemPayment, 'STATUS_CANCELLED');
    $BasketOrOrder->set( {'CancelledOn' => $hTransactionParams->{'TransTime'} } );

    return;
}

#========================================================================================
# function     _setRelatedOrderToCancelled
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_setRelatedOrderToCancelled($LineItemPaymentGUID);
#----------------------------------------------------------------------------------------
# description  Given a LineItemPayment GUID, gets related order and sets it to
#               cancelled.
#----------------------------------------------------------------------------------------
# input        $LineItemPaymentGUID | LineItemPayment GUID | string
# return       $BasketOrOrder | Basket or order | object
#========================================================================================
sub _setRelatedOrderToCancelled {
    my $self = shift;
    my ($LineItemPaymentGUID) = @_;

    my $LineItemPayment;
    if (ExistsObjectByGUID($LineItemPaymentGUID)) {
        $LineItemPayment = LoadObjectByGUID($LineItemPaymentGUID);
    }
    else {
        GetLog->warn('LineItemPayment does not exist.');
        return;
    }
    my $BasketOrOrder = $LineItemPayment->container()->parent();
    return $BasketOrOrder unless $BasketOrOrder->instanceOf('CustomerOrder');

    $self->_setStatusToCancelled($BasketOrOrder);

    return $BasketOrOrder;
}


1;