#========================================================================================
# package      DE_EPAGES::Ipayment::UI::Shop
# state        public
# description  UI class
#========================================================================================
package DE_EPAGES::Ipayment::UI::Shop;
use base DE_EPAGES::Order::UI::Shop;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Ipayment::API::CGI::Response;
use DE_EPAGES::Ipayment::API::CGI::SecurityLayer;
use DE_EPAGES::Ipayment::API::Logger;
use DE_EPAGES::Object::API::Factory qw ( ExistsObjectByGUID LoadObjectByGUID );
use DE_EPAGES::Order::UI::Basket;

use strict;

use constant ERROR_VIEWACTION_NAME => 'ViewPaymentIpaymentResult';



#========================================================================================
# function     TransactionPaymentName
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Name = $Handler->TransactionPaymentName();
#----------------------------------------------------------------------------------------
# description  return GoogleCheckout as string. Name of PaymentType for search/view
#                               transactions.
#----------------------------------------------------------------------------------------
# return       $Name | name of payment type | string
#========================================================================================
sub TransactionPaymentName { 'Ipayment' }

#========================================================================================
# function     PaymentRedirectIpayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       PaymentRedirectIpayment( $Servlet );
#----------------------------------------------------------------------------------------
# description  Backlink function which is called by the user to return from Ipayment site,
#               the porpouse is to show the order and inform of an error
#----------------------------------------------------------------------------------------
# input        $Servlet | UI object | object
#========================================================================================
sub PaymentRedirectIpayment {
    my $self = shift;
    my ($Servlet) = @_;

    if(defined $Servlet && defined $Servlet->object()) {
        DE_EPAGES::Ipayment::API::Logger->instance()->setSiteAliasWithObject($Servlet->object());
    }

    my $hFormParams = $Servlet->params();

    $self->_LogPayment('Ipayment','Redirect parameters', $hFormParams);

    $self->{'Form'} = $Servlet->form();

    my $LineItemPaymentGUID = $hFormParams->{'shopper_id'};

    if (!isValid($LineItemPaymentGUID) || !$self->_ExistsObjectByGUID($LineItemPaymentGUID)){
        $self->{'Form'}->executeFormError({
            'Name'       => 'GUID',
            'Reason'     => 'BadSecurityLayer_NoPaymentGUID',
            'ViewAction' => ERROR_VIEWACTION_NAME,
        });
    }
    my $LineItemPayment = $self->_LoadObjectByGUID($LineItemPaymentGUID);
    if (!$LineItemPayment->instanceOf('LineItemPaymentIpayment')){
        $self->{'Form'}->executeFormError({
            'Name'       => 'GUID',
            'Reason'     => 'LineItemPayment_NoIpaymentGUID',
            'ViewAction' => ERROR_VIEWACTION_NAME,
        });
    }

    my $Shop = $Servlet->object();
    my $Order = $LineItemPayment->container()->parent();
    my $ReturnStatus = defined $hFormParams->{'ret_status'} ? $hFormParams->{'ret_status'} : '';

    if($Shop->get('IsUnityRegistered')) {
        if ($ReturnStatus eq 'ERROR') {
            $LineItemPayment->set({'PaymentErrorCode' => 'Failed'});
        }

        $Servlet->redirect($Order->get('WebUrlUnitySSL'));
        return;
    }

    if ($ReturnStatus eq 'ERROR'){
        $self->{'Form'}->executeFormError({
            'Name'       => 'GUID',
            'Reason'     => 'PaymentProcess_Failed',
            'ViewAction' => ERROR_VIEWACTION_NAME,
            'ObjectID'   => $Order->id,
            'ObjectClass'=> $Order->class->alias,
        });
    }

    if (defined  $Order) {
        $Servlet->vars('ViewObjectID', $Order->id);
        $Servlet->registerSession($Order->get('User'), 0);
    }

    return;
}

#========================================================================================
# function     CaptureCallback
# state        public
#----------------------------------------------------------------------------------------
# syntax       CaptureCallback( $Servlet );
#----------------------------------------------------------------------------------------
# description  ChangeAction implementation for the capture request callback.
#               Ipayment sends us the callback url as redirect which we don't call.
#               Instead we process their redirect response directly.
#               But for doing the request, we have to provide a callback URL,
#               so we created an URL with this ChangeAction as dummy.
#               If the behaviour changes on the ipayment side and they start to call
#               the callback url directly instead of sending us a response,
#               we can easily finding this out by checking the communication.log.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | object
#========================================================================================
sub CaptureCallback {
    my $self = shift;
    my ($Servlet) = @_;

    if(defined $Servlet && defined $Servlet->object()) {
        DE_EPAGES::Ipayment::API::Logger->instance()->setSiteAliasWithObject($Servlet->object());
    }

    my $hFormParams = $Servlet->params();
    $self->_LogPayment('Ipayment', 'CaptureCallback parameters', $hFormParams);

    return undef;
}

#========================================================================================
# function     CancelCallback
# state        public
#----------------------------------------------------------------------------------------
# syntax       CancelCallback( $Servlet );
#----------------------------------------------------------------------------------------
# description  ChangeAction implementation for the cancel request callback.
#               Ipayment sends us the callback url as redirect which we don't call.
#               Instead we process their redirect response directly.
#               But for doing the request, we have to provide a callback URL,
#               so we created an URL with this ChangeAction as dummy.
#               If the behaviour changes on the ipayment side and they start to call
#               the callback url directly instead of sending us a response,
#               we can easily finding this out by checking the communication.log.
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | object
#========================================================================================
sub CancelCallback {
    my $self = shift;
    my ($Servlet) = @_;

    DE_EPAGES::Ipayment::API::Logger->instance()->setSiteAliasWithObject($Servlet->object());

    my $hFormParams = $Servlet->params();
    $self->_LogPayment('Ipayment', 'CancelCallback parameters', $hFormParams);

    return undef;
}

#========================================================================================
# function     PaymentHiddenTriggerIpayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       PaymentHiddenTriggerIpayment( $Servlet );
#----------------------------------------------------------------------------------------
# description  Callback called by Ipayment website to set the transaction status to paid
#               or failed just in case.
#----------------------------------------------------------------------------------------
# input        $Servlet | UI object | object
#========================================================================================
sub PaymentHiddenTriggerIpayment {
    my $self = shift;
    my ($Servlet) = @_;

    if(defined $Servlet && defined $Servlet->object()) {
        DE_EPAGES::Ipayment::API::Logger->instance()->setSiteAliasWithObject($Servlet->object());
    }

    $self->BlockMonitor($Servlet);
    my $hFormParams = $Servlet->params();
    my $SecurityLayer = $self->_createValidSecurityLayer($hFormParams);

    if (isValid($SecurityLayer)) {
        $self->_LogPayment('Ipayment', 'Correct HiddenTrigger call, from Ipayment IP. Parameters', $hFormParams);
        $self->_ProcessPaymentParameters($SecurityLayer, $Servlet);
        $self->_SendHTTPResponseOK($Servlet);
    }
    return;
}

#========================================================================================
# function     PaymentSilentErrorIpayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       PaymentSilentErrorIpayment( $Servlet );
#----------------------------------------------------------------------------------------
# description  Callback called by Ipayment in the case of an error
#----------------------------------------------------------------------------------------
# input        $Servlet | UI object | object
#========================================================================================
sub PaymentSilentErrorIpayment {
    my $self = shift;
    my ($Servlet) = @_;

    if(defined $Servlet && defined $Servlet->object()) {
        DE_EPAGES::Ipayment::API::Logger->instance()->setSiteAliasWithObject($Servlet->object());
    }

    $self->BlockMonitor($Servlet);
    my $hFormParams = $Servlet->params();
    my $SecurityLayer = $self->_createValidSecurityLayer($hFormParams);

    if (isValid($SecurityLayer)) {
        my $Response = $self->_createCGIResponse();
        $Response->rejectTransaction($SecurityLayer);
        $self->_LogPayment('Ipayment', 'PaymentSilentErrorIpayment', $hFormParams);
    }

    my $LineItemPaymentGUID = $hFormParams->{'shopper_id'};
    if (!isValid($LineItemPaymentGUID) || !$self->_ExistsObjectByGUID($LineItemPaymentGUID)){
        $self->{'Form'}->executeFormError({
            'Name'       => 'GUID',
            'Reason'     => 'BadSecurityLayer_NoPaymentGUID',
            'ViewAction' => ERROR_VIEWACTION_NAME,
        });
    }
    my $LineItemPayment = $self->_LoadObjectByGUID($LineItemPaymentGUID);
    if (!$LineItemPayment->instanceOf('LineItemPaymentIpayment')){
        $self->{'Form'}->executeFormError({
            'Name'       => 'GUID',
            'Reason'     => 'LineItemPayment_NoIpaymentGUID',
            'ViewAction' => ERROR_VIEWACTION_NAME,
        });
    }

    my $Shop = $Servlet->object();
    my $Order = $LineItemPayment->container()->parent();

    if($Shop->get('IsUnityRegistered')) {
        $LineItemPayment->set({'PaymentErrorCode' => 'Canceled'});
        $Servlet->redirect($Order->get('WebUrlUnitySSL'));
        return;
    }

    return;
}

#========================================================================================
# function     _createValidSecurityLayer
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createValidSecurityLayer( $hFormParams );
#----------------------------------------------------------------------------------------
# description  This method create a security layer
#               If the security layer is not valid returns undef
#----------------------------------------------------------------------------------------
# input        $hFormParams | Hash with the form params | hash
# return       $SecurityLayer | Security layer object | DE_EPAGES::Ipayment::API::CGI::SecurityLayer
#========================================================================================
sub _createValidSecurityLayer {
    my $self = shift;
    my ($hFormParams) = @_;

    my $SecurityLayer = undef;
    my $AreParamsValid = 0;
    eval {
        $SecurityLayer = $self->_getCGISecurityLayer();
        $AreParamsValid = $SecurityLayer->extractDataFromParams($hFormParams);
    };
    if ( ExistsError() ) {
        $self->_LogPayment('Ipayment', 'SecurityLayerError', GetError());
        return undef;
    }
    if (!$AreParamsValid) {
        $self->_LogPayment('Ipayment', 'Request Error: invalid params', $SecurityLayer->getErrorMessage());
        return undef;
    }
    return $SecurityLayer;
}

#========================================================================================
# function     _ProcessPaymentParameters
# state        private
#----------------------------------------------------------------------------------------
# syntax       _ProcessPaymentParameters( $SecurityLayer, $Servlet );
#----------------------------------------------------------------------------------------
# description  process the payment parameters
#----------------------------------------------------------------------------------------
# input        $SecurityLayer | SecurityLayer object | object
# input        $Servlet | Servlet | object
#=======================================================================================
sub _ProcessPaymentParameters {
    my $self = shift;
    my ($SecurityLayer, $Servlet) = @_;

    my $BasketOrOrder = $SecurityLayer->getBasketOrOrder();
    if ($BasketOrOrder->instanceOf('Basket')){
        my $Order = $self->_DoBasket2Order($Servlet, $BasketOrOrder);
        $SecurityLayer->{'Order'}= $Order;# TODO: how the security layer is implement right now, he is using the lineItemPayment and not the Orders
    }
    my $Response = $self->_createCGIResponse();
    $Response->createTransaction($SecurityLayer);
    return;
}

#========================================================================================
# function     _SendHTTPResponseOK
# state        private
#----------------------------------------------------------------------------------------
# syntax       _SendHTTPResponseOK( $Servlet );
#----------------------------------------------------------------------------------------
# description  set the HTTP response to OK 200
#----------------------------------------------------------------------------------------
# input        $Servlet | Servlet | object
#========================================================================================
sub _SendHTTPResponseOK{
    my $self = shift;
    my ($Servlet) = @_;

    $Servlet->params('ContentType', 'text/plain');
    $Servlet->params('ExitAfterEvent', 1);
    $Servlet->writeContentOutput('OK');

    return;
}

#========================================================================================
# function     _createCGIResponse
# state        private
#----------------------------------------------------------------------------------------
# syntax       _createCGIResponse( );
#----------------------------------------------------------------------------------------
# description  create a CGI Response to give an option to inject for unit tests
#----------------------------------------------------------------------------------------
# return       $ReturnVariable | object | DE_EPAGES::Ipayment::API::CGI::Response
#========================================================================================
sub _createCGIResponse(){
    my $self = shift;
    return DE_EPAGES::Ipayment::API::CGI::Response->new();
}

#========================================================================================
# function     _getCGISecurityLayer
# state        private
#----------------------------------------------------------------------------------------
# syntax       _getCGISecurityLayer( );
#----------------------------------------------------------------------------------------
# description  create a CGI SecurityLayer to give an option to inject for unit tests
#----------------------------------------------------------------------------------------
# return       $ReturnVariable | object | DE_EPAGES::Ipayment::API::CGI::SecurityLayer
#========================================================================================
sub _getCGISecurityLayer {
    my $self = shift;
    return DE_EPAGES::Ipayment::API::CGI::SecurityLayer->new();
}

#----------------------------------------------------------------------------------------
# Wrapper for core bindings
#----------------------------------------------------------------------------------------
sub _DoBasket2Order {
    my $self = shift;
    my ($Servlet, $Basket) = @_;
    return DE_EPAGES::Order::UI::Basket->doBasket2Order($Servlet, $Basket);
}

sub _ExistsObjectByGUID {
    my $self = shift;
    my ($GUID) = @_;
    return ExistsObjectByGUID($GUID);
}

sub _LoadObjectByGUID {
    my $self = shift;
    my ($GUID) = @_;
    return LoadObjectByGUID($GUID);
}

sub _LogPayment {
    my $self = shift;
    my ($PaymentTypeAlias, $Name, $DebugStuff, $Section, $Filter) = @_;
    my $Logger = DE_EPAGES::Ipayment::API::Logger->instance();
    return $Logger->logPayment($PaymentTypeAlias, $Name, $DebugStuff, $Section, $Filter);
}

1;
