#========================================================================================
# package      DE_EPAGES::Object::API::Object::Class
# state        public
# base         DE_EPAGES::Object::API::Object::Object
# description  object interface for Class.
#========================================================================================
package DE_EPAGES::Object::API::Object::Class;
use base DE_EPAGES::Object::API::Object::Object;

use strict;
use Scalar::Util qw ( weaken );
use DE_EPAGES::Core::API::Error qw ( CallerError Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( GetLog );
use DE_EPAGES::Core::API::Iterator;
use DE_EPAGES::Core::API::PerlTools qw( GenerateGUID GetCurrentDateTime UsePackage GetPackageFileName );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw (
    LoadObject
    LoadRootObject
    LoadClassByAlias
    DeleteClassAtCache
    DeleteObjectAtCache
    ResetFactoryCaches
    Transaction
);
use DE_EPAGES::Core::API::File qw ( ExistsFile );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Object::API::Object::Attribute;
use DE_EPAGES::Object::API::Object::Country;
use DE_EPAGES::Object::API::Object::Locale;
use DE_EPAGES::Object::API::Object::Object;
use DE_EPAGES::Object::API::Object::Site;
use DE_EPAGES::Object::API::Table::Currency qw( InsertCurrency );
use DE_EPAGES::Object::API::Table::Language qw( InsertLanguage );
use DE_EPAGES::Object::API::Table::ObjectDelete; # OnDeleteClass
use DE_EPAGES::WebInterfacePoolDB::API::ASPoolDB qw( RunOnASPoolDB );

our $DIE_ON_LOAD = 1; # disable die on patch process

sub ErrorDefinition {
    return {
        'NoResults_Attribute' => 'Attribute #Attribute doesn\'t exist in class #Class',
        'NoResults_Class' => 'Class (ClassID=#ClassID) doesn\'t exist',
        'NoPackage' => 'Class (ClassID=#ClassID) has no package, please remove class manually',
        'NoBaseClass' => 'A base class must be specified',
        'PARAMETER_MISSING' => 'Missing parameter #Parameter',
   };
}

sub slavePrimaryKey { 'ClassID' }

#========================================================================================
# function     load
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Class = $Package->load( $ObjectID );
# example      my $Class = LoadObject($ObjectID);
#----------------------------------------------------------------------------------------
# description  Loads an existing object.
#               Note: Use <function DE_EPAGES::Object::API::Factory::LoadObject> instead.
#----------------------------------------------------------------------------------------
# input        $ObjectID | object id | int
# return       $Class | existing Class object | object
#========================================================================================
sub load {
    my $class = shift;
    my ($ClassID) = @_;

    my $self = $class->SUPER::load( $ClassID );

    my $ClassPackage = $self->get('Package');
    Error('NoPackage', { ClassID => $ClassID }) unless defined $ClassPackage;

    unless ($DIE_ON_LOAD) {
        #normally used for the patch, if the package cant be find as file, set package to base pacage
        my $PackageFileName = $ENV{'EPAGES_CARTRIDGES'} . '/' . GetPackageFileName($ClassPackage);

        #check if the file for the package is in the current epages folder
        unless (ExistsFile($PackageFileName)){
            #file does not exists rename package to base package
            my $SubstitutePackage = $self->baseClass->get('Package');

            GetLog()->info("cant find file '$PackageFileName' for package '$ClassPackage' (ClassID=$ClassID)");

            eval {
                UsePackage($SubstitutePackage);
            };
            die "can't use substitute class package '$SubstitutePackage' (ClassID=$ClassID): $@" if $@;

            # write super package to cache, but not to database
            $self->{'Attributes'}{'Package'} = $SubstitutePackage;

        }
        #file exists normal use package
        else{
            UsePackage($ClassPackage);
        }
    }
    #die on load is on normal use package
    else{
        UsePackage($ClassPackage);
    }

    $self->_preLoad;

    return $self;
}

sub _preLoad {
    my $self = shift;

    eval {
        # pre-load all direct attributes
        $self->{'DirectAttributes'} = {};
        my $aAttributes = LoadClassByAlias('Attribute')->getObjects('ByAttributeClassID', [$self->id]);
        $self->{'DirectAttributes'}{$_->alias} = $_ foreach @$aAttributes;
        $self->{'IsNotSubClassOfObject'} = ! $self->isSubClassOf('Object');
    };
    if( ExistsError() ) {
        my $Error = GetError();
        #clean incomplete object cache (remove incomplete class data from object cache)
        DeleteObjectAtCache($self);
        Error($Error);
    }
}

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $Class->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'Class' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $Class->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Class = DE_EPAGES::Object::API::Object::Class->insert($hInfo);
# example      my $Class = LoadClassByAlias('Class');
#               my $Class = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertClass' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $Class | new object | object
# hook         OBJ_InsertClass | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ClassID       - class id             - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;
    Error('NoBaseClass', $hInfo) unless exists $hInfo->{'BaseClass'};

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertClass', {'Object' => $self, 'ClassID' => $self->id });
    delete $hInfo->{$_} foreach qw ( BaseClass Package ExportLevel );
    return $self;
}

#========================================================================================
# function     reset
# state        private
# syntax       $Object->reset;
# description  if an attribute was created or removed the class and all sub classes must be reset
#========================================================================================
sub reset {
    my $self = shift;
    return if $self->{'OnImport'};

    # reset this class
    delete $self->{'AttributePackages'};
    $self->_preLoad;

    $self->SUPER::reset;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Class->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteClass' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteClass | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ClassID       - class id             - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteClass', {'Object' => $self, 'ClassID' => $self->id });
    DE_EPAGES::Object::API::Table::ObjectDelete::OnDeleteClass({ ClassID => $self->id });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     deleteSelf
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Class->deleteSelf;
#----------------------------------------------------------------------------------------
# description  Deletes the object at database tables.
#========================================================================================
sub deleteSelf {
    my $self = shift;

    DeleteClassAtCache($self);
    $self->SUPER::deleteSelf;
    return;
}

#========================================================================================
# function     deleteAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object->deleteAttributes;
#----------------------------------------------------------------------------------------
# description  Deletes subclasses, call onDeleteClass for each object of class.
#========================================================================================
sub deleteAttributes {
    my $self = shift;

    # delete sub classes
    $_->delete foreach @{ $self->directSubClasses };

    # delete objects of class
    $_->onDeleteClass foreach @{ $self->directObjects };

    # delete all attributes
    $self->reset;
    $_->delete foreach @{ $self->directAttributes };
    $self->SUPER::deleteAttributes;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Class->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateClass' and 'OBJ_AfterUpdateClass'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateClass | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ClassID       - class id             - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateClass | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ClassID       - class id             - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    delete $self->{'BaseClass'} if exists $hValues->{'BaseClass'};

    TriggerHook('OBJ_BeforeUpdateClass', {'Object'=>$self, 'Values'=> $hValues, 'ClassID' => $self->id, 'LanguageID' => $LanguageID});
    if (defined $hValues->{'Package'}) {
        my $ClassPackage = $hValues->{'Package'};
        die "Invalid package name: $ClassPackage" unless $ClassPackage =~ /^[\w:]+$/;
        eval {
            UsePackage($ClassPackage);
        };
        warn "can't use $ClassPackage: $@" if $@;
    }
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateClass',  {'Object'=>$self, 'Values'=> $hValues, 'ClassID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     insertSubClass
# state        public
#----------------------------------------------------------------------------------------
# syntax       $SubClass = $Class->insertSubClass($hClass);
#----------------------------------------------------------------------------------------
# description  Creates a new sub class.
#               The default package is 'DE_EPAGES::Object::API::Object::Object'.
#----------------------------------------------------------------------------------------
# input        $hClass | with following keys:
#               <ul>
#                   <li>Alias         - class alias           - varchar(255)
#                   <li>Package       - attribute package for class (optional) - varchar(255)
#               </ul> | reference to hash
# return       $Class | class object | object
#========================================================================================
sub insertSubClass {
    my $self = shift;
    my ($hClass) = @_;

    $hClass->{'Package'} = $self->packageObject unless defined $hClass->{'Package'};
    $hClass->{'BaseClass'} = $self;
    $hClass->{'Parent'} = $self->parent unless defined $hClass->{'Parent'};
    $hClass->{'Class'} = $self->class;
    return __PACKAGE__->insert( $hClass );
}

#========================================================================================
# function     baseClass
#----------------------------------------------------------------------------------------
# syntax       $BaseClass = $Class->baseClass;
#----------------------------------------------------------------------------------------
# description  Returns the base class or undef if the class does not have a base class.
#----------------------------------------------------------------------------------------
# return       $BaseClass | base class | object
#========================================================================================
sub baseClass {
    my $self = shift;

    my $BaseClass= $self->{'BaseClass'};
    unless( defined $BaseClass ) {
        my $BaseClassID = $self->{'Attributes'}{'BaseClassID'};
        return undef unless defined $BaseClassID;

        $BaseClass = LoadObject($BaseClassID);
        $self->{'BaseClass'} = $BaseClass;
        weaken( $self->{'BaseClass'} );
    }

    return $BaseClass;
}

#========================================================================================
# function     baseClasses
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aBaseClasses = $Class->baseClasses;
#----------------------------------------------------------------------------------------
# description  Returns all base classes recursively in ascending order, i.e. the
#               Object class (which has no other base class) is the first array element.
#               This array includes the given class as last element.
#----------------------------------------------------------------------------------------
# return       $aBaseClasses | list of base classes | reference to array of object
#========================================================================================
sub baseClasses {
    my $self = shift;

    my $Class = $self;
    my @Classes;
    while( defined $Class ) {
        unshift @Classes, $Class;
        $Class = $Class->baseClass;
    }

    return \@Classes;
}

#========================================================================================
# function     subClasses
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aSubClasses = $Class->subClasses;
#----------------------------------------------------------------------------------------
# description  Returns all known sub classes of a class recursively.
#----------------------------------------------------------------------------------------
# return       $aSubClasses | list of sub classes | reference to array of object
#========================================================================================
sub subClasses {
    my $self = shift;

    return [
        map { ( $_, @{ $_->subClasses } ) }
        @{ $self->directSubClasses }
    ];
}

#========================================================================================
# function     directSubClasses
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aSubClasses = $Class->directSubClasses;
#----------------------------------------------------------------------------------------
# description  Returns all direct sub classes of a class (not recursively).
#----------------------------------------------------------------------------------------
# return       $aSubClasses | list of sub classes | reference to array of object
#========================================================================================
sub directSubClasses {
    my $self = shift;

    my $Class = LoadClassByAlias('Class');
    return [
        sort { $a->alias cmp $b->alias }
        @{ $Class->getObjects('ByBaseClassID', [ $self->id ]) }
    ];
}

#========================================================================================
# function     insertAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Class->insertAttribute( $hAttribute );
#----------------------------------------------------------------------------------------
# description  Adds a new attribute to the class.
#----------------------------------------------------------------------------------------
# input        $hAttribute | The hash contains these keys:
#               <ul>
#                   <li>Alias         - attribute alias      - varchar(255)
#                   <li>Type          - data type name       - varchar(30)
#                   <li>Length        - maximum length       (optional)- int
#                   <li>Package       - package name         - varchar(255)
#                   <li>IsArray       - array attribute (optional, default=0) - boolean
#                   <li>IsReadOnly    - read only (optional, default=0) - boolean
#                   <li>IsExportable  - include in export files (optional, default=1) - boolean
#               </ul> | reference to hash
#----------------------------------------------------------------------------------------
# return       $Attribute | new attribute | object
#========================================================================================
sub insertAttribute {
    my $self = shift;
    my ($hAttribute) = @_;

    $hAttribute->{AttributeClass} = $self;
    my $Attribute = $self->folder('Attributes')->insertChild( 'Attribute', $hAttribute );
    $self->reset;
    return $Attribute;
}

#========================================================================================
# function     deleteAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Deleted = $Class->deleteAttribute( $Alias );
#----------------------------------------------------------------------------------------
# description  Deletes an attribute from the class.
#----------------------------------------------------------------------------------------
# input        $Alias | attribute alias | string
# return       $Deleted | attribute was deleted | boolean
#========================================================================================
sub deleteAttribute {
    my $self = shift;
    my ($Alias) = @_;

    return 0 unless $self->existsDirectAttribute( $Alias );

    $self->attribute( $Alias )->delete;
    $self->reset;
    return 1;
}

#========================================================================================
# function     updateAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Class->updateAttribute( $Alias, $hAttribute );
#----------------------------------------------------------------------------------------
# description  Updates properties of an attribute.
#----------------------------------------------------------------------------------------
# input        $Name | attribute name | string
# input        $hAttribute | attribute properties, see <function insertAttribute> | reference to hash
#========================================================================================
sub updateAttribute {
    my $self = shift;
    my ($Alias, $hAttribute) = @_;

    $self->attribute( $Alias )->set( $hAttribute );

    $self->reset;
    return;
}

#========================================================================================
# function     attribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hAttribute = $Class->attribute( $Alias );
#----------------------------------------------------------------------------------------
# description  Returns information about an attribute.
#----------------------------------------------------------------------------------------
# input        $Alias | attribute alias | string
# return       $Attribute | attribute object, see class: Attribute | object
#========================================================================================
sub attribute {
    my $self = shift;
    my ($Alias) = @_;
    CallerError('PARAMETER_MISSING', {'Parameter' => 'Alias'}) unless $Alias;

    my $ThisClass = $self;
    while( defined $ThisClass ) {
        my $hDirectAttributes = $ThisClass->{'DirectAttributes'};
        return $hDirectAttributes->{$Alias} if exists $hDirectAttributes->{$Alias};
        $ThisClass = $ThisClass->baseClass;
    }

    Error('NoResults_Attribute', { 'Attribute' => $Alias, 'Class' => $self->alias } );
}

#========================================================================================
# function     attributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aAttributes = $Class->attributes;
#----------------------------------------------------------------------------------------
# description  Returns all attributes of a class, including inherited ones.
#----------------------------------------------------------------------------------------
# return       $aAttributes | list of attributes, see <function attribute> | reference to array of hash
#========================================================================================
sub attributes {
    my $self = shift;

    # collect all attributes, sub classes can override attributes with the same name
    my %Attributes;
    # get all base classes
    foreach my $Class ( @{$self->baseClasses} ) {
        foreach my $Attribute ( @{$Class->directAttributes}) {
            my $Alias = $Attribute->alias;
            $Attributes{$Alias} = $Attribute;
        }
    }
    return [ map { $Attributes{$_} } sort keys %Attributes ];
}

#========================================================================================
# function     directAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aAttributes = $Class->directAttributes;
#----------------------------------------------------------------------------------------
# description  Returns all direct attributes of a class (not including inherited ones).
#----------------------------------------------------------------------------------------
# return       $aAttributes | list of attributes, see <function attribute> | reference to array of hash
#========================================================================================
sub directAttributes { [ values %{shift->{'DirectAttributes'}} ] }

#========================================================================================
# function     existsAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $Class->existsAttribute( $Alias )
# example      if( $Class->existsAttribute( 'MinOrderLevel' ) ) { ... }
#----------------------------------------------------------------------------------------
# description  Returns true if an attribute with this name exists in the class.
#----------------------------------------------------------------------------------------
# input        $Name | attribute name | string
# return       $Exists | true it the attribute exists | boolean
#========================================================================================
sub existsAttribute {
    my $self = shift;
    my ($Alias) = @_;
    CallerError('PARAMETER_MISSING', {'Parameter' => 'Alias'}) unless defined $Alias;

    my $ThisClass = $self;
    while( defined $ThisClass ) {
        return 1 if exists $ThisClass->{'DirectAttributes'}{$Alias};
        $ThisClass = $ThisClass->baseClass;
    }

    return 0;
}

#========================================================================================
# function     existsDirectAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $Class->existsDirectAttribute( $Alias )
# example      if( $Class->existsDirectAttribute( 'MinOrderLevel' ) ) { ... }
#----------------------------------------------------------------------------------------
# description  Returns true if an attribute with this alias exists in the class and is
#               defined in this class (not in a base class).
#----------------------------------------------------------------------------------------
# input        $Alias | attribute alias | string
# return       $Exists | true it the attribute exists | boolean
#========================================================================================
sub existsDirectAttribute { ## no critic qw(RequireArgUnpacking RequireCommentForExportedFunctions)
#    my $keys = join(',',keys(%{$_[0]->{'DirectAttributes'}}));
#    warn"Alias:$_[1] in keys: ($keys)";
    return exists $_[0]->{'DirectAttributes'}{$_[1]};
}

#========================================================================================
# function     directAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Attribute = $Class->directAttribute( $Alias )
#----------------------------------------------------------------------------------------
# description  Returns the attribute object with this alias exists in the class and is
#               defined in this class (not in a base class).
#----------------------------------------------------------------------------------------
# input        $Alias | attribute alias | string
# return       $Exists | true it the attribute exists | boolean
#========================================================================================
sub directAttribute { ## no critic qw(RequireArgUnpacking RequireCommentForExportedFunctions)
    return $_[0]->{'DirectAttributes'}{$_[1]};
}

#========================================================================================
# function     attributePackages
# state        public
#----------------------------------------------------------------------------------------
# syntax       $haPackages = $Class->attributePackages
# syntax       $haPackages = $Class->attributePackages( $aNames );
#----------------------------------------------------------------------------------------
# description  Returns all attribute packages and the corresponding attribute names.
#----------------------------------------------------------------------------------------
# input        $aNames | list of attribute names | ref.array.string
# return       $haPackages | all packages and attributes as hash
#                           <ul>
#                               <li>{ $PackageName => \@AttributeNames, ... }
#                           </ul>
#                           | ref.hash.array
#========================================================================================
sub attributePackages ($;$) {
    my $self = shift;
    my ($aNames) = @_;

    my $haPackages = $self->{'AttributePackages'};
    return $haPackages if defined $haPackages && !defined $aNames;

    my $aAttributes = defined $aNames
        ? [ map { $self->attribute( $_ ) } @$aNames ]
        : $self->attributes;

    # group attribute names by package
    my %Packages;
    foreach my $Attribute (@$aAttributes) {
        my $Package = $Attribute->package;
        $Packages{$Package} = [] unless exists $Packages{$Package};
        push @{$Packages{$Package}}, $Attribute->alias;
    }

    $self->{'AttributePackages'} = \%Packages unless defined $aNames;

    return \%Packages;
}

#========================================================================================
# function     defaultAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hValues = $Class->defaultAttributes($Parent, $hInfo);
#----------------------------------------------------------------------------------------
# description  Returns hash of default attribute values of this class.
#----------------------------------------------------------------------------------------
# input        $Parent | parent of new or existing object | object
# input        $hInfo | (optional) additional parameters, passed through to
#                       <function DE_EPAGES::Object::API::BaseAttribute::defaultAttributes>
#                       as extra parameter
#                       | ref.hash
# return       $hValues | default attribute names and value pairs | ref.hash
#========================================================================================
sub defaultAttributes {
    my $self = shift;
    my ($Parent, $hInfo) = @_;
    my $hPackages = $self->attributePackages;
    my $hDefaults = $self->packageObject->default($Parent, $hInfo);
    foreach my $Package (keys %$hPackages) {
        my $hValues = $Package->defaultAttributes($Parent, $hPackages->{$Package}, $hInfo);
        $hDefaults->{$_} = $hValues->{$_} foreach grep { exists $hValues->{$_} } @{$hPackages->{$Package}};
    }
    return $hDefaults;
}

#========================================================================================
# function     allDefaultAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hValues = $Class->allDefaultAttributes($Parent, $hInfo);
#----------------------------------------------------------------------------------------
# description  Returns hash of default attribute values of this class (including undef
#               for not defined defaults).
#----------------------------------------------------------------------------------------
# input        $Parent | parent of new or existing object | object
# input        $hInfo | given parameter (optional) | ref.hash
# return       $hValues | default attribute names and value pairs | ref.hash
#========================================================================================
sub allDefaultAttributes {
    my $self = shift;
    my ($Parent, $hInfo) = @_;
    my $hPackages = $self->attributePackages;
    my $hDefaults = $self->packageObject->default($Parent, $hInfo);
    foreach my $Package (keys %$hPackages) {
        my $hValues = $Package->defaultAttributes($Parent, $hPackages->{$Package}, $hInfo);
        foreach my $AttributeName (@{$hPackages->{$Package}}) {
            $hDefaults->{$AttributeName} = undef unless exists $hDefaults->{$AttributeName};
            $hDefaults->{$AttributeName} = $hValues->{$AttributeName} if exists $hValues->{$AttributeName};
            delete $hDefaults->{$AttributeName} if $self->attribute($AttributeName)->get('IsReadOnly');
        }
    }
    return $hDefaults;
}

#========================================================================================
# function     cloneAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Class->cloneAttributes($Object, $Clone);
#----------------------------------------------------------------------------------------
# description  call cloneAttribute for each attribute package
#----------------------------------------------------------------------------------------
# input        $Object | object to clone| object
# input        $Clone | copy of object| object
#========================================================================================
sub cloneAttributes {
    my $self = shift;
    my ($Object, $Clone) = @_;
    my $hPackages = $self->attributePackages;
    $_->cloneAttributes($Object, $Clone) foreach keys %$hPackages;
    return;
}

#========================================================================================
# function     importXMLPackages
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Class->importXMLPackages($Handler, $hAttributes);
#----------------------------------------------------------------------------------------
# description  Converts attributes from XML file to object attribute values.
#----------------------------------------------------------------------------------------
# input        $Handler | xml import handler with support function addObject | object
# input        $hAttributes | imported attributes (modified in place) | ref.hash
#========================================================================================
sub importXMLPackages {
    my $self = shift;
    my ($Handler, $hAttributes) = @_;
    my $hPackages = $self->attributePackages;
    foreach my $Package (keys %$hPackages) {
        $Package->importXML($Handler, $self, $hPackages->{$Package}, $hAttributes );
    }
    return;
}

#========================================================================================
# function     exportXMLAttributesPackages
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Class->exportXMLAttributesPackages($Driver, $Object, $hAttributes);
#----------------------------------------------------------------------------------------
# description  Adds or modifies attribute values for XML export.
#----------------------------------------------------------------------------------------
# input        $Driver | xml export driver | object
# input        $Object | object to export | object
# input        $hAttributes | exported attributes (modified in place) | ref.hash
#========================================================================================
sub exportXMLAttributesPackages {
    my $self = shift;
    my ($Driver, $Object, $hAttributes) = @_;
    my $hPackages = $self->attributePackages;
     foreach my $Package (keys %$hPackages) {
         $Package->exportXMLAttributes($Driver, $Object, $hPackages->{$Package}, $hAttributes );
    }
    return;
}

#========================================================================================
# function     exportXMLElementsPackages
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Class->exportXMLElementsPackages($Driver, $Object, $hAttributes);
#----------------------------------------------------------------------------------------
# description  Adds elements to the export XML file.
#----------------------------------------------------------------------------------------
# input        $Driver | xml export driver | object
# input        $Object | object to export | object
# input        $hAttributes | exported attributes | ref.hash
#========================================================================================
sub exportXMLElementsPackages {
    my $self = shift;
    my ($Driver, $Object, $hAttributes) = @_;

    my $hPackages = $self->attributePackages;
    foreach my $Package (keys %$hPackages) {
         $Package->exportXMLElements($Driver, $Object, $hPackages->{$Package}, $hAttributes );
    }
    return;
}

#========================================================================================
# function     packageObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Package = $Class->packageObject;
#----------------------------------------------------------------------------------------
# description  Returns the perl package of objects of this class. The returned package
#               was imported via use before.
#----------------------------------------------------------------------------------------
# return       $Package | perl package name | string
#========================================================================================
sub packageObject {  ## no critic qw(RequireArgUnpacking)
    return $_[0]->{'Attributes'}{'Package'};
}

#========================================================================================
# function     dalObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DAL = $Class->dalObject;
#----------------------------------------------------------------------------------------
# description  Returns the dal (database abstration layer) of objects of this class,
#               not of the class it self ($Class->dal).
#----------------------------------------------------------------------------------------
# return       $DAL | database abstraction layer | object
#========================================================================================
sub dalObject {
    my $self = shift;
    return $self->packageObject->dal;
}

#========================================================================================
# function     infoObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahInfo = $Class->infoObjects;
# syntax       $ahInfo = $Class->infoObjects($IndexName, $hParameter);
#----------------------------------------------------------------------------------------
# description  Returns attributes and its values of all objects of the given class as array of hashes.
#----------------------------------------------------------------------------------------
# input        $IndexName | (optional) name of index | string
# input        $hParameter | (optional) parameter of index (see <function getObjects>) | ref.hash
# return       $ahInfo | object class info - keys: attributes names | ref.array.hash
#========================================================================================
sub infoObjects {
    my $self = shift;
    my ($IndexName, $hParameter) = @_;

    return defined $IndexName
        ? $self->packageObject->dal->get($IndexName, $hParameter)
        : $self->packageObject->dal->all;
}

#========================================================================================
# function     findObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aObjectIDs = $Class->findObjects($IndexName, $hParameter);
#----------------------------------------------------------------------------------------
# description  Returns matching objects identifiers by index.
#----------------------------------------------------------------------------------------
# input        $IndexName | name of index | string
# input        $hParameter | parameter of index (see <function getObjects>) | ref.hash
# return       $aObjectIDs | object identifier of this class matches condition | ref.array.int
#========================================================================================
sub findObjects {
    my $self = shift;
    my ($IndexName, $hParameter) = @_;

    my $PrimaryKey = $self->packageObject->primaryKey;
    return [ map { $_->{$PrimaryKey} } @{$self->infoObjects($IndexName, $hParameter)} ];
}

#========================================================================================
# function     getObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aObjects = $Class->getObjects($IndexName, $hParameter);
# syntax       $aObjects = $Class->getObjects($IndexName, $aParameter);
#----------------------------------------------------------------------------------------
# example      $Class = LoadClassByAlias('Object');
#               $aObjects = $Class->getObjects('ByParentID', [$ObjectID]);
# example      $aObjects = $Class->getObjects('ByParentID', [$ObjectID, $Alias]);
# example      $aObjects = $Class->getObjects('ByParentID', {
#                   ParentID => $ObjectID,
#               });
# example      $aObjects = $Class->getObjects('ByParentID', {
#                   ParentID => $ObjectID,
#                   Alias => $Alias,
#               });
#----------------------------------------------------------------------------------------
# description  Returns matching objects by index.
#               For long lists (>1000 items) consider using <function objectIterator>.
#----------------------------------------------------------------------------------------
# input        $IndexName | name of index | string
# input        $hParameter | parameter of index | ref.hash
# input        $aParameter | array of parameter in correct order | ref.array
# return       $aObjects | objects of this class matches condition | ref.array.object
#========================================================================================
sub getObjects {
    my $self = shift;
    my ($IndexName, $hParameter) = @_;

    return [map {$self->loadObject($_) } @{$self->findObjects($IndexName, $hParameter)}];
}

#========================================================================================
# function     objectIterator
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Iterator = $Class->objectIterator($IndexName, $hParameter);
# syntax       $Iterator = $Class->objectIterator($IndexName, $aParameter);
# syntax       $Iterator = $Class->objectIterator($IndexName, $aParameter, $aOrderBy);
# syntax       $Iterator = $Class->objectIterator($IndexName, $hParameter, $aOrderBy);
#----------------------------------------------------------------------------------------
# example      $Class = LoadClassByAlias('Object');
#               $Objects = $Class->objectIterator('ByParentID',
#                   [$Parent->id],
#                   [{ Column => 'Position', Direction => 'ASC' }]
#               );
#               say $_->alias while <$Objects>;
#----------------------------------------------------------------------------------------
# description  Returns matching objects by index as iterator.
#               Use this method for potentially long lists (>1000 items).
#               Use <function getObjects> for short lists.
#----------------------------------------------------------------------------------------
# input        $IndexName | name of index | string
# input        $hParameter | parameter of index | ref.hash
# input        $aParameter | array of parameter in correct order | ref.array
# input        $aOrderBy | (optional) attributes for sorting,
#                           e.g. [{ Column => 'Position', Direction => 'ASC' }]
#                         | ref.array.hash
# return       $Iterator | object iterator | DE_EPAGES::Core::API::Iterator
#========================================================================================
sub objectIterator {
    my $self = shift;
    my ($IndexName, $hParameter, $aOrderBy) = @_;

    return DE_EPAGES::Core::API::Iterator->wrap(
        Source => $self->packageObject->dal->iterator($IndexName, $hParameter, $aOrderBy),
        Handle => sub { $self->loadObject( $_[1], 1 ) },
    );
}

#========================================================================================
# function     deleteObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Class->deleteObjects($IndexName, $hParameter);
# syntax       $Class->deleteObjects($IndexName, $aParameter);
#----------------------------------------------------------------------------------------
# description  Removes matching objects.
#----------------------------------------------------------------------------------------
# input        $IndexName | name of index | string
# input        $hParameter | parameter of index (see <function getObjects>) | ref.hash
# input        $aParameter | array of parameter in correct order | ref.array
#========================================================================================
sub deleteObjects {
    my $self = shift;
    my ($IndexName, $hParameter) = @_;

    $_->delete foreach @{$self->getObjects($IndexName, $hParameter)};
    return;
}

#========================================================================================
# function     multiObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aObjects = $Class->multiObjects($aIDs);
#----------------------------------------------------------------------------------------
# description  Returns multiple objects by object id.
#----------------------------------------------------------------------------------------
# input        $aIDs | identifier | ref.array.int
# return       $aObjects | objects of this class matches condition | ref.array.object
#========================================================================================
sub multiObjects {
    my $self = shift;
    my ($aIDs) = @_;

    my $PrimaryKey = $self->packageObject->primaryKey;
    return [map {$self->loadObject($_->{$PrimaryKey}) } @{$self->packageObject->dal->multi($aIDs)}];
}

#========================================================================================
# function     existsObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Exists = $Class->existsObject($ObjectID);
#----------------------------------------------------------------------------------------
# description  Returns true if the object of this class exists in the database.
#----------------------------------------------------------------------------------------
# input        $ObjectID | Object ID | integer
# return       $Exists | true if object exists | boolean
#========================================================================================
sub existsObject {
    my $self = shift;
    my @Parameter = @_;

    my $Package = $self->packageObject;
    return $Package->can('dal') ? $Package->dal->exists($Parameter[0]) : $Package->existsObject(@Parameter);
}

#========================================================================================
# function     loadObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object = $Class->loadObject($ObjectID, $LoadIt);
#----------------------------------------------------------------------------------------
# description  Returns object from id of this class.
#----------------------------------------------------------------------------------------
# input        $ObjectID | Object ID | integer
# input        $LoadIt | load via package (not via LoadObject) | boolean
# return       $Object | object | object
#========================================================================================
sub loadObject {
    my $self = shift;
    my ($ObjectID, $LoadIt) = @_;

    return ($LoadIt or $self->{'IsNotSubClassOfObject'})
        ? $self->packageObject->load($ObjectID)
        : LoadObject($ObjectID, $self->packageObject->dal);
}

#========================================================================================
# function     objects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aObjects = $Class->objects;
#----------------------------------------------------------------------------------------
# example      $_->delete foreach @{ $Class->objects };
#----------------------------------------------------------------------------------------
# description  Returns all objects that are direct or indirect instances of this class.
#----------------------------------------------------------------------------------------
# return       $aObjects | list of objects | array ref
#========================================================================================
sub objects {
    my $self = shift;

    my @Classes = ( $self, @{ $self->subClasses } );
    return [ map { @{ $_->directObjects } } @Classes ];
}

#========================================================================================
# function     directObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aObjects = $Class->directObjects;
#----------------------------------------------------------------------------------------
# example      $_->delete foreach @{ $Class->directObjects };
#----------------------------------------------------------------------------------------
# description  Returns all objects that are directly instanciated from this class.
#----------------------------------------------------------------------------------------
# return       $aObjects | list of objects | array ref
#========================================================================================
sub directObjects {
    my $self = shift;

    if( $self->{'IsNotSubClassOfObject'} ) {
        my $Package = $self->packageObject;
        return [] unless $Package->can('primaryKey');  # Core::API::Object::BaseObject

        my $PrimaryKey = $Package->primaryKey;
        return [map {$self->loadObject($_->{$PrimaryKey}) } @{$self->packageObject->dal->all}];
    }
    else {
        #sort because delete the last objects first (subproducts should be deleted before their master
        return [sort {$b->id <=> $a->id } @{$self->getObjects('ByClassID',[$self->id])}];
    }
}

#========================================================================================
# function     insertObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object = $Class->insertObject($hValues);
# example      $Product = $ProductType->insertObject({
#                   Parent => $Shop->child('Products'),
#                   Alias  => '08/15',
#               });
#----------------------------------------------------------------------------------------
# description  Creates a new object instance of this class with the given attribute
#               values.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# return       $Object | object | object
#========================================================================================
sub insertObject {
    my $self = shift;
    my ($hInfo) = @_;

    my $ObjectPackage = $self->packageObject;
    my $hDefaults = $self->defaultAttributes($hInfo->{'Parent'}, $hInfo);
    my $PrimaryKey = $ObjectPackage->can('primaryKey') ? $ObjectPackage->primaryKey : undef;

    my %CopyInfo = ( %$hDefaults, %$hInfo );
    $CopyInfo{'Class'} = $self if $self->existsAttribute('Class');
    $CopyInfo{$PrimaryKey} = $self->packageObject->nextid() if defined $PrimaryKey and not exists $CopyInfo{$PrimaryKey};
    my $GUID = undef;
    if ($self->isSubClassOf('Site')) {
        $CopyInfo{'GUID'} = GenerateGUID() unless defined $CopyInfo{'GUID'};
        $GUID = $CopyInfo{'GUID'};
    }
    my $Object;
    Transaction( sub {
        # copy the original attributes, because insert() clobbers the input hash ref
        # in case of a deadlock-retry the attributes would be lost
        my %Info = %CopyInfo;
        $Object = $self->packageObject->insert( \%Info );

        # attribute ObjectID must not be set, otherwise $Object->id
        # does not work for BaseObject derived classes.
        delete $Info{$PrimaryKey} if defined $PrimaryKey;
        $Object->set( \%Info );
    }, sub {
        if (defined $GUID) {
            RunOnASPoolDB( sub { shift->deleteSite($GUID) } );
        }
    });
    return $Object;
}

#========================================================================================
# function     isSubClassOf
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsSubClass = $Class->isSubClassOf( $Class )
# syntax       $IsSubClass = $Class->isSubClassOf( $ClassName )
#----------------------------------------------------------------------------------------
# example      my $ClassProduct = LoadClassByAlias( 'Product' );
#               if( $Class->isSubClassOf( $ClassProduct ) ) { ... }
# example      if( $Class->isSubClassOf('Product' ) ) { ... }
#----------------------------------------------------------------------------------------
# description  Returns true if the class is a sub class of the class or one of its
#               sub classes.
#----------------------------------------------------------------------------------------
# input        $ClassName  | name of class (or object of class) | string|object
# return       $IsSubClass | true if the class is a sub class of the class | boolean
#========================================================================================
sub isSubClassOf {
    my $self = shift;
    my ($Class) = @_;
    $Class = LoadClassByAlias( $Class ) unless ref $Class;

    my $ClassID = $Class->id;
    my $ThisClass = $self;
    while( defined $ThisClass ) {
        return 1 if $ThisClass->id == $ClassID;
        $ThisClass = $ThisClass->baseClass;
    }
    return 0;
}

#========================================================================================
# function     importXML
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object = __PACKAGE__->importXML($Handler, $hElement, $Class, $hAttributes);
#----------------------------------------------------------------------------------------
# description  Imports an object from an XML file. This function is called at the XML
#               start tag. The end tag triggers the <function importXMLEnd> function.
#----------------------------------------------------------------------------------------
# input        $Handler | xml import handler with support function addObject | object
# input        $hElement | XML element structure, see XML::SAX | ref.hash
# input        $Class | class object; the new object will be an instance of this class | object
# input        $hAttributes | attributes hash | ref.hash
# return       $Object | the imported object | object
#========================================================================================
sub importXML {
    my $class = shift;
    my ($Handler, $hElement, $Class, $hAttributes) = @_;

    # get the base class id
    if( defined $hAttributes->{Base} ) {
        my $Parent = $Handler->topID('Object');
        $hAttributes->{BaseClass} = $hAttributes->{Base} =~ /\//
            ? $Parent->path( $hAttributes->{Base} )
            : LoadClassByAlias( $hAttributes->{Base} );
        Error('NO_CLASS', $hAttributes) unless $hAttributes->{BaseClass}->instanceOf('Class');
        delete $hAttributes->{Base};
    }
    my $Object = $class->SUPER::importXML($Handler, $hElement, $Class, $hAttributes );

    $Handler->pushID( 'Class', $Object );
    $Object->{'OnImport'} = 1;
    return $Object;
}

#========================================================================================
# function     importXMLEnd
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Object->importXMLEnd( $Handler, $hElement );
#----------------------------------------------------------------------------------------
# description  Called on the XML end tag.
#----------------------------------------------------------------------------------------
# input        $Handler | xml import handler with support function addObject | object
# input        $hElement | xml import element (undef on end document) | hash ref
#========================================================================================
sub importXMLEnd {
    my $self = shift;
    my ($Handler, $hElement) = @_;

    $self->SUPER::importXMLEnd( $Handler, $hElement );
    $Handler->popID( 'Class' );
    delete $self->{'OnImport'};
    $self->reset;
}

#========================================================================================
# function     exportXMLAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hAttributes = $Object->exportXMLAttributes( $Driver, $hAllAttributes );
#----------------------------------------------------------------------------------------
# description  Returns plain attributes of object to XML export driver. This function
#               removes localized attributes (deletes attribues like ParentID, Inherit
#               if default (1), Class if equal TagName).
#----------------------------------------------------------------------------------------
# input        $Driver | xml export driver with support function addObject | object
# input        $hAllAttributes | all attributes of object,
#                                 see $Object-><function allAttributes> | ref.hash
# return       $hAttributes | plain attributes | ref.hash.string
#========================================================================================
sub exportXMLAttributes {
    my $self = shift;
    my ($Driver, $hAttributes) = @_;
    $hAttributes = $self->SUPER::exportXMLAttributes( $Driver, $hAttributes );
    $hAttributes->{'Base'} = $hAttributes->{'BaseClass'} if exists $hAttributes->{'BaseClass'};
    delete $hAttributes->{'BaseClass'};
    return $hAttributes;
}

# push Class on stack
sub deleteXML {
    my ($ObjectPackage, $Handler, $hElement, $Class, $hAttributes) = @_;

    my $ObjectClass = DE_EPAGES::Object::API::Object::Object::deleteXML($ObjectPackage, $Handler, $hElement, $Class, $hAttributes);
    $ObjectClass->{'OnImport'} = 1 if defined $ObjectClass;

    # don't push Class=undefined, because it deleteXMLEnd() won't be called on a
    # deleted class, therefore the Class would leak on the stack
    $Handler->pushID( 'Class', $ObjectClass ) if defined $ObjectClass;
    return $ObjectClass;
}

sub deleteXMLEnd {
    my $self = shift;
    my ($Handler, $hElement) = @_;

    $Handler->popID( 'Class' );
    $self->SUPER::deleteXMLEnd( $Handler, $hElement );
    delete $self->{'OnImport'};
    $self->reset;
}

#========================================================================================
# function     initStructure
# state        private
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::Object::API::Object::Class->initStructure();
#----------------------------------------------------------------------------------------
# description  Initialized the Class/Attribute/Object structure by inserting basic
#               classes and attributes such as Object, Class, Attribute, System etc.
#========================================================================================
sub initStructure {
    shift;

    my $ObjectDAL = DE_EPAGES::Object::API::Object::Object->dal;
    my $SiteDAL = DE_EPAGES::Object::API::Object::Site->dal;
    my $ClassDAL = DE_EPAGES::Object::API::Object::Class->dal;
    my $AttributeDAL = DE_EPAGES::Object::API::Object::Attribute->dal;

    DE_EPAGES::Object::API::Object::Country->dal->insert({
        'CountryID' => 276,
        'IsUsed' => 1,
        'Code2' => 'DE',
        'Code3' => 'DEU',
        'English' => 'Germany',
        'IsEU' => 1,
        'SubdivsMandatory' => 0,
    });
    InsertCurrency({
        'CurrencyID' => 'EUR',
        'IsUsed' => 1,
        'NumCode' => 978,
        'Name' => 'Euro',
        'MinorUnit' => 2,
    });
    my $LanguageID = InsertLanguage({
        'Code2' => 'de',
        'Code3' => 'ger',
        'IsUsed' => 1,
        'IsForBackoffice' => 1,
    });
    DE_EPAGES::Object::API::Object::Locale->dal->insert({
        'LocaleID' => 'de_DE',
        'IsUsed' => 1,
        'CountryID' => 276,
        'CurrencyID' => 'EUR',
        'LanguageID' => $LanguageID,
    });

    my $Date = GetCurrentDateTime();
    # / (System)
    my $ObjectClassID = 4;
    my $ClassClassID  = 5;

    my $SystemID =  $SiteDAL->insert({
        'ObjectID' => 1,
        'ClassID' => $ObjectClassID,
        'ParentID' => undef, # root
        'Alias' => 'System',
        'GUID'  => GenerateGUID(),
        'Inherit' => 0,
        'Position' => 0,
        'CreationDate' => $Date,
        'SiteID' => 1,
        'IsStandalone' => 0,
        'LoginRequired'=> 0,
        'LoginCookie'  => 0,
        'HasSSLCertificate' => 0,
        'DomainName' => 'localhost',
        'WebServerScriptNamePart' => GetCurrentDBHandle->cacheKey,
        'IsPageCacheUsed' => 0,
        'IsClosed' => 0,
        'LocaleID' => 'de_DE',
        'UseMobileSF' => 1,
    });
    die 'SystemID != 1' unless $SystemID == 1;

    my $ClassesFolderID = $ObjectDAL->insert({
        ObjectID => 2,
        ClassID => $ObjectClassID,
        ParentID => $SystemID,
        Alias => 'Classes',
        GUID  => GenerateGUID(),
        Inherit => 1,
        Position => 0,
        CreationDate => $Date,
        SiteID => $SystemID,
    });

    # /Classes
    my @Classes = ( {
        ObjectID => 3,
        Alias => 'BaseObject',
        ClassID => $ClassClassID,
        BaseClass => 'BASE',
        Package => 'DE_EPAGES::Object::API::Object::BaseObject',
        Attributes => [],
    } , {
        ObjectID => 4,
        Alias => 'Object',
        ClassID => $ClassClassID,
        BaseClass => 'BaseObject',
        Package => 'DE_EPAGES::Object::API::Object::Object',
        Attributes => [ {
            Alias => 'Alias',
            Type => 'String',
            Length => 255,
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsCacheable => 1,
            IsMandatory => 1,
        }, {
            Alias => 'Inherit',
            Type => 'Boolean',
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsCacheable => 1,
            IsMandatory => 1,
       }, {
            Alias => 'Position',
            Type => 'Integer',
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsCacheable => 1,
            IsMandatory => 1,
        }, {
            Alias => 'Class',
            Type => 'Class',
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsObject => 1,
            IsCacheable => 1,
            IsMandatory => 1,
        }, {
            Alias => 'Parent',
            Type => 'Object',
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsObject => 1,
            IsCacheable => 1,
        }, {
            Alias => 'GUID',
            Type => 'String',
            Length => 36,
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsCacheable => 1,
        }, {
            Alias => 'Site',
            Type => 'Site',
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsObject => 1,
            IsCacheable => 1,
        }, {
            Alias => 'CreationDate',
            Type => 'DateTime',
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsCacheable => 1,
        }, {
            Alias => 'Path',
            Type => 'String',
            Package => 'DE_EPAGES::Object::API::Attributes::Object_Ext',
            IsCacheable => 0,
        } ],
    } , {
        ObjectID => 5,
        Alias => 'Class',
        ClassID => $ClassClassID,
        BaseClass => 'Object',
        Package => 'DE_EPAGES::Object::API::Object::Class',
        Attributes => [ {
            Alias => 'BaseClass',
            Type => 'Class',
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsObject => 1,
            IsCacheable => 1,
        }, {
            Alias => 'Package',
            Type => 'String',
            Length => 255,
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsCacheable => 1,
        }, {
            Alias => 'ExportLevel',
            Type => 'String',
            Length => 255,
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsCacheable => 1,
        } ],
    } , {
        ObjectID => 6,
        Alias => 'Attribute',
        ClassID => $ClassClassID,
        BaseClass => 'Object',
        Package => 'DE_EPAGES::Object::API::Object::Attribute',
        Attributes => [ {
            Alias => 'AttributeClass',
            Type => 'Class',
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsCacheable => 1,
            IsObject => 1,
        }, {
            Alias => 'IsReadOnly',
            Type => 'Boolean',
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsCacheable => 1,
        }, {
            Alias => 'IsArray',
            Type => 'Boolean',
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsCacheable => 1,
        }, {
            Alias => 'IsExportable',
            Type => 'Boolean',
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsCacheable => 1,
         }, {
            Alias => 'IsCacheable',
            Type => 'Boolean',
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsCacheable => 1,
        }, {
            Alias => 'IsObject',
            Type => 'Boolean',
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsCacheable => 1,
        }, {
            Alias => 'IsMandatory',
            Type => 'Boolean',
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsCacheable => 1,
        }, {
            Alias => 'IsUserDefined',
            Type => 'Boolean',
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsCacheable => 1,
        }, {
            Alias => 'IsHTML',
            Type => 'Boolean',
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsCacheable => 1,
        }, {
            Alias => 'IsVisible',
            Type => 'Boolean',
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsCacheable => 1,
        }, {
            Alias => 'Type',
            Type => 'String',
            Length => 255,
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsCacheable => 1,
        }, {
            Alias => 'Length',
            Type => 'Integer',
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsCacheable => 1,
        }, {
            Alias => 'Package',
            Type => 'String',
            Length => 255,
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsCacheable => 1,
        }, {
            Alias => 'ExportLevel',
            Type => 'String',
            Length => 255,
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsCacheable => 1,
        }, {
            Alias => 'DefaultValue',
            Type => 'String',
            Length => 255,
            Package => 'DE_EPAGES::Object::API::BaseAttribute',
            IsCacheable => 1,
        } ],
    } , {
        ObjectID => 7,
        Alias => 'Site',
        ClassID => $ClassClassID,
        BaseClass => 'Object',
        Package => 'DE_EPAGES::Object::API::Object::Site',
        Attributes => [],
    } , {
        ObjectID => 8,
        Alias => 'System',
        ClassID => $ClassClassID,
        BaseClass => 'Site',
        Package => 'DE_EPAGES::Object::API::Object::Site',
        Attributes => [ {
            Alias => 'TimeZone',
            Name => 'TimeZone',
            Description => 'Store Time Zone',
            Type => 'String',
            Length => 255,
            IsExportable => 1,
            IsCacheable => 1,
            Package => 'DE_EPAGES::Object::API::Attributes::DefaultAttribute',
        } ],
    }, );

    Transaction( sub {
        my %Classes; # name => id
        foreach my $hClass ( @Classes ) {

            # insert the class
            my $ClassID = $ClassDAL->insert({
                ParentID    => $ClassesFolderID,
                ClassID     => $hClass->{ClassID}, # dummy
                ObjectID    => $hClass->{ObjectID},
                Alias       => $hClass->{Alias},
                Inherit     => 1,
                GUID        => GenerateGUID(),
                Position => 0,
                CreationDate => $Date,
                SiteID       => $SystemID,
                Package     => $hClass->{Package},
                BaseClassID => defined $hClass->{BaseClass} ? $Classes{$hClass->{BaseClass}} : undef,
            } );
            $Classes{$hClass->{Alias}} = $ClassID;
        }

        die "ObjectClassID != $ObjectClassID" unless $Classes{'Object'} == $ObjectClassID;

        foreach my $hClass (@Classes) {

            my $ClassID = $Classes{$hClass->{Alias}};

            # insert the 'Attributes' folder
            my $AttributesFolderID = $ObjectDAL->insert({
                ParentID    => $ClassID,
                ClassID     => $Classes{'Object'},
                Alias       => 'Attributes',
                Inherit     => 1,
                GUID        => GenerateGUID(),
                Position    => 0,
                CreationDate => $Date,
                SiteID       => $SystemID,
            });

            # insert attributes
            foreach my $hAttribute ( @{ $hClass->{Attributes} } ) {
                $AttributeDAL->insert({
                    ParentID    => $AttributesFolderID,
                    ClassID     => $Classes{'Attribute'},
                    Alias       => $hAttribute->{Alias},
                    Inherit     => 1,
                    GUID        => GenerateGUID(),
                    Position    => 0,
                    CreationDate => $Date,
                    SiteID       => $SystemID,
                    AttributeClassID => $ClassID,
                    Type    => $hAttribute->{Type},
                    Length  => $hAttribute->{Length},
                    Package => $hAttribute->{Package},
                    IsArray => 0,
                    IsReadOnly => 0,
                    IsExportable => 1,
                    IsCacheable => $hAttribute->{IsCacheable} ? 1 : 0,
                    IsObject => $hAttribute->{IsObject} ? 1 : 0,
                    IsMandatory => $hAttribute->{IsMandatory} ? 1 : 0,
                    IsUserDefined => 0,
                    IsVisible => 0,
                    IsHTML => 0,
                } );
            }
        }

        # update class of System Object
        $SiteDAL->update( $SystemID, { ClassID => $Classes{'System'} } );
    });
    ResetFactoryCaches();

    my $DatabaseName = GetCurrentDBHandle()->cacheKey;
    my $System = LoadRootObject();
    RunOnASPoolDB( sub { shift->insertStore( $DatabaseName, $System->get('GUID') ) } );
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::Object::API::Object::Object->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'BaseClass' => undef,
        'Package' => defined $hInfo->{'BaseClass'} ? $hInfo->{'BaseClass'}->packageObject : undef,
        'ExportLevel' => 1,
    };
}

#========================================================================================
# function     lastUpdateProfile
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Profile = lastUpdateProfile( $ProfileName );
# example      $Profile = $ProductClass->lastUpdateProfile('StockLevel');
#----------------------------------------------------------------------------------------
# description  Returns update Profile with given Name
#----------------------------------------------------------------------------------------
# input        $ProfileName | name of child in folder ./LastUpdateProfiles | string #
# return       $Profile | profile object | DE_EPAGES::Object::API::Object::LastUpdateProfile
#========================================================================================
sub lastUpdateProfile {
    my $class = shift;
    my ( $ProfileName ) = @_;
    return $class->folder('LastUpdateProfiles')->child($ProfileName);
}

1;
