#========================================================================================
# package      DE_EPAGES::Presentation::UI::Object
# state        public
#----------------------------------------------------------------------------------------
# description  Common user interface to show, save and delete objects.
#========================================================================================
package DE_EPAGES::Presentation::UI::Object;

use strict;
use HTTP::Date qw( time2str str2time );
use HTTP::Status qw( HTTP_NOT_MODIFIED );
use JSON qw( from_json );
use MIME::Base64;
use feature qw ( switch );
use Encode qw(decode encode);
use DE_EPAGES::Core::API::CSV qw( CombineCSV ParseCSV );
use DE_EPAGES::Core::API::Constants qw( PASSWORD_DUMMY );
use DE_EPAGES::Core::API::Error qw( ExistsError GetError Error ResetError );
use DE_EPAGES::Core::API::File qw( fsplit WriteFile ConvertPathChars ExistsFile RealPath ); # DeleteFile
use DE_EPAGES::Core::API::IDN qw ( EncodeDomain );
use DE_EPAGES::Core::API::Log qw( GetLog LogDebug );
use DE_EPAGES::Core::API::PerlTools qw( ElementOf IsA );
use DE_EPAGES::Core::API::TestParameter qw( TestHash TestObject TestString );
use DE_EPAGES::Core::API::Unicode qw (Unicode2Alias);
use DE_EPAGES::Core::API::Url qw (BuildParameterString);
use DE_EPAGES::Core::API::Warning qw( Warning );
use DE_EPAGES::Object::API::Domain qw ( GetUrlInfo );
use DE_EPAGES::Object::API::Factory qw( ExistsObject LoadClassByAlias Transaction );
use DE_EPAGES::Object::API::PageCache;
use DE_EPAGES::Permission::API::Permission qw (ActionByName);
use DE_EPAGES::Presentation::API::Object::PagerChildren;
use DE_EPAGES::Presentation::API::Object::Tray;
use DE_EPAGES::Presentation::API::PageType qw (LoadPageTypeByViewAction);
use DE_EPAGES::Presentation::API::Session qw( GetAdminSFSessionCookie SetSessionAttribute TestAdminSFSession );
use DE_EPAGES::Presentation::API::Upload qw ( Upload );
use DE_EPAGES::Trigger::API::BatchProcessor qw ( RunBatchProcess );
use DE_EPAGES::Trigger::API::Trigger qw( TriggerHook );

sub ErrorDefinition {
    return {
        'NoServlet' => 'DE_EPAGES::Presentation::UI::Object->new() requires a servlet.',
        'MoreValuesAsLanguages' => 'Form element #Name occurs #Index, but LanguageID only #CountLanguageIDs times.',
        'NoResultTemplateType' => 'TemplateType #TemplateTypeID does not exist.',
    }
}

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler = DE_EPAGES::Presentation::UI::Object->new(%options);
#----------------------------------------------------------------------------------------
# example      $Handler = DE_EPAGES::Presentation::UI::Object->new(Servlet => $Servlet);
#----------------------------------------------------------------------------------------
# description  create new instance of UI. This method will be called by servlet, there
#               is no need to call it again.
#----------------------------------------------------------------------------------------
# return       $Handler | UI handler object | DE_EPAGES::Presentation::UI::Object
# input        %options | required keys: Servlet | hash
#========================================================================================
sub new {
    my $class = shift;
    my (%options) = @_;

    # 327-674 For compatibility with 5.0.4 don't throw an error if there is no servlet
    Warning('NoServlet') unless defined $options{'Servlet'};
    return bless \%options, $class;
}

#========================================================================================
# function     servlet
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Servlet = $UI->servlet;
#----------------------------------------------------------------------------------------
# description  Returns the servlet, which calls the current action.
#----------------------------------------------------------------------------------------
# return       $Servlet | current servlet | object
#========================================================================================
sub servlet {
    return shift->{'Servlet'};
}

#========================================================================================
# function     object
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object = $UI->object;
#----------------------------------------------------------------------------------------
# description  Returns the object on which the current action operates.
#----------------------------------------------------------------------------------------
# return       $Object | object of the current request | object
#========================================================================================
sub object {
    return shift->{'Servlet'}->object;
}

#========================================================================================
# function     _servletClass
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->_servletClass($Servlet);
#----------------------------------------------------------------------------------------
# description  Return serlvet defined Class, reads servlet parameter Class or ClassID,
#               used to <function ViewNewObject> or create a new object.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# return       $Class | class | object (<package Object::API::Object::Class>)
#========================================================================================
sub _servletClass {
    my $self = shift;
    my ( $Servlet ) = @_;

    my $Site   = $Servlet->object()->getSite();
    my $Class  = $Servlet->params('Class');

    $Class = LoadClassByAlias($Class) if defined $Class and ref $Class eq '';
    if (defined $Class) {
        my $Form = $Servlet->form;
        $Form->{'Values'}->{'Class'} = [ $Class->id ];
        return $Class;
    }

    my $ClassID  = $Servlet->params('ClassID');
    die 'no class defined at servlet' unless defined $ClassID;
    return $Site->loadObject($ClassID, 'Class', { AllowSystemSite => 1 });
}

#========================================================================================
# function     View
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->View();
#----------------------------------------------------------------------------------------
# description  UI Action to view a object without any work.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub View {
    my $self = shift;
    my ($Servlet) = @_;

    my $Object = $Servlet->object;
    $Servlet->vars('THIS', $Object->tleHash);

    return;
}

#========================================================================================
# function     ViewCachedJS
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->ViewCachedJS();
#----------------------------------------------------------------------------------------
# description  UI Action to access the jQuery configuration page type. Adds long-term
#               cache header for caching the request in the browser cache.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub ViewCachedJS {
    my $self = shift;
    my ( $Servlet ) = @_;

    $Servlet->vars('ContentType', 'text/javascript');
    $Servlet->response->header('Last-Modified', time2str(0));
    $Servlet->response->header('Cache-Control','max-age=31536000,public'); #1 year
    return;
}

sub _SetSessionAttribute {
    my ($Servlet, $Key, $Value) = @_;

    my $IntSessionID = $Servlet->intSessionID;
    if(not defined $IntSessionID){
        $Servlet->createAnonymousUser($Servlet->object->getSite);
        $IntSessionID = $Servlet->intSessionID;
    }

    SetSessionAttribute($IntSessionID, $Key, $Value);
}

#========================================================================================
# function     SetDebugSession
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SetDebugSession();
#----------------------------------------------------------------------------------------
# description  UI Action to set / unset a JSDebugLevel to the user session
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub SetDebugSession {
    my $self = shift;
    my ( $Servlet ) = @_;

    my $DebugSession = $Servlet->params('Debug');
    my $DisableAdditionalHead = $Servlet->params('DisableAdditionalHead');

    _SetSessionAttribute($Servlet, 'JSDebugLevel', $DebugSession);
    _SetSessionAttribute($Servlet, 'DisableAdditionalHead', $DisableAdditionalHead);

    return;
}

#========================================================================================
# function     ViewCached
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->ViewCached( $Servlet );
#----------------------------------------------------------------------------------------
# description  UI Action to view a object, this function tries to save the processed
#               content at file system.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub ViewCached {
    my $self = shift;
    my ( $Servlet ) = @_;

    # use SSL or not
    $self->_handleRedirect($Servlet);

    my $Object = $Servlet->object;
    if( ! _IsCacheable( $Servlet, $Object ) ) {
        $self->View( $Servlet );
        return;
    }

    # get the PageCache object
    my $SitePageCache = DE_EPAGES::Object::API::PageCache->new( Site => $Object->getSite );

    # determine the name of the cache key
    my $CacheKey = _GetCacheKey( $Servlet );

    # get the response content from cache or re-build it
    my $IfModifiedSinceHeader = $Servlet->request->header('If-Modified-Since');
    my $IfModifiedSince = $IfModifiedSinceHeader ? str2time($IfModifiedSinceHeader) : undef;
    my ($Content, $LastModified) = $SitePageCache->compute( $Object, $CacheKey, $IfModifiedSince, sub {
        $self->View( $Servlet );
        return $Servlet->processContent( $Servlet->action ), 1;
    });

    # return no content if the page has not been modified since If-Modified-Since.
    if( !defined $Content ) {
        GetLog()->debug('304 not modified');
        $Servlet->response->code( HTTP_NOT_MODIFIED );
        $Servlet->params('ExitAfterEvent', 1);
        return;
    }

    $Servlet->response->header('Last-Modified' => time2str($LastModified));
    $Servlet->response->header('Cache-Control' => 'max-age='.$Object->pageCacheMaxAge); # seconds without modified since request

    $Servlet->writeContentOutput($Content);
    $Servlet->params('ExitAfterEvent', 1);
    return;
}

#========================================================================================
# function     _IsCacheable
# state        private
#----------------------------------------------------------------------------------------
# syntax       $IsCacheable = _IsCacheable( $Servlet, $Object );
#----------------------------------------------------------------------------------------
# description  Returns true if the current page is cacheable.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# input        $Object | object | DE_EPAGES::Object::API::Object::Object
# return       $IsCacheable | true if the current page is cacheable | string
#========================================================================================
sub _IsCacheable {
    my ($Servlet, $Object) = @_;

    my $Test =sub {
        my ($Message, $ConditionSub) = @_;

        if( $ConditionSub->() ) {
            return 1;
        }
        else {
            GetLog('ViewCached::Conditions')->debug(sprintf '%s is not cacheable, because: %s',
                $Object->pathString, $Message );
            return 0;
        }
    };

    if( $Test->('active session', sub { not $Servlet->intSessionID })
        && $Test->('form error', sub { not $Servlet->vars('FormError') })
        && $Test->('performance bar', sub { not $Servlet->params('PerformanceBar') })
        && $Test->('cookies', sub { 0 == scalar grep { !/^ShopInit|MerchantSF$/ } keys %{$Servlet->cookies} })
        && $Test->('editor mode', sub { not $Servlet->isEditorMode })
        && $Test->('page cache disabled', sub { $Object->pageCacheDuration })
        && $Test->('admin session', sub { TestAdminSFSession($Object->getSite, GetAdminSFSessionCookie()) != -1 }) # do not cache if shop is closed and admin session is invalid
        && $Test->('JSON snippet', sub { not $Servlet->params('IsJSONSnippet') })
    ) {
        GetLog('ViewCached::Conditions')->debug(sprintf '%s is cacheable', $Object->pathString);
        return 1;
    }
    else {
        return 0;
    }
}

#========================================================================================
# function     _GetCacheKey
# state        private
#----------------------------------------------------------------------------------------
# syntax       $CacheKey = _GetCacheKey( $Servlet );
#----------------------------------------------------------------------------------------
# description  Returns the name of the cache key relative to $Servlet->object.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# return       $CacheKey | cache key | string
#========================================================================================
sub _GetCacheKey {
    my ($Servlet) = @_;

    my $CacheKey = $Servlet->action->alias . '_' . $Servlet->locale;
    my $hParams = $Servlet->params;

    my $SSL = $hParams->{'SERVER_PORT_SECURE'} ? 1 : undef;
    $CacheKey .= "_ssl" if defined $SSL;
    my $Page = $hParams->{'Page'};
    $CacheKey .= "_page$Page" if defined $Page;
    my $PageSize = $hParams->{'PageSize'};
    $CacheKey .= "_pagesize$PageSize" if defined $PageSize;
    my $OrderBy = $hParams->{'OrderBy'};
    $CacheKey .= "_orderby$OrderBy" if defined $OrderBy;
    my $OrderDesc = $hParams->{'OrderDesc'};
    $CacheKey .= '_orderdesc' if $OrderDesc;
    return $CacheKey;
}

#========================================================================================
# function     ViewNewObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->ViewNewObject( $Servlet );
#----------------------------------------------------------------------------------------
# description  Returns the default attribute values for a new object.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# return       $hDefaults | default values | ref.hash
#========================================================================================
sub ViewNewObject {
    my $self = shift;
    my ($Servlet) = @_;

    my $Parent = $Servlet->object;
    my $Class  = $self->_servletClass($Servlet);
    return $self->defaultValues($Parent, $Class);
}

#========================================================================================
# function     testObjectDeleteListedObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PermissionObject = $Handler->testObjectDeleteListedObjects( $Object );
#----------------------------------------------------------------------------------------
# description  Returns the object to test the Delete permission (this function returns
#               the Object).
#----------------------------------------------------------------------------------------
# input        $Object | object to delete | object
# return       $PermissionObject | same object | object
#========================================================================================
sub testObjectDeleteListedObjects {
    my $self = shift;
    my ( $Object ) = @_;
    return $Object;
}

#========================================================================================
# function     DeleteListedObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->DeleteListedObjects( $Servlet, $Class);
#----------------------------------------------------------------------------------------
# description  Deletes the selected objects. The object ids are passed in the form field
#               'ListObjectID'.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# input        $Class | class of listed objects, required on object which are not in based on class Object (optional)| object
#========================================================================================
sub DeleteListedObjects {
    my $self = shift;
    my ( $Servlet, $Class ) = @_;

    # nothing to do
    return if ! $Servlet->params('ListObjectID');

    my $Form = $Servlet->form;
    my $hValues = $Form->form($Servlet->object, 'ListedObjects');
    $Class //= LoadClassByAlias('Object');

    my @ListObjectIDs = map { $_->{'ListObjectID'} }
        grep { $Class->existsObject( $_->{'ListObjectID'} ) }
        @{$hValues->{'ListObjectIDs'}};
    my $aListObjects = $Servlet->object->getSite->loadObjects( \@ListObjectIDs, $Class );

    # test permission
    foreach my $Index ( 0 .. scalar(@$aListObjects)-1 ) {
        my $Object = $aListObjects->[$Index];
        unless ( $Servlet->hasPermission($self->testObjectDeleteListedObjects($Object), 'Delete')) {
            $Form->addFormError( {
                Reason => 'PermissionDenied',
                Index => $Index,
                LoopName => 'ListObjectIDs',
                Form => 'ListedObjects',
                Name => 'ListObjectID'
            } );
            next;
        }
        my $aWarnings = [];
        $Object->canDelete($aWarnings);
        $Form->mapFormError({
            'Form'     => 'ListedObjects',
            'Name'     => 'ListObjectID',
            'Index'    => $Index,
            'LoopName' => 'ListObjectIDs',
        }, $_) foreach @$aWarnings;
    }
    $Form->executeFormError;

    # delete only if accepted if confirmation is used
    if ($Servlet->user->get('DeleteConfirmation') and not $Servlet->params('ACCEPT')) {
        $Servlet->vars('DeleteObjects', [ map { $_->tleHash } @$aListObjects]);
        $Servlet->vars('DELETE',1);
        return;
    } else {
        # else reset delete
        $Servlet->vars('DELETE',0);
    }

    # now delete objects
    RunBatchProcess(['DeleteObjects', 'DeleteCategories'], sub{
        foreach my $Object (@$aListObjects) {
            # skip child objects that are already deleted
            $Object->delete if $Object->exists;
        }
    });
    return;
}

#========================================================================================
# function     SetAttributeOnListedObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SetAttributeOnListedObjects( $Servlet, $AttributeName, $Value, $Permission);
#----------------------------------------------------------------------------------------
# description  sets the selected objects with given attribute value.
#               The object ids are passed in the form field 'ListObjectID'.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# input        $AttributeName | name of attribute | string
# input        $Value | new value for each object | *
# input        $Permission | name of action to test each listed object (default: 'Save') | string
#========================================================================================
sub SetAttributeOnListedObjects {
    my $self = shift;
    my ($Servlet, $AttributeName, $Value, $Permission) = @_;

    my $Object  = $Servlet->object();
    my $Site    = $Object->getSite();

    $Permission = 'Save' unless defined $Permission;
    my $hValues = $Servlet->form->form($Object, 'ListedObjects');

    my @Objects = grep { $Servlet->hasPermission($_, $Permission) }
                  map  { $Site->loadObject( $_->{'ListObjectID'}, 'Object' ) }
                  @{$hValues->{'ListObjectIDs'}};

    if (scalar @Objects > 0 ){
        RunBatchProcess('UpdateObjects', sub{
            $_->set({$AttributeName => $Value}) foreach @Objects;
            $Objects[0]->getSite->invalidatePageCache;
        });
    }
}

#========================================================================================
# function     SetInvisibleListedObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SetInvisibleListedObjects( $Servlet );
#----------------------------------------------------------------------------------------
# description  sets the selected objects invisible. The object ids are passed in the form field
#               'ListObjectID'.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub SetInvisibleListedObjects {
    my $self = shift;
    my ( $Servlet ) = @_;

    $self->SetAttributeOnListedObjects($Servlet, 'IsVisible', 0, 'SetVisible');
    return;
}

#========================================================================================
# function     SetVisibleListedObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SetVisibleListedObjects( $Servlet );
#----------------------------------------------------------------------------------------
# description  sets the selected objects visible. The object ids are passed in the form field
#               'ListObjectID'.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub SetVisibleListedObjects {
    my $self = shift;
    my ( $Servlet ) = @_;

    $self->SetAttributeOnListedObjects($Servlet, 'IsVisible', 1, 'SetVisible');
    return;
}

#========================================================================================
# function     checkRequiredClass
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->checkRequiredClass($Servlet, $Class);
#----------------------------------------------------------------------------------------
# description  Helper method for NewObject(). Checks whether the required class of
#               the parent object allows the created object's class.
#----------------------------------------------------------------------------------------
# input        $Form | servlet form | object
# input        $Parent | (optional) parent object | object
# input        $Class | (optional) class of the new object | object
# return       0 or 1 | 0 if the class is not allowed, otherwise 1 | boolean
#========================================================================================
sub checkRequiredClass() {
    my $self = shift;
    my ($Form, $Parent, $Class) = @_;
    # check if class is allowed as a child object of parent
    my $RequiredClass = $Parent->requiredChildClass;
    if (not defined $RequiredClass) {
        # undef means no children allowed
        $Form->addFormError({
            'Form'=> 'NewObject',
            'Reason'=> 'NO_CHILD_OBJECTS_ALLOWED',
            'Name'  => 'Class',
            'Value' => $Class->get('Alias'),
        });
        $Form->executeFormError;
        return 0;
    } elsif (not $Class->isSubClassOf($RequiredClass)) {
        $Form->addFormError({
            'Form'=> 'NewObject',
            'Reason'=> 'CHILD_CLASS_NOT_ALLOWED',
            'Name'  => 'Class',
            'Value' => $Class->get('Alias'),
        });
        $Form->executeFormError;
        return 0;
    }
    return 1;
}

#========================================================================================
# function     NewObject
# state        public
#----------------------------------------------------------------------------------------
# syntax       $NewObject = $Handler->NewObject( $Servlet );
# syntax       $NewObject = $Handler->NewObject( $Servlet, $Parent );
# syntax       $NewObject = $Handler->NewObject( $Servlet, $Parent, $Class );
# syntax       $NewObject = $Handler->NewObject( $Servlet, $Parent, $Class, $hAttributes );
#----------------------------------------------------------------------------------------
# description  Creates a new object and sets the attribute values.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# input        $Parent | (optional) parent object | object
# input        $Class | (optional) class of the new object | object
# input        $hAttributes | (optional) attribute values to override form values | ref.hash
# return       $NewObject | the created object | object
#========================================================================================
sub NewObject {
    my $self = shift;
    my ($Servlet, $Parent, $Class, $hAttributes) = @_;

    $Parent = $Servlet->object unless defined $Parent;
    $Class  = $self->_servletClass($Servlet) unless defined $Class;
    my $Form = $Servlet->form;

    return if !$self->checkRequiredClass($Form, $Parent, $Class);

    # validate form data
    my $hhValues = $self->attributeValues( $Servlet, $Parent, $Class );
    $Servlet->form->executeFormError;

    if( defined $hAttributes ) {
        # copy override values
        $hhValues->{Values}->{$_} = $hAttributes->{$_} foreach keys %$hAttributes;
    }

    unless( grep { $_->{'Name'} eq 'Alias' } @{$Servlet->form->formErrors} ) {
        my $NewAlias = $hhValues->{'Values'}->{'Alias'};
        if (not defined $NewAlias or $NewAlias eq '') {
            # no alias, so generate one from unicode and make sure it's unique
            $NewAlias = $Parent->uniqueChildAlias(Unicode2Alias($Servlet->form->value('Name')));
            $hhValues->{'Values'}->{'Alias'} = $NewAlias;
        } elsif( $Parent->existsChild($NewAlias) ) {
            # check if another object exists with the same alias
            $Servlet->vars('OBJECT_EXISTS',1);
            $Servlet->form->addFormError({
                'Form'=> 'NewObject',
                'Reason'=> 'OBJECT_EXISTS',
                'Name'  => 'Alias',
                'Value' => $NewAlias,
            });
            $Servlet->form->executeFormError;
        }
    }

    if (not defined $hhValues->{'Values'}->{'Position'}) {
            $hhValues->{'Values'}->{'Position'} = $Parent->nextPosition();
    }

    # Parent is required for insertObject()
    $hhValues->{Values}->{Parent} = $Parent;

    my $NewObject;
    Transaction( sub {
        LogDebug( 'insert object', $hhValues->{'Values'} );
        eval{
            $NewObject = $Class->insertObject( $hhValues->{Values} );
        };
        if (ExistsError()) {
            my $Error = GetError();
            Error($Error) unless $Error->code eq 'FeatureLimitExceeded';
            $Servlet->form->mapFormError({
                'Form'    => 'NewObject',
                'Message' => $Error->message,
                'Name'    => 'Alias',
                'Value'   => $hhValues->{'Values'}->{'Alias'},
            }, $Error);
            $Servlet->form->executeFormError;
        }

        LogDebug( 'set localized attributes', $hhValues->{'LocalizedValues'} );
        foreach my $LanguageID ( keys %{$hhValues->{'LocalizedValues'}} ) {
            $NewObject->set( $hhValues->{'LocalizedValues'}{$LanguageID},  $LanguageID );
        }
    });

    $self->saveFileAttributes( $Servlet, $NewObject, $hhValues );
    $NewObject->updatePageCache;

    return $NewObject;
}

#========================================================================================
# function     Delete
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->Delete( $Servlet );
#----------------------------------------------------------------------------------------
# description  Deletes the current object ($Servlet->object).
#               <br/>
#               Discards the operation if user enables DeleteConfirmation and not
#               $Servlet->params('ACCEPT'). Sets the $Servlet->vars('DELETE',1) in this
#               case.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# return       $Parent | parent object of $Servlet->object | object
#========================================================================================
sub Delete {
    my $self = shift;
    my ( $Servlet ) = @_;

    my $Object = $Servlet->object;
    my $aWarnings = [];

    $Object->canDelete($aWarnings);
    $Servlet->form->mapFormError({'Form'     => 'Delete',
                                  'Name'     => 'Delete',
                                 }, $_) foreach @$aWarnings;
    $Servlet->form->executeFormError;

    # delete only if accepted if confirmation is used
    if ($Servlet->user->get('DeleteConfirmation') and not $Servlet->params('ACCEPT')) {
        # set delete mark to template
        $Servlet->vars('DELETE',1);
        return;
    } else {
        # else reset delete
        $Servlet->vars('DELETE',0);
    }

    my $Parent = $Object->parent;
    eval {
        RunBatchProcess(['DeleteObjects', 'DeleteCategories'], sub{
            $Object->delete;
        });
    };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) unless $Error->code eq 'NoResults_ObjectID';
    }
    return $Parent;
}

#========================================================================================
# function     SetInvisible
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SetInvisible( $Servlet );
#----------------------------------------------------------------------------------------
# description  sets the object invisible.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub SetInvisible {
    my $self = shift;
    my ( $Servlet ) = @_;

    $Servlet->object->set({'IsVisible' => 0});
    return;
}

#========================================================================================
# function     SetVisible
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SetVisible( $Servlet );
#----------------------------------------------------------------------------------------
# description  sets the object visible.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub SetVisible {
    my $self = shift;
    my ( $Servlet ) = @_;

    $Servlet->object->set({'IsVisible' => 1});
    return;
}

#========================================================================================
# function     DeleteFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->DeleteFile( $Servlet );
#----------------------------------------------------------------------------------------
# description  Deletes a file from the public directory of the current object..
#               The attribute name of the file attribute is passed as form field with
#               the name "File". The attribute value is set to undef.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub DeleteFile {
    my $self = shift;
    my ( $Servlet ) = @_;

    my $AttributeName = $Servlet->params('File');
    $self->_deleteFileAndUnsetAttribute($Servlet, $AttributeName);
}

#========================================================================================
# function     DeleteMultipleFiles
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->DeleteMultipleFiles( $Servlet );
#----------------------------------------------------------------------------------------
# description  Behaves like DeleteFile, but deletes a list of files via a given List of
#               attribute names
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub DeleteMultipleFiles {
    my $self = shift;
    my ( $Servlet ) = @_;

    my @AttributeNames = split(/,/, $Servlet->params('Files'));
    foreach my $AttributeName (@AttributeNames) {
        $self->_deleteFileAndUnsetAttribute($Servlet, $AttributeName);
    }
}

sub _deleteFileAndUnsetAttribute {
    my $self = shift;
    my ($Servlet, $AttributeName) = @_;

    $Servlet->params('DELETE_FILE', $AttributeName);
    $Servlet->params('ChoiceID', $Servlet->params('ChoiceID'));
    return if $Servlet->user->get('DeleteConfirmation') and not $Servlet->params('ACCEPT');
    $Servlet->params('DELETE_FILE', undef);

    my $Object = $Servlet->object;
    my $DeleteLanguageID = $Servlet->params('DeleteLanguageID');
    my $LanguageID = defined $DeleteLanguageID && $DeleteLanguageID =~/\S/
        ? $DeleteLanguageID
        : $Servlet->params('LanguageID');
    my $FileName = $Object->get($AttributeName, $LanguageID);
    return unless defined $FileName;

    my $Shop = $Object->getSite();
    my $hLanguages = $Shop->get('Languages');
    my $DeleteFile = 1;
    if ($Object->class->attribute($AttributeName)->get('Type') eq 'LocalizedFile') {
        foreach my $hLanguage (@$hLanguages) {
            my $TranslationLanguageID = $hLanguage->{'LanguageID'};
            my $TranslationFileName = ($TranslationLanguageID != $LanguageID) ?
                $Object->get($AttributeName, $TranslationLanguageID) :
                undef;
            $DeleteFile = 0 if (defined $TranslationFileName and $TranslationFileName eq $FileName);
        }
    }

    $Object->set({$AttributeName => undef}, $LanguageID);
    DE_EPAGES::Core::API::File::DeleteFile($Object->get('PublicPath').'/'.$FileName) if $DeleteFile;
     # EPG-EPG-10737:
    $Shop->invalidatePageCache;
    return;
}

#========================================================================================
# function     Save
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->Save( $Servlet );
#----------------------------------------------------------------------------------------
# description  Saves the attributes of the current object ($Servlet->object).
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub Save {
    my $self = shift;
    my ( $Servlet ) = @_;

    my $Object = $Servlet->object;
    my $Class = $Object->class;

    my $FormClassID = $Servlet->form->value('Class');
    if( defined $FormClassID ) {
        $Class = $Object->getSite()->loadObject($FormClassID, 'Class', { AllowSystemSite => 1 });
    }

    my $hhValues = $self->attributeValues( $Servlet, $Object, $Class );

    # check if another object exists with the same alias
    if( exists $hhValues->{'Values'}->{'Alias'}
        && ! grep { $_->{'Name'} eq 'Alias' } @{$Servlet->form->formErrors} )
    {
        my $NewAlias = $hhValues->{'Values'}->{'Alias'};
        if( $Object->alias !~ /^\Q$NewAlias\E$/i && $Object->parent->existsChild($NewAlias) ) {
            $Servlet->vars('OBJECT_EXISTS',1);
            $Servlet->form->addFormError({
                'Form'  => 'Save',
                'Reason'=> 'OBJECT_EXISTS',
                'Name'  => 'Alias',
                'Value' => $NewAlias,
            });
        }
    }

    # die with INVALID_FORM error if any field is invalid
    $Servlet->form->executeFormError;

    Transaction( sub {
        LogDebug( 'set attributes', $hhValues->{'Values'} );
        $Object->set( $hhValues->{'Values'} );

        LogDebug( 'set localized attributes', $hhValues->{'LocalizedValues'} );
        foreach my $LanguageID ( keys %{$hhValues->{'LocalizedValues'}} ) {
            $Object->set( $hhValues->{'LocalizedValues'}{$LanguageID},  $LanguageID );
        }
    });

    $self->saveFileAttributes( $Servlet, $Object, $hhValues );
    $Object->updatePageCache;
    return;
}

#========================================================================================
# function     saveFileAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->saveFileAttributes( $Servlet, $Object, $hhValues );
#----------------------------------------------------------------------------------------
# description  Saves attribute value of type File and LocalizedFile.
#               Saves the files to the directory $Object->get('PublicPath').
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# input        $Object | object | object
# input        $hhValues | checked attribute vlaues, result of
#                           <function attributeValues>
#                       | ref.hash.hash
# input        $Prefix (optional) | prefix string of the inputnames | string
#========================================================================================
sub saveFileAttributes {
    my $self = shift;
    my ( $Servlet, $Object, $hhValues, $Prefix ) = @_;
    TestObject( 'Servlet', $Servlet );
    TestObject( 'Object', $Object );
    TestHash( 'hhValues', $hhValues );

    if(defined $Prefix) {
        TestString( 'Prefix', $Prefix );
    } else {
        $Prefix = '';
    }

    # save non-localized files
    foreach my $Name ( keys %{$hhValues->{Files}} ) {

        my $Value = $hhValues->{Files}->{$Name};

        if( defined $Value ) {
            $Value = $self->saveFile($Servlet, $Object, $Prefix.$Name);
            $Servlet->form->executeFormError;
        }
        else {
            $self->removeFile($Servlet, $Object, $Name);
        }

        $Object->set({ $Name => $Value });
    }

    # save localized files
    foreach my $LanguageID ( keys %{$hhValues->{LocalizedFiles}} ) {
        my $hFiles = $hhValues->{LocalizedFiles}->{$LanguageID};

        foreach my $Name ( keys %$hFiles ) {

            my $Value = $hFiles->{$Name}{'Value'};
            my $Index = $hFiles->{$Name}{'Index'};

            if( defined $Value ) {
                $Value = $self->saveFile($Servlet, $Object, $Prefix.$Name, $Index);
                $Servlet->form->executeFormError;
            }
            else {
                $self->removeFile($Servlet, $Object, $Name);
            }

            $Object->set({ $Name => $Value }, $LanguageID);
        }
    }
}

#========================================================================================
# function     defaultValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hValues = $Handler->defaultValues($Parent, $Class);
#----------------------------------------------------------------------------------------
# description  Returns the default attribute values for new objects of a specified class
#               as TLE hash.
#----------------------------------------------------------------------------------------
# input        $Parent | parent object | object
# input        $Class | class | object
# return       $hValues | default attribute values | ref.hash.hash
#========================================================================================
sub defaultValues {
    my $self = shift;
    my ($Parent, $Class) = @_;
    TestObject('Parent', $Parent);
    TestObject('Class', $Class);

    my $hValues = $Class->allDefaultAttributes($Parent);
    foreach my $Key (keys %$hValues) {
        if( IsA( $hValues->{$Key}, 'DE_EPAGES::Object::API::Object::Object' ) ) {
            $hValues->{$Key} = $hValues->{$Key}->tleHash;
        }
    }
    delete $hValues->{'ObjectID'};
    return $hValues;
}

#========================================================================================
# function     attributeValues
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hhValues = $Handler->attributeValues( $Servlet, $Object, $Class, $Prefix );
#----------------------------------------------------------------------------------------
# example      $hhValues = $Handler->attributeValues( $Servlet, $Object, $Class, 'Address.' );
#               $Form->executeFormError;
#----------------------------------------------------------------------------------------
# description  Checks the form values against the type and range specification of the
#               given class and the formatters of the given object.<br>
#               Note: Call $Servlet->form->executeFormError() to abort processing in
#               case of input errors.<br>
#               You can override this function to perform additional tests. To call the
#               default implementation, use $self->SUPER::attributeValues(@_).
#----------------------------------------------------------------------------------------
# input        $Object | object that specifies the context for formatters | object
# input        $Class | class that specifies type and range of the attributes | object
# input        $Object | object that specifies the context for formatters. For existing
#                           objects this is the object that is to be saved; for new
#                           objects this may be a direct or indirect parent object.
#                        | object
# input        $Prefix | prefix for input field names (default '') | string
# return       $hhValues | checked attribute values
#                   <ul>
#                       <li>Values - non-localized attribute values execpt files
#                                   { $AttributeName => $Value }</li>
#                       <li>Files - non-localized attribute values of type File
#                                   { $AttributeName => $FileName }
#                                   if the file name is undef, then the file is about to be deleted</li>
#                       <li>LocalizedValues - localized attribute values execpt files
#                                   { $LanguageID => { $AttributeName => {'Value'=>$Value, 'Index'=>$Index } }</li>
#                       <li>LocalizedFiles - localized attribute values of type File
#                                   { $LanguageID => { $AttributeName => $FileName } }
#                                   if the file name is undef, then the file is about to be deleted</li>
#                   </ul>
#                   | ref.hash.hash
#========================================================================================
sub attributeValues {
    my $self = shift;
    my ( $Servlet, $Object, $Class, $Prefix ) = @_;
    TestObject( 'Servlet', $Servlet );
    TestObject( 'Object', $Object );
    TestObject( 'Class', $Class );
    $Prefix = '' unless defined $Prefix;

    my $Form = $Servlet->form;
    my @LanguageIDs = $Form->value('LanguageID');
    @LanguageIDs = ( $Servlet->languageID ) unless scalar @LanguageIDs;
    my $aAttributes = $Class->attributes;

    LogDebug( 'language ids', \@LanguageIDs );

    # construct form field definitions from attribute definitions

    my $hhValues = {
        'Values' => {},
        'Files' => {},
        'LocalizedValues' => {},
        'LocalizedFiles' => {},
    };

    foreach my $Attribute (@$aAttributes) {
        next if $Attribute->get('IsReadOnly');
        my $Type = $Attribute->get('Type');

        next if $Attribute->get('IsArray') and not ($Type eq 'PreDefMultiString' or $Type eq 'PreDefMultiLocalizedString');

        my $AttributeAlias = $Attribute->alias;

        # LanguageID is used internally by this function, but it's also an
        # attribute of some objects
        next if $AttributeAlias eq 'LanguageID';

        my $ElementName = $Prefix . $AttributeAlias;
        # save only those attribute that exist in the form
        next unless $Form->existsValue( $ElementName );

        _fillValues($Form, $Object, $Attribute, $ElementName, \@LanguageIDs, $hhValues);
    }

    LogDebug( 'checked values', $hhValues );
    return $hhValues;
}

#========================================================================================
# function     saveFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FileName = $Handler->saveFile( $Servlet, $Object, $Name );
# syntax       $FileName = $Handler->saveFile( $Servlet, $Object, $Name, $Index );
#----------------------------------------------------------------------------------------
# description  Saves an uploaded file on the server. Returns the file name relative
#               to object directory $Object->get('PublicPath').
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
# input        $Object | the object, to which the image belongs (as attribute) | object
# input        $Name | form field name | string
# input        $Index | (optional) form field index if the form contains multiple
#                       fields with the same name | integer
# return       $FileName | relative file name | string
#========================================================================================
sub saveFile {
   my $self = shift;
   my ($Servlet, $Object, $Name, $Index) = @_;

   $Index = 0 unless defined $Index;

   my ($AttributeValue, $hResponseContent) = Upload($Servlet->object, $Servlet->form, {'InputName' => $Name, 'Index' => $Index, 'Object' => $Object});
   $Servlet->vars('ResponseContent', $hResponseContent);

   return $AttributeValue;
}

#========================================================================================
# function     savePrivateFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FileName = $Handler->savePrivateFile( $Servlet, $Object, $Name );
# syntax       $FileName = $Handler->savePrivateFile( $Servlet, $Object, $Name, $Index );
#----------------------------------------------------------------------------------------
# description  Saves an uploaded file on the server. Returns the file name relative
#               to object directory $Object->get('PrivatePath').
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
# input        $Object | the object, to which the image belongs (as attribute) | object
# input        $Name | form field name | string
# input        $Index | (optional) form field index if the form contains multiple
#                       fields with the same name | integer
# return       $FileName | relative file name | string
#========================================================================================
sub savePrivateFile {
    my $self = shift;
    my ($Servlet, $Object, $Name, $Index) = @_;
    $Index = 0 unless defined $Index;

    my $Form = $Servlet->form;
    my @FileNames = $Form->value($Name);
    my $FilePath = $FileNames[$Index];

    return undef unless defined $FilePath;  # file input field does not exist in form
    return undef unless $FilePath =~ /\S/;  # no file name was submitted

    my $rContent = \$Form->files->{$Name}->[$Index];
    return undef unless length $$rContent;    # no file content was submitted

    # build filename and attribute value
    my (undef, $FName, $Extension) = fsplit($FilePath);
    $FName = ConvertPathChars($FName);
    my $AttributeValue = $FName.$Extension;
    my $PrvatePath = $Object->get('PrivatePath');

    WriteFile( $PrvatePath.'/'.$AttributeValue, $rContent, { binary=>1, create_directory=>1 } );

    TriggerHook('UI_AfterUploadWriteFile',{
        'Shop' => $Object->getSite(),
        'FileNameCustomer' => $FName.$Extension,
        'FileNameEPages' => $FName.$Extension,
        'FilePath' => $PrvatePath
    });

    return $AttributeValue;
}

#========================================================================================
# function     removeFile
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->removeFile( $Servlet, $Object, $Name );
# syntax       $Handler->removeFile( $Servlet, $Object, $Name, $LanguageID );
#----------------------------------------------------------------------------------------
# description  Deletes a file from the server.
#----------------------------------------------------------------------------------------
# input        $Object | the object, to which the image belongs (as attribute) | object
# input        $Servlet | current servlet (contains form data) | object
# input        $Name | attribute name | string
# input        $LanguageID | (optional) language id | integer
#========================================================================================
sub removeFile {
    my $self = shift;
    my ($Servlet, $Object, $Name, $LanguageID) = @_;

    my $FileName = $Object->get('PublicPath') .'/'. $Object->get($Name, $LanguageID);
    DE_EPAGES::Core::API::File::DeleteFile( $FileName );
    return;
}

#========================================================================================
# function     ViewPagedChildren
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hVars = $Handler->ViewPagedChildren( $Servlet );
#----------------------------------------------------------------------------------------
# description  Creates a pager for child objects of $Servlet->object.
#               The default sort order is by Position, ascending.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
# return       $hVars | TLE variables for 'Pager' | ref.hash
#========================================================================================
sub ViewPagedChildren {
    my $self = shift;
    my ( $Servlet ) = @_;

    my $Pager = DE_EPAGES::Presentation::API::Object::PagerChildren->new({
        'Servlet' => $Servlet,
        'Object' => $Servlet->object
    });
    return $Pager->page;
}

#========================================================================================
# function     AddToTray
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->AddToTray( $Servlet );
#----------------------------------------------------------------------------------------
# description  Adds current object to the tray (via <function DE_EPAGES::Presentation::API::Object::Tray::addObjectsToClassGroup>).
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub AddToTray {
    my $self = shift;
    my ( $Servlet ) = @_;

    return unless defined $Servlet->intSessionID;
    my $Object = $Servlet->object;
    my $Tray = DE_EPAGES::Presentation::API::Object::Tray->load($Servlet->intSessionID);
    $Tray->addObjectsToClassGroup([$Object]);
    return;
}

#========================================================================================
# function     AddListToTray
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->AddListToTray( $Servlet );
#----------------------------------------------------------------------------------------
# description  Adds objects (listed by form Object.ListedObjects) to the tray
#               (via <function DE_EPAGES::Presentation::API::Object::Tray::addObjectsToClassGroup>).
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub AddListToTray {
    my $self = shift;
    my ( $Servlet ) = @_;

    return unless defined $Servlet->intSessionID;
    my $Object = $Servlet->object;
    my $Site = $Object->getSite;

    my $hValues = $Servlet->form->form($Object, 'ListedObjects');
    my @ObjectIDs = map { $_->{'ListObjectID'} } @{$hValues->{'ListObjectIDs'}};
    my $aObjects = $Site->loadObjects( \@ObjectIDs, 'Object' );

    my $Tray = DE_EPAGES::Presentation::API::Object::Tray->load($Servlet->intSessionID);
    $Tray->addObjectsToClassGroup($aObjects);
    return;
}

#========================================================================================
# function     DeleteTray
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->DeleteTray( $Servlet );
#----------------------------------------------------------------------------------------
# description  Removes tray of current session.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub DeleteTray {
    my $self = shift;
    my ($Servlet) = @_;

    return unless defined $Servlet->intSessionID;
    my $Tray = DE_EPAGES::Presentation::API::Object::Tray->load($Servlet->intSessionID);
    $Tray->delete;
    return;
}

#========================================================================================
# function     DeleteTrayGroup
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->DeleteTrayGroup( $Servlet );
#----------------------------------------------------------------------------------------
# description  Removes tray group (indentified via form value 'TrayGroupID') of current session.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub DeleteTrayGroup {
    my $self = shift;
    my ($Servlet) = @_;

    return unless defined $Servlet->intSessionID;
    my ($GroupID) = $Servlet->form->value('TrayGroupID');
    my $Tray = DE_EPAGES::Presentation::API::Object::Tray->load($Servlet->intSessionID);
    $Tray->deleteGroup($GroupID);
    return;
}

#========================================================================================
# function     BlockMonitor
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->BlockMonitor( $Servlet );
#----------------------------------------------------------------------------------------
# description  Blocks appearance of monitor.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub BlockMonitor {
    my $self = shift;
    my ($Servlet) = @_;

    # block monitor (no monitor of monitor)
    $Servlet->monitor->blockTimeOut;
    return;
}

#========================================================================================
# function     Monitor
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->Monitor( $Servlet );
#----------------------------------------------------------------------------------------
# description  Reads the response of an previous appeared (form value GUID) monitor.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub Monitor {
    my $self = shift;
    my ($Servlet) = @_;

    $self->BlockMonitor($Servlet);

    # update current reponse with old response
    my ($GUID) = $Servlet->form->value('GUID');
    eval {
        $Servlet->monitor->readResponse($Servlet->response, $GUID);
    };
    if (ExistsError()) {
        my $Error = GetError();
        Error ($Error) unless $Error->code eq 'FILE_NOT_FOUND';
        $Servlet->handleMonitorNotFound($Error);
        ResetError();
    }
    $Servlet->params('ExitAfterEvent', 1);
    return;
}

#========================================================================================
# function     Button
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->Button( $Servlet );
#----------------------------------------------------------------------------------------
# description  This function calls a new UI function by a pressed button of an html-form.
#               Which buttons are allowed, is defined by form value ValidButton. No java-script
#               is needed to select the real ChangeAction.
#----------------------------------------------------------------------------------------
# example      <form href="?ChangeAction=Button">
#                 <input type="hidden" name="ValidButton" value="Save" />
#                 <input type="hidden" name="ValidButton" value="Delete" />
#                 <input type="submit" name="Save" value="Save This" />
#                 <input type="submit" name="Delete" value="Delete This" />
#               </form>
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub Button {
    my $self = shift;
    my ($Servlet) = @_;

    my $Form = $Servlet->form;
    my @ValidButtons = $Form->value('ValidButton');
    foreach my $Button (@ValidButtons) {
        if( $Form->existsValue($Button) or $Form->existsValue($Button .'.x') ) {
            $Servlet->doAction($Button);
            return;
        }
    }
    return;
}

#========================================================================================
# function     SetCookiePopUp
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SetCookiePopUp( $Servlet );
#----------------------------------------------------------------------------------------
# description  This function sets the session cookie PopUp.
#----------------------------------------------------------------------------------------
# input        $Servlet | current servlet (contains form data) | object
#========================================================================================
sub SetCookiePopUp {
    my $self = shift;
    my ( $Servlet ) = @_;

    my $Path = $Servlet->serverVariables->{'SCRIPT_NAME'}.$Servlet->serverVariables->{'PATH_INFO'};
    $Servlet->cookies('PopUp', 1, { Path => $Path } );
    return;
}

#========================================================================================
# function     SerializeForm
# state        private
#----------------------------------------------------------------------------------------
# syntax       $SerializedForm = SerializeForm( $haForm );
#----------------------------------------------------------------------------------------
# description  Serializes form fields as a string.
#----------------------------------------------------------------------------------------
# input        $haForm | form fields | hash.array.scalar
# return       $SerializedForm | serialized form data | string
#========================================================================================
sub SerializeForm ($) {
    my ($haForm) = @_;

    my @Lines;
    foreach my $Key (keys %$haForm) {
        my @Values = ($Key, @{$haForm->{$Key}});
        push @Lines, CombineCSV( \@Values, {'sep_char' => '|'});
    }

    return encode_base64(encode('utf8', CombineCSV( \@Lines, {'sep_char' => '~'})));
}

#========================================================================================
# function     DeserializeForm
# state        private
#----------------------------------------------------------------------------------------
# syntax       $haForm = DeserializeForm( $SerializedForm );
#----------------------------------------------------------------------------------------
# description  Deserializes the given string to form fields.
#----------------------------------------------------------------------------------------
# input        $SerializedForm | serialized form string, created by <function SerializeForm> | string
# return       $haForm | form fields | hash.array.scalar
#========================================================================================
sub DeserializeForm ($) {
    my ($SerializedForm) = @_;

    my $aLines = ParseCSV(decode('utf8', decode_base64($SerializedForm)), {'sep_char' => '~'});

    my %hValues;
    foreach my $Line (@$aLines) {
        my $aValues = ParseCSV($Line, {'sep_char' => '|'});
        my $Key = shift @$aValues;
        $hValues{$Key} = $aValues;
    }

    return \%hValues;
}

#========================================================================================
# function     MergeForm
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->MergeForm( $Servlet );
#----------------------------------------------------------------------------------------
# description  Merges the serialized values of form field "SerializedForm" to the
#               current Form and Servlet vars. This is very useful for wizards with
#               unknown form element names. Last submitted values and default values like
#               ObjectID, ViewAction, etc. are not overwritten or added.
#               You have to submit the last (serialized) form via #INPUT.SerializedForm
#               on each page using this action.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub MergeForm {
    my $self = shift;
    my ( $Servlet ) = @_;

    my $SerializedForm = $Servlet->form->formValue('SerializedForm');
    my @ForbiddenElements = qw (ObjectID ViewObjectID ChangeObjectID ViewAction ChangeAction
                                ErrorAction ErrorViewAction LoginAction BatchAction SerializedForm );

    # add old form values from serialized data to form vars
    my $hValues = $Servlet->form->formValues;
    if (defined($SerializedForm)) {
        my $hSerializedForm = DeserializeForm($SerializedForm);
        foreach my $Key (grep { /\S/ } keys %$hSerializedForm) {
            $hValues->{$Key} = $hSerializedForm->{$Key} unless (ElementOf($Key, @ForbiddenElements) or exists $hValues->{$Key});
        }
    }
    # add new submitted values
    my $hNewSerializedForm = {};
    $hNewSerializedForm->{$_} = $hValues->{$_} foreach grep {not ElementOf($_, @ForbiddenElements)} keys %$hValues;
    $Servlet->vars('SerializedForm', SerializeForm($hNewSerializedForm));
    return;
}

#========================================================================================
# function     SaveDesign
# state        private
#----------------------------------------------------------------------------------------
# syntax       SaveDesign( Servlet );
#----------------------------------------------------------------------------------------
# description  Save design choosen in MBO
#----------------------------------------------------------------------------------------
# input        $Servlet | current Servlet | object
#========================================================================================
sub SaveDesign {
    my $self = shift;
    my ($Servlet) = @_;

    my $Object = $Servlet->object;
    my $PageType = LoadPageTypeByViewAction( ActionByName( $Object->class, 'View' ) );
    foreach my $TemplateTypeFolder (@{$PageType->templateTypeFolders}){
        my $TemplateName = $TemplateTypeFolder->alias;
        my $TemplateTypeName = $Servlet->form->value($TemplateName);
        next unless defined $TemplateTypeName;
        $PageType->setTemplateType( $Object, $TemplateName, $TemplateTypeName );
    }
    $Object->updatePageCache;

    return;
}


#========================================================================================
# function     SaveOnList
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SaveOnList( $Servlet, $Name, Value );
# example      $self->SaveOnList($Servlet, "IsVisible", 1);
#----------------------------------------------------------------------------------------
# description  Changes the attribute given with the $Name parameter to the value given
#               in the $Value parameter for all objects in the ListedObjects form
#               contained in the $Servlet object.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet containing the ListedObject form | object
# input        $Name | name of the attribute | string
# input        $Value | value to wchich the attribute will be set| depends on $Name
#========================================================================================
sub SaveOnList {
    my $self = shift;
    my ($Servlet, $Name, $Value ) = @_;

    my $Object  = $Servlet->object();
    my $Site    = $Object->getSite();
    my $hValues = $Servlet->form->form($Object, 'ListedObjects');

    my @ListObjects;

    foreach my $hListObjectID (@{$hValues->{'ListObjectIDs'}}){
        my $ListObject = $Site->loadObject( $hListObjectID->{'ListObjectID'}, 'Object');
        $Servlet->testPermission($ListObject, 'Save');
        push @ListObjects,$ListObject;
    }
    if(scalar(@ListObjects) > 0) {
        $_->set( { $Name => $Value } ) foreach @ListObjects;
        # EPG-10681,EPG-11046:
        $ListObjects[0]->getSite()->invalidatePageCache;
    }
    return;
}

#==============================================================================
# function     DeleteFileReference
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Handler->DeleteFileReference( $Servlet );
#----------------------------------------------------------------------------------------
# description  deletes file attribute value (not the image)
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#==============================================================================
sub DeleteFileReference {
    my $self = shift;
    my ( $Servlet ) = @_;

    $Servlet->object->set({$Servlet->form->value('File') => undef});
    return;
}

#==============================================================================
# Event handlers
#==============================================================================
sub UploadFiles {
    my $self = shift;
    my ( $Servlet ) = @_;

    my $Object = $Servlet->object;
    my $Form = $Servlet->form;

    my (undef, $hResponseContent) = Upload($Object, $Form);

    $Servlet->vars('ResponseContent',$hResponseContent);
    return;
}

#========================================================================================
# function     _handleRedirect
# state        public
#----------------------------------------------------------------------------------------
# syntax       _handleRedirect($Servlet);
#----------------------------------------------------------------------------------------
# description  Decides if we have to redirect to domain with SSL on NonSSL
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
#========================================================================================
sub _handleRedirect {
    my $self = shift;
    my ( $Servlet ) = @_;

    #  $Redirect == 0 redirect to non SSL domain
    #  $Redirect == 1 redirect to SSL domain (shop or provider)
    my $Redirect = $self->_testRedirectCondition($Servlet);

    # trigger hook to prevent redirect
    my $PreventRedirect = 0;
    my $hParams = {
     'Servlet' => $Servlet,
     'PreventRedirect' => $PreventRedirect,
    };

    TriggerHook('UI_BeforeTestSEORedirectConditions', $hParams);
    return if $hParams->{'PreventRedirect'};


    # redirect
    $self->_redirectoToDomain($Servlet,$Redirect) if defined $Redirect;
    return;
}

#========================================================================================
# function     _testRedirectCondition
# state        public
#----------------------------------------------------------------------------------------
# syntax       _testRedirectCondition($Servlet);
#----------------------------------------------------------------------------------------
# description  Decides if we have to redirect to shop/provider domain with SSL on NonSSL
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# return       1 if a redirection to SSL shop/provider domain is required,
#               0 if we have to redirect to non SSL shop/provider
#               undef if no redirection is needed | boolean
#========================================================================================
sub _testRedirectCondition {
    my $self = shift;
    my ($Servlet) = @_;

    my $Site = $Servlet->object->getSite();

    # info from site and shop
    my ($hSite) = GetUrlInfo($Site);

    # do we have currently SSL in this request?
    my $RequestIsSSL = $Servlet->params('SERVER_PORT_SECURE') ? 1 : 0;

    my $HTTPHost = $Servlet->params('HTTP_HOST');
    $HTTPHost =~ s/:\d+$//;

    # do we have a domain name for the shop
    my $ShopDomainName = $hSite->{'DomainName'};
    $ShopDomainName = EncodeDomain($ShopDomainName) if $ShopDomainName;

    # do we have a request from a provider url?
    my $IsProviderRequest = ((not $ShopDomainName) or ($HTTPHost ne $ShopDomainName)) ? 1 : 0;

    # is a certificate for the shop available?
    my $ShopHasSSLCertificate = $hSite->{'HasSSLCertificate'};

# SSL/Non SSL redirects
#----------------------------|-----------------------|-----------------------|------------------------|---------------------------|
#PBO Configuration / Call to |   SSL shop domain     |  SSL provider domain  |   Non SSL shop domain  |   Non SSL provider domain |
#                            |    points to (1)      |        points to (2)  |        points to (3)   |         points to (4)     |
#----------------------------|-----------------------|-----------------------|------------------------|---------------------------|
# Own shop domain and        |  SSL shop domain(1a)  | SSL shop domain(2a)   |  SSL shop domain* (3a) |  SSL shop domain*(4a)     |
# certificate                |                       |                       |                        |                           |
#----------------------------|-----------------------|-----------------------|------------------------|---------------------------|
# Own shop domain            |non SSL shop domain(1b)|SSL provider domain(2b)| non SSL shop domain(3b)|  non SSL shop domain(4b)  |
# without certificate        | (certificate error)   |                       |                        |                           |
#----------------------------|-----------------------|-----------------------|------------------------|---------------------------|
# No own shop domain         |      error(1c)        |SSL provider domain(2c)|     error(3c)          | SSL provider domain*(4c)  |
#----------------------------|-----------------------|-----------------------|------------------------|---------------------------|
# * if configured in epages.conf
# (by setting:UseAlwaysSSLForShopDomains=1 and UseAlwaysSSLForProviderDomains=1)
# otherwise the "non SSL shop domain" is used for (3a) and (4a)
# and the non SSL shop domain is used for (4c)
# (SSL pages are also cached in the new handling by using an own cache)

    # (1/2) Request is SSL and test if we have to redirect to non SSL or to SSL shop domain
    if ($RequestIsSSL) {

        # (1b) redirect to non ssl shop domain (should normally not happen because of certificate errors)
        return 0 if (not $IsProviderRequest) and $ShopDomainName and not $ShopHasSSLCertificate and $self->_canRedirectToNonSSL($Servlet);

        # (2a) redirect to ssl shop domain
        return 1 if $IsProviderRequest and $ShopDomainName and $ShopHasSSLCertificate;

        # (1a/2b/2c) no redirect
        return undef;

    }
    else {

        # (3/4) Request is NOT SSL and test if we have to redirect to SSL shop or provider domain

        # we use config settings from epages.conf used for (3a/4a)
        my $UseAlwaysSSLForShopDomains     = $Servlet->useSSLForShopDomains();
        my $UseAlwaysSSLForProviderDomains = $Servlet->useSSLForProviderDomains();

        # (3a) redirect to SSL shop domain if activated in config
        # otherwise stay on non SSL shop domain
        if (not $IsProviderRequest and $ShopDomainName and $ShopHasSSLCertificate) {
            return 1 if $UseAlwaysSSLForShopDomains;
            return undef;
        }

        # (4a) redirect to SSL shop domain if activated in config
        # otherwise go to non SSL shop domain
        if ($IsProviderRequest and $ShopDomainName and $ShopHasSSLCertificate) {
            return 1 if $UseAlwaysSSLForProviderDomains;
            return 0;
        }

        # (4b) redirect to non SSL shop domain if we have no shop certificate
        return 0 if $IsProviderRequest and $ShopDomainName and not $ShopHasSSLCertificate;


        # (4c) redirect to SSL provider domain if we have no shop domain
        # and redirect for provider domains is active in config
        if ($IsProviderRequest and not $ShopDomainName) {
            return 1 if $UseAlwaysSSLForProviderDomains;
            return undef;
        }

        # (3b) no redirect
        return undef;

    }

}

#========================================================================================
# function     _redirectoToDomain
# state        public
#----------------------------------------------------------------------------------------
# syntax       _redirectoToDomain($Servlet);
#----------------------------------------------------------------------------------------
# description  Rewrites the response header to do a 301 redirect to the  shop or provider
#               domain with/without SSL.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# input        $UseSSL | redirect to ssl url if 1 | boolean
#========================================================================================
sub _redirectoToDomain {
    my $self = shift;
    my ($Servlet, $UseSSL) = @_;

    my $Object = $Servlet->object;
    my $Site   = $Object->getSite();
    my $SSL    = $UseSSL ? 'SSL' : '';

    my $LocaleID = $Servlet->locale;                                                                 # de_DE
    my $Location = $Servlet->params('IsMobile') ? $Site->get('WebServerScriptUrlMobile' . $SSL) :    # http://<domain>/epages/<shop>.mobile
    $Site->get('WebServerScriptUrl' . $SSL);                                                         # http://<domain>/epages/<shop>.sf

    my %Params = ('ObjectPath' => $Object->get('Path'));
    my $ViewAction = $Servlet->params('ViewAction');
    $Params{'ViewAction'} = $ViewAction unless $ViewAction eq 'View';

    # http://<domain>/epages/<shop>.sf/de_DE/?ViewAction=...&Objectpath=...
    $Location .= '/' . $LocaleID . '/?' . BuildParameterString(\%Params);
    $Servlet->redirect($Location);
    return;
}

#========================================================================================
# function     _canRedirectToNonSSL
# state        private
#----------------------------------------------------------------------------------------
# syntax       _canRedirectToNonSSL($Servlet);
#----------------------------------------------------------------------------------------
# description  SSL and non-Shopdomain URLs are redirected to the correct shop non SSL URL
#               only under a bunch of conditions: no Session, no MBO preview mode, no form
#               data, etc.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | object
# return       1 if a redirection to non SSL is possible, 0 otherwise | boolean
#========================================================================================
sub _canRedirectToNonSSL {
    my $self = shift;
    my ( $Servlet ) = @_;

    my $IsSession = defined $Servlet->intSessionID ? 1 : 0;
    return 0 if $IsSession;

    my $IsForm = scalar keys %{$Servlet->form->formValues} > 0 ? 1 : 0;
    return 0 if $IsForm;

    my $IsFormError = $Servlet->vars('FormError') ? 1 : 0;
    return 0 if $IsFormError;

    my $IsPreview = defined $Servlet->cookies('PreviewStyle') ? 1 : 0;
    return 0 if $IsPreview;

    my $IsEditor = $Servlet->isEditorMode;
    return 0 if $IsEditor;

    my $isDesignEditor = $Servlet->isEditorNavElements;
    return 0 if $isDesignEditor;

    my $IsValidAdminSession = TestAdminSFSession($Servlet->object->getSite, GetAdminSFSessionCookie) == 1;
    return 0 if $IsValidAdminSession;

    return 1;
 }

#========================================================================================
# function     _mapPreDefValues
# state        private
#----------------------------------------------------------------------------------------
# syntax       _mapPreDefValues($Form, $Object, $ElementName, $AttributeAlias, $hhValues);
#----------------------------------------------------------------------------------------
# description  Maps the values for PreDef* attribute types from Form fields to
#               $hhValues.
#----------------------------------------------------------------------------------------
# input        $Form | servlet form | object
# input        $Object | object from servlet | object
# input        $ElementName | name of the attribute in the form | string
# input        $AttributeAlias | alias of the attribute | string
# input        $hhValues | see <function attributeValues> | ref.hash.hash
#========================================================================================
sub _mapPreDefValues {
    my ($Form, $Object, $ElementName, $AttributeAlias, $hhValues) = @_;
    my @OrgValues = $Form->value( $ElementName );
    my %FieldDef = (
        Name        => $ElementName,
        Mandatory   => 1,
        Type        => 'integer',
        MinValue    => 1,
    );
    my $Site = $Object->getSite;
    my @Values;
    foreach ( my $Index=0; $Index<scalar @OrgValues; $Index++ ) {
        next if $OrgValues[$Index] eq '';
        $FieldDef{'Index'} = $Index;
        my $ValueID = $Form->field( $Object, $OrgValues[$Index], \%FieldDef );
        push @Values, $Site->loadObject($ValueID, 'PreDefAttribute');
    }
    $hhValues->{'Values'}{$AttributeAlias} = \@Values;
}

#========================================================================================
# function     _mapLocalizedValues
# state        private
#----------------------------------------------------------------------------------------
# syntax       _mapLocalizedValues($Form, $Object, $ElementName, $AttributeAlias, $Attribute, $aLanguageIDs, $Type, $hhValues);
#----------------------------------------------------------------------------------------
# description  Maps the values for LocalizedString and LocalizedFile attribute types
#               from Form fields to $hhValues.
#----------------------------------------------------------------------------------------
# input        $Form | servlet form | object
# input        $Object | object from servlet | object
# input        $ElementName | name of the attribute in the form | string
# input        $AttributeAlias | alias of the attribute | string
# input        $Attribute | affected attribute | object
# input        $aLanguageIDs | list of language ids | ref to array of strings
# input        $Type | attribute type | string
# input        $hhValues | see <function attributeValues> | ref.hash.hash
#========================================================================================
sub _mapLocalizedValues {
    my ($Form, $Object, $ElementName, $AttributeAlias, $Attribute, $aLanguageIDs, $Type, $hhValues) = @_;

    my $hAttributeValues = $Attribute->get([ qw ( Length IsHTML IsMandatory ) ]);

    my $hFieldDef = {
        Name        => $ElementName,
        Type        => 'string',
        MaxValue    => $hAttributeValues->{'Length'},
        IsHTML      => $hAttributeValues->{'IsHTML'},
        LoopName    => 'dummy',
    };
    $hFieldDef->{Mandatory} = 1 if $hAttributeValues->{'IsMandatory'};

    my @OrgValues = $Form->value( $ElementName );
    foreach ( my $Index=0; $Index<scalar @OrgValues; $Index++ ) {
        my $LanguageID = $aLanguageIDs->[$Index];
        Error('MoreValuesAsLanguages', {
            'Name'  => $ElementName,
            'Index' => scalar(@OrgValues),
            'CountLanguageIDs' => scalar(@$aLanguageIDs)
        }) unless defined $LanguageID;
        $hFieldDef->{Index} = $Index;
        if($Type eq 'LocalizedFile') {
            $hFieldDef->{Type} = 'file';
        }
        else { # LocalizedString
            $hFieldDef->{Type} = 'string';
        }

        my $Value = $Form->field( $Object, $OrgValues[$Index], $hFieldDef );

        if($Type eq 'LocalizedFile') {
            if( defined $Value && $Value =~ /\S/ ) {
                $hhValues->{'LocalizedFiles'}{$LanguageID}{$AttributeAlias} = {
                    'Value' => $Value,
                    'Index' => $Index
                };
            }
            elsif ($Form->value('Delete'.$ElementName.'_'.$LanguageID)) {
                $hhValues->{'LocalizedFiles'}{$LanguageID}{$AttributeAlias} = undef;

            } #else don't change the file
        }
        else { # LocalizedString
            # delete attribute value if the form field is empty
            $Value = undef unless (defined $Value and $Value =~ /\S/);
            $hhValues->{'LocalizedValues'}{$LanguageID}{$AttributeAlias} = $Value;
        }
    }
}

#========================================================================================
# function     _mapPasswordValue
# state        private
#----------------------------------------------------------------------------------------
# syntax       _mapPasswordValue($Form, $Object, ElementName, $AttributeAlias, $Attribute, $hhValues);
#----------------------------------------------------------------------------------------
# description  Maps the values for the ExternalPassword attribute type from Form fields
#               into $hhValues.
#----------------------------------------------------------------------------------------
# input        $Form | servlet form | object
# input        $Object | object from servlet | object
# input        $ElementName | name of the attribute in the form | string
# input        $AttributeAlias | alias of the attribute | string
# input        $Attribute | affected attribute | object
# input        $hhValues | see <function attributeValues> | ref.hash.hash
#========================================================================================
sub _mapPasswordValue {
    my ($Form, $Object, $ElementName, $AttributeAlias, $Attribute, $hhValues) = @_;
    my $OrgValue = $Form->value( $ElementName );
    my $PasswordDummy = PASSWORD_DUMMY;
    if ($OrgValue =~ /\Q$PasswordDummy\E/) {
        delete $hhValues->{'Values'}{$AttributeAlias};
    } else {
        my $hFieldDef = {
            Name => $ElementName,
            Type => 'string',
            MaxValue => $Attribute->get('Length'),
            LoopName => 'dummy',
        };
        $hFieldDef->{Mandatory} = 1 if $Attribute->get('IsMandatory');
        my $Value = $Form->field( $Object, $OrgValue, $hFieldDef );
        if (defined $Value and $Value =~ /\S/) {
            $hhValues->{'Values'}{$AttributeAlias} = $Value;
        } else {
            $hhValues->{'Values'}{$AttributeAlias} = undef;
        }
    }
}

#========================================================================================
# function     _mapFileValue
# state        private
#----------------------------------------------------------------------------------------
# syntax       _mapFileValue($Form, $Object, ElementName, $AttributeAlias, $Attribute, $hhValues);
#----------------------------------------------------------------------------------------
# description  Maps the values for the File attribute type from Form fields
#               into $hhValues.
#----------------------------------------------------------------------------------------
# input        $Form | servlet form | object
# input        $Object | object from servlet | object
# input        $ElementName | name of the attribute in the form | string
# input        $AttributeAlias | alias of the attribute | string
# input        $Attribute | affected attribute | object
# input        $hhValues | see <function attributeValues> | ref.hash.hash
#========================================================================================
sub _mapFileValue {
    my ($Form, $Object, $ElementName, $AttributeAlias, $Attribute, $hhValues) = @_;

    my $hFieldDef = {
        Name => $ElementName,
        Type => 'file',
    };

    my $OrgValue = $Form->value( $ElementName );
    if ($OrgValue eq '_DELETE_') {
        my $OldValue = $Object->get($AttributeAlias);
        if ($OldValue =~ /^MediaGallery/) {
            $hhValues->{'Values'}{$AttributeAlias} = undef;
        }
        else {
            $hhValues->{'Files'}{$AttributeAlias} = undef;
        }
    }
    elsif( defined $OrgValue && $OrgValue =~ /\S/ ) {
        if (not exists $Form->files->{$ElementName}[0] and ExistsFile(RealPath($Object->get('PublicPath').'/'.$OrgValue))) {
            $hhValues->{'Values'}{$AttributeAlias} = $OrgValue;
        }
        else {
            my $Value = $Form->field( $Object, $OrgValue, $hFieldDef );
            $hhValues->{'Files'}{$AttributeAlias} = $Value;
        }

    }
    elsif ($Form->value('Delete'.$ElementName)) { # delete is only possible if file Attribute is in form also
        $hhValues->{'Files'}{$AttributeAlias} = undef;
    }
}

#========================================================================================
# function     _mapObjectValue
# state        private
#----------------------------------------------------------------------------------------
# syntax       _mapObjectValue($Form, $Object, ElementName, $AttributeAlias, $Attribute, $Type, $hhValues);
#----------------------------------------------------------------------------------------
# description  Maps the values for Object attributes from form fields
#               into $hhValues.
#----------------------------------------------------------------------------------------
# input        $Form | servlet form | object
# input        $Object | object from servlet | object
# input        $ElementName | name of the attribute in the form | string
# input        $AttributeAlias | alias of the attribute | string
# input        $Attribute | affected attribute | object
# input        $Type | attribute type | string
# input        $hhValues | see <function attributeValues> | ref.hash.hash
#========================================================================================
sub _mapObjectValue {
    my ($Form, $Object, $ElementName, $AttributeAlias, $Attribute, $Type, $hhValues) = @_;

    my $OrgValue = $Form->value($ElementName); # scalar context - get only the first value

    my $hFieldDef = {
        Name => $ElementName,
        Type => 'integer',
        MinValue => 1,
        Mandatory => $Attribute->get('IsMandatory') ? 1 : 0,
    };
    my $ValueID = $Form->field( $Object, $OrgValue, $hFieldDef );

    my $Reference = undef;
    if (defined $ValueID && $ValueID =~ /\S/) {
        $Type = 'PreDefAttribute' if $Type =~ /^PreDef/;
        $Reference = $Object->getSite->loadObject($ValueID, $Type, { 'AllowSystemSite' => 1 });
    }
    $hhValues->{'Values'}{$AttributeAlias} = $Reference;
}

#========================================================================================
# function     _mapScalarValue
# state        private
#----------------------------------------------------------------------------------------
# syntax       _mapScalarValue($Form, $Object, ElementName, $AttributeAlias, $Attribute, $Type, $hhValues);
#----------------------------------------------------------------------------------------
# description  Maps the values for an attribute of type String, Boolean, Integer, ...
#               from form fields into $hhValues.
#
#               For the REST API: we copied this in
#               DE_EPAGES::Shop::API::AddressValidation::_createFieldDefinition
#----------------------------------------------------------------------------------------
# input        $Form | servlet form | object
# input        $Object | object from servlet | object
# input        $ElementName | name of the attribute in the form | string
# input        $AttributeAlias | alias of the attribute | string
# input        $Attribute | affected attribute | object
# input        $Type | attribute type | string
# input        $hhValues | see <function attributeValues> | ref.hash.hash
#========================================================================================
sub _mapScalarValue {
    my ($Form, $Object, $ElementName, $AttributeAlias, $Attribute, $Type, $hhValues) = @_;

    my $OrgValue = $Form->value($ElementName); # scalar context - get only the first value

    my $hFieldDef = {
        Name => $ElementName,
        Mandatory => $Attribute->get('IsMandatory') ? 1 : 0,
    };

    given($Type) {
        when('String') {
            my $hAttributeValues = $Attribute->get( ['Length', 'IsHTML' ] );
            $hFieldDef->{'Type'} = 'string';
            $hFieldDef->{'MaxValue'} = $hAttributeValues->{'Length'};
            $hFieldDef->{'IsHTML'} = $hAttributeValues->{'IsHTML'};
        }
        when('EMail') {
            $hFieldDef->{'Type'} = 'email_address';
        }
        when('MultipleEMails') {
            $hFieldDef->{'Type'} = 'multiple_email_address';
        }
        when('IPAddress') {
            $hFieldDef->{'Type'} = 'ip_address';
        }
        when('Boolean') {
            $hhValues->{'Values'}{$ElementName} = 0 unless defined $OrgValue;
            $hFieldDef->{'Type'} = 'checkbox';
        }
        when('HTML') {
            $hFieldDef->{'Type'} = 'html';
            my $hAttributeValues = $Attribute->get( ['Length', 'IsHTML' ] );
            $hFieldDef->{'MaxValue'} = $hAttributeValues->{'Length'};
        }
        default { # Integer, Float, DateTime, Money
            $hFieldDef->{'Type'} = 'reg_'. lc $Type;
        }
    }

    my $Value = $Form->field( $Object, $OrgValue, $hFieldDef );

    # delete attribute value if the form field is empty
    $hhValues->{'Values'}{$AttributeAlias} = (defined $Value && $Value =~ /\S/ ) ? $Value : undef;
}

#========================================================================================
# function     _fillValues
# state        private
#----------------------------------------------------------------------------------------
# syntax       _fillValues($Form, $Object, $ElementName, $AttributeAlias, $hhValues);
#----------------------------------------------------------------------------------------
# description  Controls the mapping of the form input values into the final checked
#               attribute values for the given attribute type.
#----------------------------------------------------------------------------------------
# input        $Form | servlet form | object
# input        $Object | object from servlet | object
# input        $Attribute | affected attribute | object
# input        $ElementName | name of the attribute in the form | string
# input        $aLanguageIDs | list of language ids | ref to array of strings
# input        $hhValues | checked attribute values
#                   <ul>
#                       <li>Values - non-localized attribute values execpt files
#                                   { $AttributeName => $Value }</li>
#                       <li>Files - non-localized attribute values of type File
#                                   { $AttributeName => $FileName }
#                                   if the file name is undef, then the file is about to be deleted</li>
#                       <li>LocalizedValues - localized attribute values execpt files
#                                   { $LanguageID => { $AttributeName => {'Value'=>$Value, 'Index'=>$Index } }</li>
#                       <li>LocalizedFiles - localized attribute values of type File
#                                   { $LanguageID => { $AttributeName => $FileName } }
#                                   if the file name is undef, then the file is about to be deleted</li>
#                   </ul>
#                   | ref.hash.hash
#========================================================================================
sub _fillValues {
    my ($Form, $Object, $Attribute, $ElementName, $aLanguageIDs, $hhValues) = @_;

    my $Type = $Attribute->get('Type');
    my $AttributeAlias = $Attribute->alias;

    if( $Attribute->get('IsObject') ) {
        _mapObjectValue($Form, $Object, $ElementName, $AttributeAlias, $Attribute, $Type, $hhValues);
        return;
    }

    given($Type) {
        when(['PreDefMultiString','PreDefMultiLocalizedString','PreDefCustomerString']) {
            _mapPreDefValues($Form, $Object, $ElementName, $AttributeAlias, $hhValues);
        }
        when(['LocalizedString','LocalizedFile']) {
            _mapLocalizedValues($Form, $Object, $ElementName, $AttributeAlias, $Attribute, $aLanguageIDs, $Type, $hhValues);
        }
        when('ExternalPassword') {
            _mapPasswordValue($Form, $Object, $ElementName, $AttributeAlias, $Attribute, $hhValues);
        }
        when('File') {
            _mapFileValue($Form, $Object, $ElementName, $AttributeAlias, $Attribute, $hhValues);
        }
        default {
            _mapScalarValue($Form, $Object, $ElementName, $AttributeAlias, $Attribute, $Type, $hhValues);
        }
    }

    return;
}

#========================================================================================
# function     SetTemplateTypeAttribute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Handler->SetTemplateTypeAttribute($Servlet);
# example      $self->SetTemplateTypeAttribute($Servlet);
#----------------------------------------------------------------------------------------
# description  Sets any number of string attributes for a template type related to a certain object.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet containing the TemplateTypeID, ObjectID and AttributeData form values | Object
#========================================================================================
sub SetTemplateTypeAttribute {
    my $self = shift;
    my ($Servlet) = @_;

    my $TemplateTypeID = $Servlet->params('TemplateTypeID');
    my $Object         = $Servlet->object;
    my $hAttributeData = from_json(decode_base64($Servlet->params('AttributeData')));

    TestHash('DataHash', $hAttributeData);
    TestObject('Object', $Object);

    Error('NoResultTemplateType', {
        'TemplateTypeID' => $TemplateTypeID,
    }) unless ExistsObject($TemplateTypeID);

    my $TemplateType = $Object->getSite->loadObject($TemplateTypeID, 'TemplateType');

    $Servlet->params('ExitAfterEvent', 1);
    return unless $TemplateType->get('IsCustomizable');

    my $Template = $TemplateType->get('Template');
    my $Map = $Template->existsDirectTemplateMap($Object)
                ? $Template->getDirectTemplateMap($Object)
                : $Template->setTemplateMap($Object, $TemplateType);

    $Map->set($hAttributeData);
    return;
}

#========================================================================================
# function     listItems
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ahListItems = $self->listItems( $Servlet, $Object, $ClassAlias )
# syntax       $ahListItems = $self->listItems( $Servlet, $Object, $ClassAlias, $ActionAlias )
#----------------------------------------------------------------------------------------
# description  Returns a list of objects and position values. Tests if the current user
#               has permission to save these objects. The permission test is only for
#               objects (SubClass of 'Object' class).
#               If the LineItem object doesn't exist, it will be ignored ignored.
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet | DE_EPAGES::Presentation::Servlet::Permission
# input        $Object | object | DE_EPAGES::Object::API::Object::Object
# input        $ClassAlias | class alias of the list object | string
# input        $ActionAlias | (optional; default: 'Save') action for permission test | string
# return       $ahListItems | list of hashes with keys 'Object' and 'Position' | ref.array.hash
#========================================================================================
sub listItems {
    my $self = shift;
    my ($Servlet, $Object, $ClassAlias, $ActionAlias) = @_;
    $ActionAlias //= 'Save';
    my $Class = LoadClassByAlias($ClassAlias);

    my $Site = $Object->getSite;
    my $hForm = $Servlet->form->form($Object, 'ListedPositions');

    my @ListItems;
    foreach my $hItem( @{$hForm->{'ListPositions'}} ) {
        next unless $Class->existsObject($hItem->{'PositionObjectID'});

        my $ListObject = $Site->loadObject( $hItem->{'PositionObjectID'}, $Class );
        $Servlet->testPermission( $ListObject, $ActionAlias ) if $Class->isSubClassOf('Object');

        push @ListItems, {
            'Object' => $ListObject,
            'Position' => $hItem->{'ListPosition'},
        }
    }
    return \@ListItems
}

1;
