#========================================================================================
# package      package DE_EPAGES::Shop::API::JWT;
# state        public
#----------------------------------------------------------------------------------------
# description  This package provides the JsonWebToken for the Shop.
#========================================================================================
package DE_EPAGES::Shop::API::JWT;
use base Exporter;
use strict;

use DE_EPAGES::Core::API::PerlTools qw( CreateJsonWebToken );
use DE_EPAGES::Database::API::Config qw ( DatabaseConnectionData );
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Shop::API::Constants qw ( EDITOR_JWT_LIFETIME );

our @EXPORT_OK = qw( CreateShopJsonWebToken AddEditorTokenToURL );


#========================================================================================
# function     CreateShopJsonWebToken
# state        public
#----------------------------------------------------------------------------------------
# syntax       CreateShopJsonWebToken($Lifetime, $Shop, $Scope);
#----------------------------------------------------------------------------------------
# description  Returns a newly created JSON web token with the shop's alias as subject
#               and the secret taken from the "jwtsecret" key-value pair of the
#               respective store conf.
#               See <function DE_EPAGES::Core::API::PerlTools::CreateJsonWebToken>.
#----------------------------------------------------------------------------------------
# input        $Lifetime | the token's lifetime in seconds| integer
# input        $Shop | shop object | object
# input        $Scope | the token's scope (e.g. editor or provisioning)| string
# return       $JWT | encoded JSON web token | string
#========================================================================================
sub CreateShopJsonWebToken {
    my ($Lifetime, $Shop, $Scope) = @_;

    return CreateJsonWebToken($Lifetime, $Shop->alias, $Scope,
        'http://epages.com/epages6', 'http://epages.com/unity',
        DatabaseConnectionData(LoadRootObject->get('StoreName'))->{'jwtsecret'});
}

#========================================================================================
# function     AddEditorTokenToURL
# state        public
#----------------------------------------------------------------------------------------
# syntax       AddEditorTokenToURL($Servlet, $URL);
#----------------------------------------------------------------------------------------
# description  Creates a new JWT with the editor scope and adds it to the given URL
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet  | object
# input        $URL | cureent URL | string
# return       $URL | URL with added JWT | string
#========================================================================================
sub AddEditorTokenToURL {
    my ($Servlet, $URL) = @_;

    # create a JWT for the editor with a lifetime of 3 minutes
    # add token to the end of the URL
    my $Delimiter = $URL =~ /\?/ ? '&' : '?';
    $URL .= $Delimiter . "token=" . CreateShopJsonWebToken(EDITOR_JWT_LIFETIME, $Servlet->object->getSite(), 'editor');

    return $URL;
}

1;
