#========================================================================================
# package      DE_EPAGES::Shop::Hooks::Shop
# state        public
# description  Imports default shop data into a new shop when it is created.
#========================================================================================
package DE_EPAGES::Shop::Hooks::Shop;

use strict;
use DE_EPAGES::Core::API::EpagesConfig qw ( EpagesConfig );
use DE_EPAGES::Core::API::PerlTools qw( GetCurrentDateTime );
use DE_EPAGES::OAuthTokenClient::API::OAuth2::OAuth2CredentialsClient;
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use DE_EPAGES::Shop::API::Cartridge;
use DE_EPAGES::Shop::API::Constants qw( UNITY_STOREFRONT_APP_NAME UNITY_STOREFRONT_APP_TYPE );

#========================================================================================
# function     OnInsertShop
#----------------------------------------------------------------------------------------
# syntax       OnInsertShop( $hParams );
#----------------------------------------------------------------------------------------
# description  Imports default shop data into a new shop when it is created.
#               (Hook OBJ_InsertShop)
#----------------------------------------------------------------------------------------
# input        $hParams | primary key of tax class, key is
#               <ul>
#                   <li>Object - shop - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnInsertShop {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};
    my $Cartridge = DE_EPAGES::Shop::API::Cartridge->new;
    ImportObjects($Cartridge->xmlDirectory().'/DefaultShop.xml', $Shop->pathString);
}

#========================================================================================
# function     GetShopFeatureCurrentValue
#----------------------------------------------------------------------------------------
# syntax       GetShopFeatureCurrentValue( $hParams );
#----------------------------------------------------------------------------------------
# description  Dynami Values of current features i.e. WebServiceFetures with date reset
#               (Hook OBJ_GetShopFeatureCurrentValue)
#----------------------------------------------------------------------------------------
# input        $hParams | hook parameters
#               <ul>
#                   <li>Object - shop - object
#                   <li>Name - feature name - string
#                   <li>FeatureID - feature id - integer
#               </ul> | ref.hash.*
#----------------------------------------------------------------------------------------
# return      $hParams | hook parameters
#               <ul>
#                   <li>Value - current value - integer
#               </ul> | ref.hash.*
#========================================================================================
sub GetShopFeatureCurrentValue {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};
    my $FeatureAlias = $hParams->{'Name'};

    if ( $FeatureAlias =~ /WebServices./ ) {
        # compare current date with last WS date, reset feature count if not equal
        my $CurrentDate = GetCurrentDateTime();
        my $DateAttributeName = $FeatureAlias.'Date';
        if ( $Shop->class->existsAttribute($DateAttributeName) ) {
            my $LastDate = $Shop->get( $DateAttributeName );
            if ( !$LastDate ||
                  $LastDate->day    != $CurrentDate->day ||
                  $LastDate->month  != $CurrentDate->month )
            {
                $Shop->set( {$DateAttributeName => $CurrentDate} );
                $Shop->featureCurrentValue( $FeatureAlias, 0 );
                $hParams->{'Value'} = 0;
            }
        }
    }

    return 0;
}

#========================================================================================
# function     GetShopFeatureMaxValue
#----------------------------------------------------------------------------------------
# syntax       GetShopFeatureMaxValue( $hParams );
#----------------------------------------------------------------------------------------
# description  Returns the max value of a features
#               (Hook OBJ_GetShopFeatureMaxValue)
#----------------------------------------------------------------------------------------
# input        $hParams | hook parameters
#               <ul>
#                   <li>Object - shop - object
#                   <li>Name - feature name - string
#               </ul> | ref.hash.*
#----------------------------------------------------------------------------------------
# return      $hParams | hook parameters
#               <ul>
#                   <li>Value - current value - integer
#               </ul> | ref.hash.*
#========================================================================================
sub GetShopFeatureMaxValue {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};
    my $FeatureAlias = $hParams->{'Name'};

    $hParams->{'Value'} = $Shop->featureMaxValue($FeatureAlias);
    return $hParams->{'Value'};
}

#========================================================================================
# function     OnBeforePatchUnityShop
#----------------------------------------------------------------------------------------
# syntax       OnBeforePatchUnityShop( $hParams );
#----------------------------------------------------------------------------------------
# description  Triggered by the hook API_BeforePatchUnityShop.
#               Sets the shop content that should be patched to the Unity database.
#----------------------------------------------------------------------------------------
# input        $hParams | hook parameters
#               <ul>
#                 <li>Object  - shop                              - object
#                 <li>Values  - new shop values                   - ref.hash
#                 <li>Content - content to patch to the unity API - ref.hash
#               </ul>
#========================================================================================
sub OnBeforePatchUnityShop {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};
    my $hValues = $hParams->{'Values'};
    my $hContent = $hParams->{'Content'};

    if (   exists $hValues->{'ClosedByMerchant'}
        or exists $hValues->{'ClosedByProvider'}
        or exists $hValues->{'ClosedByProviderTemporarily'}) {
        if ($hValues->{'ClosedByMerchant'} or $hValues->{'ClosedByProvider'} or $hValues->{'ClosedByProviderTemporarily'}) {
            $hContent->{'isClosed'} = JSON::true;
        }
        else {
            my $hClosedSettings = $Shop->get(['ClosedByMerchant', 'ClosedByProvider', 'ClosedByProviderTemporarily']);
            $hContent->{'isClosed'} = JSON::false unless ($hClosedSettings->{'ClosedByMerchant'}
                or $hClosedSettings->{'ClosedByProvider'} or $hClosedSettings->{'ClosedByProviderTemporarily'});
        }
    }

    if (exists $hValues->{'Name'}) {
        $hContent->{'title'} = $hValues->{'Name'} // '';
    }

    if (exists $hValues->{'Logo'}) {
        $hContent->{'logoUrl'} = $Shop->getLogoUrl($hValues->{'Logo'});
    }

    # add elemments for patching cookie notice here
    if ((exists $hValues->{'ShowCookieLawText'}) or (exists $hValues->{'CookieLawMessage'})) {
        my ($IsActive, $Message) = $Shop->processCookieLawMessage($hValues);
        $hContent->{'userSettings'}->{'cookieNotice'}->{'active'} =  $IsActive;
        $hContent->{'userSettings'}->{'cookieNotice'}->{'messageHtml'} = $Message;
    }

    if (exists $hValues->{'DomainName'}) {
        $hContent->{'domain'} = $hValues->{'DomainName'};
    }

    # Patch URL values in case the SSL or domain setting of the shop changes
    if ((exists $hValues->{'HasSSLCertificate'}) or ((exists $hValues->{'DomainName'}))) {
        $hContent->{'logoUrl'} = $Shop->getLogoUrl($Shop->get('Logo'));
        $hContent->{'mboBaseUrl'} = $Shop->get('WebUrlAdminSSL');
    }

    # if Unity accepts attributes for trial shop and shop domain we just can decomment the lines below
    # if (exists $hValues->{'IsTrialShop'}) {
    #     $hContent->{'isTrialShop'} = $hValues->{'IsTrialShop'};
    # }
    return;
}

#========================================================================================
# function     OnBeforeCreateUnityShop
#----------------------------------------------------------------------------------------
# syntax       OnBeforeCreateUnityShop( $hParams );
#----------------------------------------------------------------------------------------
# description  Triggered by the hook API_OnBeforeCreateUnityShop.
#               Sets the shop content for the shop creation on the Unity database.
#----------------------------------------------------------------------------------------
# input        $hParams | hook parameters
#               <ul>
#                 <li>Object  - shop                             - object
#                 <li>Content - content to send to the unity API - ref.hash
#               </ul>
#========================================================================================
sub OnBeforeCreateUnityShop {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};
    my $hContent = $hParams->{'Content'};

    my $Alias = $Shop->get('WebServerScriptNamePart');
    my $System = LoadRootObject();

    # Post initial cookie notice to the UNITY shop
    my ($IsActive, $Message) = $Shop->processCookieLawMessage({});

    # close every new created Unity shop per default
    $Shop->set({'ClosedByMerchant' => 1});

    $hContent->{'isClosed'}   = JSON::true;
    $hContent->{'title'}      = '';
    $hContent->{'locale'}     = $Shop->get('LocaleID');
    $hContent->{'mboBaseUrl'} = $Shop->get('WebUrlAdminSSL');
    $hContent->{'logoUrl'}    = $Shop->getLogoUrl($Shop->get('Logo'));
    $hContent->{'domain'}     = $Shop->get('DomainName');
    $hContent->{'jrest'}      = {
        'baseUrl'  => $System->get('UnityBaseURL'),
        'publicId' => $Alias
    };
    $hContent->{'userSettings'} = {
        'cookieNotice' => {
            'active'      => $IsActive,
            'messageHtml' => $Message
        }
    };


    if (EpagesConfig->get('DE_EPAGES::Object', 'UnityUseAuth', 1)) {
        my $Client = DE_EPAGES::OAuthTokenClient::API::OAuth2::OAuth2CredentialsClient->create;
        my $Credentials = $Client->createAndSaveCredentials($Shop->get('GUID'), {
            'name' => UNITY_STOREFRONT_APP_NAME,
            'type' => UNITY_STOREFRONT_APP_TYPE,
        });

        $hContent->{'jrest'}->{'oauth'} = {
            'version'     => '2.0',
            'credentials' => {
                'clientId'           => $Credentials->clientId,
                'clientSecret'       => $Credentials->clientSecret,
                'accessToken'        => $Credentials->accessToken
            }
        };
    }
}

1;
