package DE_EPAGES::ShopCSVExportImport::UI::CSVExportImport;
use base qw ( DE_EPAGES::CSVExportImport::UI::CSVExportImport );

use strict;
use DE_EPAGES::Core::API::File qw( fsplit GetFileSize );
use DE_EPAGES::Core::API::Log qw( LogDebug );
use DE_EPAGES::Core::API::Unicode qw ( Unicode2Filename );
use DE_EPAGES::CSVExportImport::API::Constants qw( MAX_WARNINGS );
use DE_EPAGES::Presentation::API::Upload qw( Upload );
use DE_EPAGES::ShopCSVExportImport::API::CSV::ShopExportImport qw ( ExportCSV ImportCSV );
use DE_EPAGES::Trigger::API::BatchProcessor qw ( RunBatchProcess );

#==============================================================================
# Helper methods
#==============================================================================

sub createDriver {
    die 'abstract method \'createDriver\' was not implemented';

# example:
#    return DE_EPAGES::Product::API::CSV::ProductExportImport->new;
}

sub type {
    die 'abstract method \'type\' was not implemented';

# example:
#    return 'Product';
}

sub ExportFileName {
    my $self = shift;
    my ($FileName) = @_;
    return $FileName ? Unicode2Filename($FileName) : 'export';
}

#==============================================================================
# Constants
#==============================================================================

my %DELIMITERS = (
    'Comma' => ',',
    'Semicolon' => ';',
    'Tabulator' => "\t",
);

#==============================================================================
# Event handlers
#==============================================================================

sub View {
    my $self = shift;
    my ($Servlet) = @_;

    $Servlet->params( 'CSVExportImportType', $self->type );
    return;
}

sub CSVExportImport {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $hForm = $Servlet->form->form($Shop, 'CSVExportImport');

    if (defined $hForm->{'Import'}) {
        $self->CSVImport($Servlet);
    }
    elsif (defined $hForm->{'Export'}) {
        $self->CSVExport($Servlet);
    }

    return;
}

sub CSVExport {
    my $self    = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object;
    my $hForm = $Servlet->form->form($Shop, 'CSVExportImport');
    my @ObjectIDs = $Servlet->form->value('SelectedObjects');
    my @Objects;

    $Servlet->params('CSVFileName', $self->ExportFileName($Servlet->params('ExportFileName')));

    #check if selective csv export is required
    if($Servlet->params('EXPORT_SELECTED_OBJECTS')) {
        foreach my $ObjectID (@ObjectIDs){
            my $Object = $Shop->loadObject($ObjectID, 'Object');
            push(@Objects, $Object->id);
        }
    }

    my $PathFileName = $self->getPrivateCSVExportFileName($Servlet);
    my $aLanguages = $hForm->{'CSVLanguage'} eq 'AllLanguages' ?  [] : [$hForm->{'CSVLanguage'}];
    my %Options = (
        'LanguageCode2s' => $aLanguages,
        'LocaleID' => $hForm->{'CSVLocale'},
    );
    $Options{'sep_char'} = $DELIMITERS{$hForm->{'CSVDelimiter'}} if $hForm->{'CSVDelimiter'};
    $Options{'encoding'} = $hForm->{'CSVEncoding'} if $hForm->{'CSVEncoding'};
    my $DriverPackage = $self->createDriver;
    my $aWarnings;
    if($Servlet->params('EXPORT_SELECTED_OBJECTS')){#selective csv export
        $aWarnings = ExportCSV($Shop, $PathFileName, $DriverPackage, \%Options, \@Objects);
    } else {#all objects should be exported
        $aWarnings = ExportCSV($Shop, $PathFileName, $DriverPackage, \%Options, undef );
    }
    LogDebug('Export Warnings', $aWarnings );

    $Servlet->vars( 'Redirect', 1);#show exportfile to user in following view action
    $Servlet->vars( 'CSVExportImportType', $self->type );
    $Servlet->form->mapFormError({  'Form'  => 'CSVExportImport',
                                    'Name'  => 'Export'}, $_) foreach @$aWarnings;
    $Servlet->form->executeFormError;

    #check if view action have to be changed after export
    if (defined $Servlet->params('ViewActionAbort') && defined $Servlet->params('ObjectIDAbort')){
        $Servlet->vars( 'ViewAction', $Servlet->params('ViewActionAbort'));
        $Servlet->vars( 'ObjectID', $Servlet->params('ObjectIDAbort'));
    }
    return;
}

sub CSVImport {
    my $self    = shift;
    my ($Servlet, $BatchHandlers) = @_;

    my $Shop = $Servlet->object;
    my $hForm = $Servlet->form->form($Shop, 'CSVExportImport');

    my $PathFileName = $self->getPrivateCSVImportFileName($Servlet);
    my ($Path, $FileName, $Extension) = fsplit($PathFileName);

    my (undef, $hResponseContent) = Upload($Shop, $Servlet->form, {
        'FormName' => 'CSVExportImport',
        'InputName' => 'NewImportFile',
        'Index' => 0,
        'Path' => $Path,
        'FileName' => $FileName,
        'Extension' => $Extension
    });
    $Servlet->vars('ResponseContent', $hResponseContent);

    $Servlet->vars('ImportFileName', $hForm->{'NewImportFile'});
    $Servlet->vars('ImportFileSize', GetFileSize($PathFileName) / 1024); ## no critic qw( ProhibitMagicNumbers )

    my $aLanguages = $hForm->{'CSVLanguage'} eq 'AllLanguages' ?  [] : [$hForm->{'CSVLanguage'}];
    my %Options = (
        'LanguageCode2s' => $aLanguages,
        'LocaleID' => $hForm->{'CSVLocale'},
    );
    $Options{'sep_char'} = $DELIMITERS{$hForm->{'CSVDelimiter'}} if $hForm->{'CSVDelimiter'};
    $Options{'encoding'} = $hForm->{'CSVEncoding'} if $hForm->{'CSVEncoding'};

    my ($aWarnings, $TooManyWarnings);
    if(defined $BatchHandlers) {
        RunBatchProcess($BatchHandlers, sub {
            ($aWarnings, $TooManyWarnings) = ImportCSV($Shop, $PathFileName, $self->createDriver, \%Options);
        });
    } else {
        ($aWarnings, $TooManyWarnings) = ImportCSV($Shop, $PathFileName, $self->createDriver, \%Options);
    }

    if ($TooManyWarnings) {
        $Servlet->vars( 'MaxWarn', MAX_WARNINGS );
        $Servlet->form->addFormError({
            'Reason'  => 'TOO_MANY_WARNINGS',
            'Form'    => 'CSVExportImport',
            'Name'    => 'Import',
        });
    }
    LogDebug( 'Import Warnings', $aWarnings );
    $Servlet->params( 'CSVExportImportType', $self->type );
    $Servlet->form->mapFormError({ 'Form'  => 'CSVExportImport',
                                   'Name'  => 'Import'}, $_) foreach @$aWarnings;
    $Servlet->form->executeFormError;
    $Servlet->vars( 'IMPORT_OK', 1 );
    return;
}

1;
