﻿use strict;
use utf8;
use Getopt::Long;
use DE_EPAGES::Core::API::Script  qw( RunScript );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Database::API::Connection qw( RunOnStore );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject LoadObject );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );

use constant TAX_REGIONS_IMPORT_XML => 'fixSwitzerlandLiechtensteinTaxExemption.xml';

sub Main {

    local $| = 1;

    my $Password;
    my $StoreName;
    my $ImportFile;
    my $Help;
    my $NoOutput;

    GetOptions(
        'help'          => \$Help,
        'nooutput'      => \$NoOutput,
        'passwd=s'      => \$Password,
        'storename=s'   => \$StoreName,
        'importfile=s'  => \$ImportFile,
    );
    usage() if $Help;
    unless( $StoreName ) {
        print STDERR "Missing parameter -storename\n\n";
        usage();
    }
    unless( $ImportFile ) {
        print STDERR "Missing parameter -importfile\n\n";
        usage();
    }

    RunOnStore(
        Store       => $StoreName,
        DBPassword  => $Password,
        Sub         => sub  {
            my $System = LoadRootObject();
            my $aShops = $System->child('Shops')->findChildren();

            foreach my $ShopID (@{$aShops}) {
                my $Shop = LoadObject($ShopID);
                my $ShopAlias = $Shop->alias;
                my $TaxMatrix = $Shop->get('TaxMatrix');

                next unless defined $TaxMatrix;

                my $TaxMatrixAlias = $TaxMatrix->alias;
                if ($TaxMatrixAlias eq 'TaxMatrixSwitzerland') {
                    print("$ShopAlias: Swiss taxes found! Fixing region countries...\n") unless $NoOutput;
                    my $ObjectPath = '/Shops/'.$ShopAlias;
                    ImportObjects($ImportFile, $ObjectPath);
                    print("  ...$ImportFile imported!\n") unless $NoOutput;
                } else {
                    print("$ShopAlias: $TaxMatrixAlias -> no Swiss taxes. Do nothing!\n") unless $NoOutput;
                }
            }

        }
    );

    eval { SynchronizeCache(); };
    Warning( GetError() ) if ExistsError();
}

sub usage {
    print <<END_USAGE;
Description:
    Fixes the Liechtenstein tax exemption for Swiss shops
    (Liechtenstein should be treated as Switzerland).

Usage:
    perl $0 [options] flags objects

options:
    -passwd     database user password                        ()
    -storename  name of store                                 required
    -importfile filepath to XML file to import                required
flags:
    -help       show the command line options
    -nooutput   do not print info to stdout

Example:
    perl $0 -storename Store -importfile fixSwitzerlandLiechtensteinTaxExemption.xml
END_USAGE
    exit 2;
}

RunScript(
    'Sub' => \&Main,
);

1;
