﻿#========================================================================================
# §package      DE_EPAGES::ShopNewsletter::API::TLE::NewsletterHandler
# §state        public
#----------------------------------------------------------------------------------------
# §description  Implements TLE functions for shop newsletters.
#========================================================================================
package DE_EPAGES::ShopNewsletter::API::TLE::NewsletterHandler;

use utf8;
use strict;

use DE_EPAGES::ShopNewsletter::DAL::Newsletter qw ( DAL_GetIsSubscribed );
use DE_EPAGES::Core::API::Error qw ( GetError ExistsError );
use DE_EPAGES::Core::API::Log qw ( LogError );

#========================================================================================
# §function     RegisterHandlerProc
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $RegisterHandlerProc( $hParams );
#----------------------------------------------------------------------------------------
# §description  this function registers this attribute handle on the processor via hook
#               function on hook TLEProcessorRegistration
#----------------------------------------------------------------------------------------
# §input        $hParams | hook arguments keys :
#               <ul>
#                   <li>Processor - tle processor - object
#               </ul> | ref.hash
#========================================================================================
sub RegisterHandlerProc {
    my ($Params) = @_;

    __PACKAGE__->new()->register( $Params->{'Processor'} );
    return;
}

#========================================================================================
# §function     new
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Handler = new DE_EPAGES::ShopNewsletter::API::TLE::NewsletterHandler;
# §example      my $PrefixHandler  = new DE_EPAGES::Product::API::TLE::PagerHandler;
#----------------------------------------------------------------------------------------
# §description  create a new handler
#----------------------------------------------------------------------------------------
# §return       $Handler | blessed object | object
#========================================================================================
sub new { bless {}, shift; }

#========================================================================================
# §function     register
# §state        private
#----------------------------------------------------------------------------------------
# §syntax       $Handler->register( $Processor );
#----------------------------------------------------------------------------------------
# §description  this function register this handle on the processor
#----------------------------------------------------------------------------------------
# §input        $Processor | tle processor | object
#========================================================================================
sub register {
    my $self = shift;
    my ($Processor) = @_;

    $Processor->registerHandler('FunctionHandler', $self, 'IsSubscribed');
    return;
}

#========================================================================================
# §function     IsSubscribed
# §state        public
#----------------------------------------------------------------------------------------
# §syntax       #IsSubscribed(#NewsletterID, #EMail)
# §example      #IF(#IsSubscribed(42217, 'ozscheyge@epages.com')
#                 Yo, subscribed!
#               #ENDIF
#----------------------------------------------------------------------------------------
# §description  Checks whether an email address is subscribed (and confirmed) to a newsletter campaign.
#----------------------------------------------------------------------------------------
# §input        #NewsletterID | ID of the newsletter campaign | int
# §input        #EMail | Email address to check | string
# §return       1 if the email address is subscribed, 0 if not | int/bool
#========================================================================================
sub IsSubscribed {
    my $self = shift;
    my ($Processor, $aParams) = @_;

    my $Result = eval {
        my $NewsletterCampaignID = $aParams->[0];
        my $EmailAddress = $aParams->[1];

        return DAL_GetIsSubscribed( $NewsletterCampaignID, $EmailAddress );
    };
    if ( ExistsError() ) {
        my $Error = GetError();
        LogError( $Error );
        return 0;
    }

    return $Result;
}

1;
